/*
 * Module ID: prime.h
 *
 * Auteur   : Olivier Langlois <olanglois@sympatico.ca>
 * Date     : 09 octobre 1997
 *
 *                       The First 1,000 Primes
 *                        (the 1,000th is 7919)
 *  For more information on primes see http://www.utm.edu/research/primes
 */

#ifndef _PRIME_H_
#define _PRIME_H_

extern unsigned primeTab[];

/******************************************************************************
 *
 * Nom       : modexpo
 *
 * Utilite   : Calcule a^b MOD m pour b >= 0
 *
 * Note      : Peut produire un overflow si la plateforme n'est pas x86.
 *
 ****************************************************************************/
unsigned modexpo( unsigned a, unsigned b, unsigned m );

/******************************************************************************
 *
 * Nom       : b_SPRP
 *
 * Utilite   : Strong Probable Prime test a la base b
 *
 * Note      : Peut produire un overflow si la plateforme n'est pas x86.
 *
 ****************************************************************************/
unsigned char b_SPRP( unsigned n, unsigned b );

/******************************************************************************
 *
 * Nom       : trialDivision
 *
 * Utilite   : Effectue le test trial division sur n sur les maxIdx (must be < 1000)
 *             premiers nombres premiers de la table des nombres premiers.
 *
 * Valeur de retour : true si n passe le test.
 *
 ****************************************************************************/
unsigned char trialDivision( unsigned n, int maxIdx );

/******************************************************************************
 *
 * Nom       : findUpperPrime
 *
 * Utilite   : Trouve le premier nombre premier > que n
 *
 ****************************************************************************/
unsigned findUpperPrime( unsigned n );

#endif /* _PRIME_H_ */
