#include <stdio.h>
#include <stdint.h>
#include <ghttp.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include <opctime.h>
#include <opcxml.h>
#include "dime.h"

ghttp_request *req;

double smin=1E+10, smax=0, smid=0;
double rmin=1E+10, rmax=0, rmid=0;
unsigned long iters=0;

int mainloop() {
    FILE *thefile;
    xmlDocPtr doc;
    xmlNodePtr node;
    double duration;
    struct timeval tm1, tm2;
    double time1;
    const char *rcvtime, *replytime;
    struct OPCTimeT otime1, otime2;
    struct OPCDurationT oduration;
    unsigned long length;
    ghttp_status req_status;

    ghttp_set_uri(req,"http://ipepc47.ka.fzk.de/opc/hell.opc");
    ghttp_set_type(req, ghttp_type_post);
    
//    char request[256]="<GetStatus xmlns=\"http://opcfoundation.org/webservices/XMLDA/1.0/\"/>";
    char request[256]="<Read xmlns=\"http://opcfoundation.org/webservices/XMLDA/1.0/\"><ItemList ItemPath=\"/Examples/random2\"><Item ItemName=\"complex\"/></ItemList></Read>";
    ghttp_set_body(req,request,strlen(request));
/*    ghttp_set_sync(req, ghttp_async);
    ghttp_set_chunksize(req,65535);*/
    ghttp_prepare(req);


    
    thefile = fopen("output.xml", "w");
    dime = 0;

    gettimeofday(&tm1,NULL);
    ghttp_process(req);
    writefunc(ghttp_get_body(req),1,ghttp_get_body_len(req),thefile);
    gettimeofday(&tm2,NULL);
    time1 = 1.*((tm2.tv_sec - tm1.tv_sec)*1000000 + tm2.tv_usec - tm1.tv_usec)/1000;
    
/*    
    do {
	req_status = ghttp_process(req);
	if (req_status == ghttp_error) return;
	length = ghttp_get_body_len(req);
	if (length) {
	    ghttp_flush_response_buffer(req);
	    writefunc(ghttp_get_body(req),1,length,thefile);
	}
    } while (req_status == ghttp_not_done);
*/
    fclose(thefile);

    ghttp_clean(req);

    if (dime<-1) return;
    
    printf("\n");
    printf("Request: %.3lfms, ", time1);

    
    doc = xmlReadFile("output.xml", NULL, 0);
    node=xmlDocGetRootElement(doc)->children;
    rcvtime=opcXmlGetPropString(node,"RcvTime",NULL);
    replytime=opcXmlGetPropString(node,"ReplyTime",NULL);
    opcTimeSetDateTime(&otime1, rcvtime);
    opcTimeSetDateTime(&otime2, replytime);
    opcTimeCalcDuration(&oduration, &otime1, &otime2);
    xmlFreeDoc(doc);
    duration = 1000.*oduration.s + 1.*oduration.us/1000;
    printf("Internal: %.3lfms\n", duration);
    
    if (duration<smin) smin=duration;
    if (duration>smax) smax=duration;
    smid = (smid*iters + duration)/(iters+1);

    if (time1<rmin) rmin=time1;
    if (time1>rmax) rmax=time1;
    rmid = (rmid*iters + time1)/(iters+1);
    
    printf("Internal: %3.3lf(%.3lf,%.3lf)ms, iter: %lu\n",smid,smin,smax,iters);
    printf("Request : %3.3lf(%.3lf,%.3lf)ms, iter: %lu\n",rmid,rmin,rmax,iters);
    iters++;
    printf("\n");
}

int main(int argc, char *argv[]) {
    unsigned long iters = 1;
    

    req = ghttp_request_new();
    
    if (argc>=2) {
	iters = atol(argv[1]);
	if (!iters) iters=-1;
    }
    
    for (;iters>0;iters--) {
	mainloop();
//	sleep(1);
    }
	
    ghttp_request_destroy(req);
}
