#include <opcerr.h>
#include <opcdebug.h>
#include <opcassert.h>

#include <opcallocator.h>

OPCAllocatorContext opcAllocatorCreateContext(OPCAllocator allocator, OPCMallocContext mctx) {
    OPCAllocatorContext res;
    assert(allocator);

    if (allocator->CreateContext) {
	res=allocator->CreateContext(mctx);
	memcpy(&res->allocator, allocator, sizeof(struct OPCAllocatorT));
/*	res->allocator = allocator;*/
	res->malloc = mctx;
    } else 
	res=NULL;
	
    opcDebugServer(OPC_DEBUG_DEBUG, "opcAllocator(%p): CreateContext: %p", res, allocator);    

    return res;
}


int opcAllocatorInitContext(OPCAllocatorContext ctx, OPCAllocatorOptions options) {
/*    OPCAllocator allocator; */
    
    opcDebugServer(OPC_DEBUG_DEBUG, "opcAllocator(%p): InitContext", ctx);    

    assert(ctx);
/*    allocator = ctx->allocator;
    assert(allocator);*/
    
    if ((!ctx->allocator.allocate)||(!ctx->allocator.free)||
	(!ctx->allocator.accept)||(!ctx->allocator.accept_chunk)||
	(!ctx->allocator.cancel)||(!ctx->allocator.cancel_chunk)||
	(!ctx->allocator.call)||(!ctx->allocator.sz)||
	(!ctx->allocator.get)||(!ctx->allocator.get_next))
	return ERR_OPC_ALLOCATOR_INCOMPLETE;

    if (ctx->allocator.InitContext) return ctx->allocator.InitContext(ctx,options);
    return 0;
}

void opcAllocatorFreeContext(OPCAllocatorContext ctx) {
/*    OPCAllocator allocator;*/

    opcDebugServer(OPC_DEBUG_DEBUG, "opcAllocator(%p): FreeContext", ctx);    

    assert(ctx);

/*   
    allocator = ctx->allocator;
    assert(allocator);
*/

    if (ctx->allocator.FreeContext) ctx->allocator.FreeContext(ctx);
}

void opcAllocatorDestroyContext(OPCAllocatorContext ctx) {
/*    OPCAllocator allocator;*/

    opcDebugServer(OPC_DEBUG_DEBUG, "opcAllocator(%p): DestroyContext", ctx);    

    assert(ctx);
/*    
    allocator = ctx->allocator;
    assert(allocator);
*/
    
    if (ctx->allocator.DestroyContext) ctx->allocator.DestroyContext(ctx);
    else {
	if (ctx->allocator.FreeContext) ctx->allocator.FreeContext(ctx);
	opcFree(ctx->malloc, ctx);
    }
}


OPCAllocatorOptions opcAllocatorGetDefaultOptions(OPCAllocator ctx) {
    int err;
    OPCAllocatorOptions options;
    
    if (ctx->CreateOptions) return ctx->CreateOptions();
    
    options = (OPCAllocatorOptions)malloc(sizeof(struct OPCAllocatorOptionsT));
    if (!options) return options;

    err = opcAllocatorInitDefaultOptions(ctx, options);
    if (err) {
	opcAllocatorDestroyOptions(ctx, options);
	return NULL;
    }
    return options;
}

int opcAllocatorInitDefaultOptions(OPCAllocator ctx, OPCAllocatorOptions options) {
    if (ctx->InitOptions) return ctx->InitOptions(options);
    memset(options, 0, sizeof(struct OPCAllocatorOptionsT));
    return 0;
}

void opcAllocatorDestroyOptions(OPCAllocator ctx, OPCAllocatorOptions options) {
    if (ctx->DestroyOptions) ctx->DestroyOptions(options);
    else free(options);
}

OPCAllocation opcAllocatorSimpleAllocate(OPCAllocatorContext allocator, unsigned long size) {
    OPCAllocation allocation;
    if (!allocator) return NULL;
    if (allocator->allocator.simple) {
	return allocator->allocator.simple(allocator,size);
    } else {
	if (allocator->allocator.start) allocation = allocator->allocator.start(allocator,size);
	else allocation = NULL;
	if (allocator->allocator.simple_chunk) allocator->allocator.simple_chunk(allocator,allocation,size);
	else {
	    allocator->allocator.allocate(allocator,allocation,size,0);
	    allocator->allocator.accept_chunk(allocator,allocation,NULL);
	}
	return allocator->allocator.accept(allocator,allocation);	
    }    
}
