#ifndef _OPC_ALLOCATOR_H
#define _OPC_ALLOCATOR_H

#include <opcmalloc.h>
#include <opclib_types.h>

typedef struct OPCAllocatorOptionsT *OPCAllocatorOptions;
typedef struct OPCAllocatorContextT *OPCAllocatorContext;
typedef const struct OPCAllocatorT *OPCAllocator;
typedef void *OPCAllocation;

typedef void *(*OPCAllocatorCreateFunction)(OPCMallocContext mctx);
typedef int (*OPCAllocatorInitFunction)(OPCAllocatorContext ctx, OPCAllocatorOptions options);
typedef void (*OPCAllocatorFreeFunction)(OPCAllocatorContext ctx);
typedef void (*OPCAllocatorDestroyFunction)(OPCAllocatorContext ctx);

typedef OPCAllocatorOptions (*OPCAllocatorGetDefaultOptions)();
typedef int (*OPCAllocatorInitDefaultOptions)(OPCAllocatorOptions options);
typedef void (*OPCAllocatorDestroyOptions)(OPCAllocatorOptions options);

typedef OPCAllocation (*OPCAllocatorStartAllocationFunction)(OPCAllocatorContext ctx, unsigned long preallocate);
typedef void *(*OPCAllocatorAllocateMemFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned long size, unsigned char alignment);
typedef void (*OPCAllocatorFreeMemFunction)(OPCAllocatorContext ctx, OPCAllocation allocation);
typedef void *(*OPCAllocatorAcceptChunkFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned long *size);
typedef OPCAllocation (*OPCAllocatorAcceptFunction)(OPCAllocatorContext ctx, OPCAllocation allocation);
typedef void (*OPCAllocatorCancelFunction)(OPCAllocatorContext ctx, OPCAllocation allocation);
typedef void (*OPCAllocatorCancelChunkFunction)(OPCAllocatorContext ctx, OPCAllocation allocation);
typedef void (*OPCAllocatorTrimFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned long size);
typedef unsigned long (*OPCAllocatorReserveFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned long size);

typedef void *(*OPCAllocatorGetFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned long *size);
typedef void *(*OPCAllocatorGetNextFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, void *current, unsigned long *size);
typedef int (*OPCAllocatorCallFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, OPCSendFunction func, void *attr, unsigned char align);
typedef unsigned long (*OPCAllocatorSizeofFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned char align);

typedef void *(*OPCAllocatorAllocateSimpleChunkFunction)(OPCAllocatorContext ctx, OPCAllocation allocation, unsigned long size);
typedef OPCAllocation (*OPCAllocatorAllocateSimpleFunction)(OPCAllocatorContext ctx, unsigned long size);

typedef void (*OPCAllocatorMantainFunction)(OPCAllocatorContext ctx);

struct OPCAllocatorT {
    char *name;
    
    OPCAllocatorCreateFunction CreateContext;
    OPCAllocatorInitFunction InitContext;
    OPCAllocatorFreeFunction FreeContext;
    OPCAllocatorDestroyFunction DestroyContext;

    OPCAllocatorGetDefaultOptions CreateOptions;
    OPCAllocatorInitDefaultOptions InitOptions;
    OPCAllocatorDestroyOptions DestroyOptions;
    
    OPCAllocatorMantainFunction mantain;
    
    OPCAllocatorStartAllocationFunction start;
    OPCAllocatorAllocateMemFunction allocate;
    OPCAllocatorFreeMemFunction free;
    OPCAllocatorAcceptChunkFunction accept_chunk;
    OPCAllocatorCancelChunkFunction cancel_chunk;
    OPCAllocatorAcceptFunction accept;
    OPCAllocatorCancelFunction cancel;

    OPCAllocatorAllocateSimpleChunkFunction simple_chunk;
    OPCAllocatorAllocateSimpleFunction simple;

    OPCAllocatorReserveFunction reserve;
    OPCAllocatorTrimFunction trim;

    OPCAllocatorGetFunction get;
    OPCAllocatorGetNextFunction get_next;
    OPCAllocatorCallFunction call;
    OPCAllocatorSizeofFunction sz;
};

struct OPCAllocatorContextT {
    struct OPCAllocatorT allocator;
    OPCMallocContext malloc;
};

struct OPCAllocatorOptionsT {
    void *pool;
    unsigned long minimal_size;
    unsigned long maximal_size;
    unsigned long fixed_allocation_size;
};

OPCAllocatorOptions opcAllocatorGetDefaultOptions(OPCAllocator ctx);
int opcAllocatorInitDefaultOptions(OPCAllocator ctx, OPCAllocatorOptions options);
void opcAllocatorDestroyOptions(OPCAllocator ctx, OPCAllocatorOptions options);


OPCAllocatorContext opcAllocatorCreateContext(OPCAllocator allocator, OPCMallocContext mctx);
int opcAllocatorInitContext(OPCAllocatorContext ctx, OPCAllocatorOptions options);
void opcAllocatorFreeContext(OPCAllocatorContext ctx);
void opcAllocatorDestroyContext(OPCAllocatorContext ctx);


#define opcAllocatorStart(ctx, size) \
    ((ctx&&ctx->allocator.start)?ctx->allocator.start(ctx,size):NULL)
#define opcAllocatorAllocate(ctx, allocation, size, alignment) \
    ((ctx)?ctx->allocator.allocate(ctx,allocation,size,alignment):NULL)
#define opcAllocatorFree(ctx, ptr) \
    ((ctx)?ctx->allocator.free(ctx,ptr):0)
#define opcAllocatorAcceptChunk(ctx,allocation,size) \
    ((ctx)?ctx->allocator.accept_chunk(ctx,allocation,size):NULL)
#define opcAllocatorAccept(ctx,allocation) \
    ((ctx)?ctx->allocator.accept(ctx,allocation):NULL)
#define opcAllocatorCancelChunk(ctx,allocation) \
    ((ctx)?ctx->allocator.cancel_chunk(ctx,allocation):0)
#define opcAllocatorCancel(ctx,allocation) \
    ((ctx)?ctx->allocator.cancel(ctx,allocation):0)
#define opcAllocatorCall(ctx, ptr, func, attr, align) \
    ((ctx)?ctx->allocator.call(ctx, ptr, func, attr, align):ERR_OPC_PARAM)
#define opcAllocatorSizeof(ctx, ptr, align) \
    ((ctx)?ctx->allocator.sz(ctx, ptr, align):0)
#define opcAllocatorTrim(ctx, allocation, size) \
    (((ctx)&&(ctx->allocator.trim))?ctx->allocator.trim(ctx, allocation, size):0)
#define opcAllocatorReserve(ctx, allocation, size) \
    (((ctx)&&(ctx->allocator.reserve))?ctx->allocator.reserve(ctx, allocation, size):0)
#define opcAllocatorMantain(ctx) \
    (((ctx)&&(ctx->allocator.mantain))?ctx->allocator.mantain(ctx):0)
#define opcAllocatorSimpleChunk(ctx,allocation,size) \
    (ctx?((ctx->allocator.simple_chunk)?ctx->allocator.simple_chunk(ctx,allocation,size):((ctx->allocator.allocate(ctx,allocation,size,0))?ctx->allocator.accept_chunk(ctx,allocation,NULL):NULL)):NULL)
#define opcAllocatorGet(ctx,allocation,size) \
    (ctx?ctx->allocator.get(ctx,allocation,size):NULL)
#define opcAllocatorGetNext(ctx,allocation,current,size) \
    (ctx?ctx->allocator.get_next(ctx,allocation,current,size)

OPCAllocation opcAllocatorSimpleAllocate(OPCAllocatorContext allocator, unsigned long size);
    
#endif /* _OPC_ALLOCATOR_H */
