#define _OPC_ALLOCATOR_SIMPLE_C
#include <opcerr.h>
#include <opcdebug.h>
#include <opcassert.h>

#include <opcbinaryconfiguration.h>

#include <opcallocatorsimple.h>

#define HEADER_SIZE (sizeof(void*) + sizeof(unsigned long))
#define OUTBUF_SIZE 4096
#define ALIGN(pos, align) \
    if ((align)&&(pos&(align-1))) pos = (pos&~(align-1))+align;

static unsigned long header_size = 0;


OPCAllocator opcAllocatorSimple() {
    return &opc_allocator_simple;
}


OPCAllocatorSimpleContext opcAllocatorSimpleCreateContext(OPCMallocContext mctx) {
    OPCAllocatorSimpleContext ctx;

    ctx = (OPCAllocatorSimpleContext)opcMalloc(mctx, sizeof(struct OPCAllocatorSimpleContextT));
    if (ctx) {
	memcpy(&ctx->base.allocator, &opc_allocator_simple, sizeof(struct OPCAllocatorT));
	ctx->base.malloc = mctx;
	
	if (!header_size) {
	    header_size = HEADER_SIZE;
	    ALIGN(header_size, opc_binary_configuration->align_allocation);
	}
    }
    return ctx;
}

int opcAllocatorSimpleInitContext(OPCAllocatorSimpleContext ctx, const void *options) {
    return 0;
}

void opcAllocatorSimpleFreeContext(OPCAllocatorSimpleContext ctx) {
    assert(ctx);
}

void opcAllocatorSimpleDestroyContext(OPCAllocatorSimpleContext ctx) {
    opcAllocatorSimpleFreeContext(ctx);
    opcFree(ctx->base.malloc, ctx);
}

OPCAllocation opcAllocatorSimpleStart(OPCAllocatorSimpleContext ctx, unsigned long preallocate) {
    OPCAllocationSimple allocation;

    allocation = (OPCAllocationSimple)opcMalloc(ctx->base.malloc, sizeof(struct OPCAllocationSimpleT));
    if (!allocation) return NULL;
    
    allocation->buffer = NULL;
    allocation->prebuffer = NULL;
    allocation->allocation = NULL;
    return (OPCAllocation)allocation;
    
}

void *opcAllocatorSimpleAllocateMemory(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, unsigned long size, unsigned char alignment) {
    char *buffer;
    unsigned long new_size, buff_size;

    assert(ctx);
    assert(allocation);
    assert(size>0);

    if (((OPCAllocationSimple)allocation)->buffer) {
	new_size = ((OPCAllocationSimple)allocation)->current_size;
	ALIGN(new_size,alignment);
	new_size+=size + header_size;

	if (new_size>((OPCAllocationSimple)allocation)->buffer_size) {
	    for (buff_size=((OPCAllocationSimple)allocation)->buffer_size*2;buff_size<new_size;buff_size*=2);
	    buffer=opcRealloc(ctx->base.malloc,((OPCAllocationSimple)allocation)->buffer,buff_size);
	    if (buffer) {
		((OPCAllocationSimple)allocation)->buffer = buffer;
		((OPCAllocationSimple)allocation)->buffer_size = buff_size;
	    } else return NULL;
	}
	buff_size = ((OPCAllocationSimple)allocation)->current_size;
	((OPCAllocationSimple)allocation)->current_size = new_size - header_size;
	return buffer+buff_size;
    } else {
	new_size = size + header_size;
	ALIGN(new_size, opc_binary_configuration->align_allocation);
	((OPCAllocationSimple)allocation)->buffer=opcMalloc(ctx->base.malloc, new_size);
	if (((OPCAllocationSimple)allocation)->buffer) {
	    ((OPCAllocationSimple)allocation)->current_size = size;
	    ((OPCAllocationSimple)allocation)->buffer_size = new_size;
	}
	return ((OPCAllocationSimple)allocation)->buffer + header_size;
    }
}

void *opcAllocatorSimpleAcceptChunk(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, unsigned long *size) {
    char *buffer;

    assert(ctx);
    assert(allocation);
    
    buffer=((OPCAllocationSimple)allocation)->buffer;

    if (buffer) {
	if (((OPCAllocationSimple)allocation)->prebuffer) *((void**)(((OPCAllocationSimple)allocation)->prebuffer+sizeof(unsigned long)))=buffer;
	else ((OPCAllocationSimple)allocation)->allocation = (OPCAllocation)buffer;
	*((unsigned long*)(buffer))=((OPCAllocationSimple)allocation)->current_size;
	*((void**)(buffer+sizeof(unsigned long)))=NULL;
	((OPCAllocationSimple)allocation)->prebuffer = buffer;
	((OPCAllocationSimple)allocation)->buffer = NULL;
	if (size) *size = ((OPCAllocationSimple)allocation)->current_size;
	return buffer + header_size;
    }
    return NULL;
}

OPCAllocation opcAllocatorSimpleAccept(OPCAllocatorSimpleContext ctx, OPCAllocation allocation) {
    OPCAllocation res;

    assert(ctx);
    assert(allocation);

    if (((OPCAllocationSimple)allocation)->buffer) opcAllocatorSimpleAcceptChunk(ctx, allocation, NULL);
    res = (OPCAllocation)((OPCAllocationSimple)allocation)->allocation;
    opcFree(ctx->base.malloc,allocation);
    return res;
}


void opcAllocatorSimpleCancelChunk(OPCAllocatorSimpleContext ctx, OPCAllocation allocation) {
    assert(ctx);
    assert(allocation);
    
    if (((OPCAllocationSimple)allocation)->buffer) {
	opcFree(ctx->base.malloc, ((OPCAllocationSimple)allocation)->buffer);
	((OPCAllocationSimple)allocation)->buffer = NULL;
    }
}

void opcAllocatorSimpleCancel(OPCAllocatorSimpleContext ctx, OPCAllocation allocation) {
    char *ptr, *next;

    assert(ctx);
    assert(allocation);
    
    opcAllocatorSimpleCancelChunk(ctx, allocation);
    if (((OPCAllocationSimple)allocation)->allocation) {
	for (ptr = (char*)((OPCAllocationSimple)allocation)->allocation;ptr;ptr=next) {
	    next = *((void**)(ptr+sizeof(unsigned long)));
	    opcFree(ctx->base.malloc, ptr);
	}
    }
    opcFree(ctx->base.malloc, allocation);
}

void opcAllocatorSimpleFree(OPCAllocatorSimpleContext ctx, OPCAllocation allocation) {
    char *ptr, *next;
    
    assert(ctx);
    assert(allocation);

    for (ptr = (char*)allocation;ptr;ptr=next) {
        next = *((void**)(ptr+sizeof(unsigned long)));
        opcFree(ctx->base.malloc, ptr);
    }
}

void *opcAllocatorSimpleAllocateChunk(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, unsigned long size) {
    unsigned long new_size;
    char *buffer;

    assert(ctx);
    assert(!((OPCAllocationSimple)allocation)->buffer);

    new_size = size + header_size;
    ALIGN(new_size, opc_binary_configuration->align_allocation);
    buffer=opcMalloc(ctx->base.malloc, new_size);
    if (!buffer) return NULL;
    
    if (((OPCAllocationSimple)allocation)->prebuffer) 
	*((void**)(((OPCAllocationSimple)allocation)->prebuffer+sizeof(unsigned long)))=buffer;
    else 
	((OPCAllocationSimple)allocation)->allocation = (OPCAllocation)buffer;
    *((unsigned long*)(buffer))=size;
    *((void**)(buffer+sizeof(unsigned long)))=NULL;
    ((OPCAllocationSimple)allocation)->prebuffer = buffer;
    
    return buffer + header_size;
}

OPCAllocation opcAllocatorSimpleAllocateSimple(OPCAllocatorSimpleContext ctx, unsigned long size) {
    unsigned long new_size;
    char *buffer;
    
    assert(ctx);

    new_size = size + header_size;
    ALIGN(new_size, opc_binary_configuration->align_allocation);
    buffer=opcMalloc(ctx->base.malloc, new_size);
    if (!buffer) return NULL;
    *((unsigned long*)(buffer))=size;
    *((void**)(buffer+sizeof(unsigned long)))=NULL;
    return (OPCAllocation)buffer;
}

void* opcAllocatorSimpleGet(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, unsigned long *size) {
    char *buffer;
    
    assert(ctx);
    assert(allocation);
    
    buffer = (char*)allocation;
    if (size) *size = *((unsigned long*)buffer);
    return buffer+header_size;
}

void* opcAllocatorSimpleGetNext(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, void *current, unsigned long *size) {
    char *buffer;

    assert(ctx);
    assert(allocation);
    assert(current);
    
    buffer = *((char**)((char*)current-header_size+sizeof(unsigned long)));
    if (!buffer) return buffer;
    
    if (size) *size = *((unsigned long*)buffer);
    return buffer+header_size;
}

int opcAllocatorSimpleCall(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, OPCSendFunction func, void *attr, unsigned char align) {
    int err = 0;
    unsigned long size;
    unsigned long pos = 0;
    char *ptr, *next;
    char *buffer;
    
    assert(ctx);
    assert(allocation);
    
    buffer = opcMalloc(ctx->base.malloc, OUTBUF_SIZE);
    if (!buffer) return ERR_OPC_MEM;

    for (ptr = (char*)allocation;ptr;ptr=next) {
	size = *((unsigned long*)ptr);
        next = *((void**)(ptr+sizeof(unsigned long)));
	if ((next)&&(align))
	    ALIGN(size,opc_binary_configuration->align_allocation);

	if (pos+size>OUTBUF_SIZE) {
	    if (pos) {
	        err=func(attr,buffer,pos);
		if (err) break;
	    }

	    if (size>OUTBUF_SIZE/2) {
		err=func(attr,ptr+header_size,size);
		if (err) break;
		pos = 0;
	    } else {
		memcpy(buffer,ptr+header_size,size);
		pos = size;
	    }
	} else {
	    memcpy(buffer+pos,ptr+header_size,size);
	    pos += size;
	}
    }
    if ((!err)&&(pos)) err=func(attr,buffer,pos);
    
    free(buffer);
    return err;
}

unsigned long opcAllocatorSimpleSizeof(OPCAllocatorSimpleContext ctx, OPCAllocation allocation, unsigned char align) {
    unsigned long res=0, size;
    char *ptr, *next;
    
    assert(ctx);
    assert(allocation);

    for (ptr = (char*)allocation;ptr;ptr=next) {
	size = *((unsigned long*)ptr);
        next = *((void**)(ptr+sizeof(unsigned long)));
	if ((next)&&(align))
	    ALIGN(size,opc_binary_configuration->align_allocation);
	res += size;
    }
    return res;
}
