#include <assert.h>

#include <opcbinaryconfiguration.h>
#include <opcbuffer.h>

#define HEADER_SIZE (sizeof(void*)+sizeof(unsigned long))
#define OPC_BUFFER_BREAK 0xFFFFFFFF
#define ALIGN(pos,alignment) \
    if ((alignment)&&(pos&(alignment-1))) pos = (pos&~(alignment-1))+alignment;


int opcBufferInitContext(OPCBufferContext ctx) {
    opcBufferRestoreContext(ctx);
    return 0;
}

void opcBufferFreeContext(OPCBufferContext ctx) {
    assert(ctx);
}

void opcBufferRestoreContext(OPCBufferContext ctx) {
    assert(ctx);
    
    ctx->buffer_start = 0;
    ctx->buffer_end = 0;
    ctx->buffer_pos = 0;

    ctx->last_allocation_end = (unsigned long)-1;
    ctx->block_start = (unsigned long)-1;
    
    ctx->next = NULL;
    ctx->preptr = NULL;
}

/*
static void opcBufferCancelFailed(OPCBufferContext ctx) {
    OPCBufferContext next;
    
    assert(ctx);
    
    if (ctx->last_allocation_end==(unsigned long)-1) {
	opcBufferPoolReturnBuffer(ctx->pool, ctx);
    } else {
	ctx->buffer_end = ctx->last_allocation_end;
	ctx->buffer_pos = ctx->buffer_end;
    }
}
*/

unsigned long opcBufferReserve(OPCBufferContext ctx, unsigned long softmax, unsigned long size) {
    OPCBufferContext nctx;
    unsigned char *nbuffer;
    unsigned long esize, ssize;
    unsigned long new_size;
    unsigned long pos;

    assert(ctx);

//    printf("%lu: %lu %lu %lu (%lu)\n",ctx->buffer_size, ctx->buffer_start, ctx->buffer_end, ctx->buffer_pos, size);
    
    if (ctx->buffer_end == ctx->buffer_pos) {
	if (ctx->buffer_start == ctx->buffer_end) {
	    ctx->buffer_start = 0;
	    ctx->buffer_end = 0;
	    if (size > ctx->buffer_size) {
		if ((!softmax)||(ctx->buffer_size<softmax))
		    for (new_size = 2*ctx->buffer_size; new_size < size; new_size*=2);
		else 
		    for (new_size = ctx->buffer_size; new_size < size; new_size*=2);
		
		if (opcBufferPoolReplaceBuffer(ctx->pool, ctx, new_size)) return 0;
		return new_size;
	    }
	    return ctx->buffer_size;
	} else {
	    if (ctx->buffer_end < ctx->buffer_start) {
		ssize = ctx->buffer_start - ctx->buffer_end - 1;
		if (size > ssize) {
		    if ((!softmax)||(ctx->buffer_size<softmax))
			for (new_size = 2*ctx->buffer_size; new_size < size; new_size*=2);
		    else
			for (new_size = ctx->buffer_size; new_size < size; new_size*=2);
		    nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
		    if (!nctx) return 0;
		    if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
		    else nctx->preptr = ctx->preptr;
		    ctx->next = nctx;
		    return new_size;
		}
		return ssize;
	    } else {
    		esize = ctx->buffer_size - ctx->buffer_end - 1;
		ssize = ctx->buffer_start;
//		printf("%lu %lu\n", ssize, esize);
		if (esize<ssize) {
		    if (size > ssize) {
			if ((!softmax)||(ctx->buffer_size<softmax))
			    for (new_size = 2*ctx->buffer_size; new_size < size; new_size*=2);
			else
			    for (new_size = ctx->buffer_size; new_size < size; new_size*=2);
			nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
			if (!nctx) return 0;
			if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
			else nctx->preptr = ctx->preptr;
			ctx->next = nctx;
			return new_size;
		    }
		    if (esize>=sizeof(unsigned long)) 
			*(unsigned long*)(ctx->buffer + ctx->buffer_end)=OPC_BUFFER_BREAK;
		    if (ctx->buffer_end==ctx->last_allocation_end) ctx->last_allocation_end=0;
		    ctx->buffer_end=0;
		    ctx->buffer_pos=0;
		    ctx->block_start=(unsigned long)-1;
		    return ssize;
		} else {
		    if (size > esize) {
			if ((!softmax)||(ctx->buffer_size<softmax))
			    for (new_size = 2*ctx->buffer_size; new_size < size; new_size*=2);
			else
			    for (new_size = ctx->buffer_size; new_size < size; new_size*=2);
			nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
			if (!nctx) return 0;
			if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
			else nctx->preptr = ctx->preptr;
			ctx->next = nctx;
			return new_size;
		    }
		    return esize;
		}
	    }
	}
    } else {
	pos = ctx->buffer_pos;
	esize = ((pos < ctx->buffer_start) ? ctx->buffer_start : ctx->buffer_size) - pos;
    
	if (esize < size ) {
	    ssize = ctx->buffer_pos - ctx->chunk_start;

	    if ((ctx->buffer_end >= ctx->buffer_start)&&(ctx->buffer_start>(ctx->chunk_alignment+ssize+size)*2)) {
		new_size = HEADER_SIZE;
		ALIGN(new_size, ctx->chunk_alignment);
		
		if (ctx->block_start != (unsigned long)-1) {
		    memcpy(ctx->buffer+new_size, ctx->buffer + ctx->chunk_start, ssize);
		    ctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
		    ctx->buffer_pos = new_size + ssize;
		} else {
		    memcpy(ctx->buffer+new_size,ctx->buffer + ctx->chunk_start + HEADER_SIZE, ssize - HEADER_SIZE);
		    if (ctx->last_allocation_end != ctx->buffer_end) ctx->last_allocation_end = 0;
		    ctx->buffer_pos = new_size + ssize - HEADER_SIZE;
		}
		if (ctx->buffer_start == ctx->buffer_end) ctx->buffer_start = 0;
		else *(unsigned long*)(ctx->buffer + ctx->buffer_end)=OPC_BUFFER_BREAK;
		
		ctx->block_start = (unsigned long)-1;
		ctx->chunk_start = new_size - HEADER_SIZE;
		ctx->buffer_end = 0;
		
		return (((ctx->buffer_start)?ctx->buffer_start : ctx->buffer_size) - ctx->buffer_pos);
	    }

	    if ((!softmax)||(ctx->buffer_size<softmax))
	        for (new_size = 2*ctx->buffer_size; new_size < (ssize+size); new_size*=2);
	    else
	        for (new_size = ctx->buffer_size; new_size < (ssize+size); new_size*=2);

	    if (ctx->buffer_start == ctx->buffer_end) {
		if (!opcBufferPoolResizeBuffer(ctx->pool, ctx, new_size)) {
		    return new_size-ssize;
		}
	    }
	    
	    nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
	    if (!nctx) return 0;
	    if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
	    else nctx->preptr = ctx->preptr;
	    ctx->next = nctx;

//	    printf("S: %lu %lu %lu\n",ctx->last_allocation_end, ctx->block_start, ctx->buffer_end);

	    if (ctx->block_start != (unsigned long)-1) {
		nbuffer=opcBufferFastAllocate(nctx, ssize, ctx->chunk_alignment, NULL);
		memcpy(nbuffer,ctx->buffer + ctx->chunk_start, ssize);
		nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
	    } else {
		nbuffer=opcBufferFastAllocate(nctx, ssize - HEADER_SIZE, ctx->chunk_alignment, NULL);
		memcpy(nbuffer,ctx->buffer + ctx->chunk_start + HEADER_SIZE, ssize - HEADER_SIZE);
		nctx->preptr = ctx->preptr;
	    }
	    ctx->buffer_pos = ctx->buffer_end;
	    
//	    printf("%p:%p. %lu %lu %lu\n",nctx, ctx, nctx->buffer_start, nctx->buffer_end, nctx->buffer_pos);
//	    printf("S: %lu\n",nctx->buffer_pos - nctx->chunk_start - HEADER_SIZE);

       	    return new_size-nctx->buffer_pos;
	}
	return esize;
    }
    return 0;
}

void opcBufferTrim(OPCBufferContext ctx, unsigned long size) {
    assert(ctx->buffer_end+size+HEADER_SIZE<ctx->buffer_pos);
    ctx->buffer_pos-=size;
}

unsigned char *opcBufferAllocate(OPCBufferContext ctx, unsigned long softmax, unsigned long size, unsigned char alignment) {
    OPCBufferContext nctx;
    unsigned char *buffer, *nbuffer;
    unsigned long esize, ssize;
    unsigned long allocation_size;
    unsigned long new_size;
    unsigned long pos;
    unsigned char first_chunk;
    
    assert(ctx);


    if (ctx->buffer_end == ctx->buffer_pos) {
	allocation_size = opc_binary_configuration->align_allocation + HEADER_SIZE + size; /* A bit more not a problem */

	if (ctx->buffer_start == ctx->buffer_end) {
	    ctx->buffer_start = 0;
	    ctx->buffer_end = 0;
	    if (allocation_size > ctx->buffer_size) {
		if ((!softmax)||(ctx->buffer_size<softmax))
		    for (new_size = 2*ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);
		else 
		    for (new_size = ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);
		if (opcBufferPoolReplaceBuffer(ctx->pool, ctx, new_size)) return NULL;
	    }
	    pos = 0;
	    first_chunk = 1;
	} else {
	    if (ctx->buffer_end < ctx->buffer_start) {
		esize = 0;
		ssize = ctx->buffer_start - ctx->buffer_end - 1;

		if (allocation_size > ssize) {
		    if ((!softmax)||(ctx->buffer_size<softmax))
			for (new_size = 2*ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);
		    else 
			for (new_size = ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);

		    nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
		    if (!nctx) return NULL;
		    if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
		    else nctx->preptr = ctx->preptr;
		    ctx->next = nctx;
		    return opcBufferFastAllocate(nctx, size, alignment, NULL);
		}
		pos = ctx->buffer_end;
		if (ctx->block_start==(unsigned long)-1) first_chunk=1;
		else first_chunk=0;
	    } else {
    		esize = ctx->buffer_size - ctx->buffer_end - 1;
		ssize = ctx->buffer_start;
		
		if (esize<ssize) {
		    if (allocation_size > ssize) {
			if ((!softmax)||(ctx->buffer_size<softmax))
			    for (new_size = 2*ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);
			else 
			    for (new_size = ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);

			nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
			if (!nctx) return NULL;
			if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
			else nctx->preptr = ctx->preptr;
			ctx->next = nctx;
			return opcBufferFastAllocate(nctx, size, alignment, NULL);
		    }
		    if (esize>=sizeof(unsigned long)) 
			*(unsigned long*)(ctx->buffer + ctx->buffer_end)=OPC_BUFFER_BREAK;
		    if (ctx->buffer_end==ctx->last_allocation_end) ctx->last_allocation_end=0;
		    ctx->buffer_end=0;
		    ctx->buffer_pos=0;
		    first_chunk = 1;
		    pos = 0;
		} else {
		    if (allocation_size > esize) {
			if ((!softmax)||(ctx->buffer_size<softmax))
			    for (new_size = 2*ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);
			else 
			    for (new_size = ctx->buffer_size; new_size < 2*allocation_size; new_size*=2);
		    		    
			nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
			if (!nctx) return NULL;
		        if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
			else nctx->preptr = ctx->preptr;
			ctx->next = nctx;
			return opcBufferFastAllocate(nctx, size, alignment, NULL);
		    }
		    if (ctx->block_start==(unsigned long)-1) first_chunk=1;
		    else first_chunk=0;
		    pos = ctx->buffer_end;
		}
	    }
	}
	buffer = ctx->buffer;
	
	if (first_chunk) {
	    alignment = opc_binary_configuration->align_allocation;
	    new_size = pos + HEADER_SIZE;
	} else {
	    if (!alignment) alignment = opc_binary_configuration->align_allocation;
	    new_size = pos;
	}
	
	if ((alignment)&&(new_size&(alignment-1))) new_size = (new_size&~(alignment-1))+alignment;
	ctx->buffer_pos = new_size + size;

	ctx->chunk_alignment = alignment;
	if (first_chunk) ctx->chunk_start = new_size - HEADER_SIZE;
	else ctx->chunk_start = new_size;

/*	
	if (ctx->chunk_start > pos)
	    memset(buffer+pos, OPC_BUFFER_ALIGN_BYTE, ctx->chunk_start - pos);
*/
	return buffer + new_size;
    } 
    
    buffer = ctx->buffer;
    
    
    pos = ctx->buffer_pos;
    esize = (pos < ctx->buffer_start) ? ctx->buffer_start : ctx->buffer_size;

    if ((alignment)&&(pos&(alignment-1)))
	pos = (pos&~(alignment-1))+alignment;
	
    if (esize - pos < size ) {
	ssize = ctx->buffer_pos - ctx->chunk_start;
	
	if ((!softmax)||(ctx->buffer_size<softmax))
	    for (new_size = 2*ctx->buffer_size; new_size < (ssize+size+alignment); new_size*=2);
	else
	    for (new_size = ctx->buffer_size; new_size < (ssize+size+alignment); new_size*=2);

	if (ctx->buffer_start == ctx->buffer_end) {
	    if (opcBufferPoolResizeBuffer(ctx->pool, ctx, new_size)) return NULL;
	} else {
	    nctx = opcBufferPoolGetBuffer(ctx->pool, new_size);
	    if (!nctx) return NULL;
	    if (ctx->block_start!=(unsigned long)-1) nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
	    else nctx->preptr = ctx->preptr;
	    ctx->next = nctx;

	    if (ctx->block_start != (unsigned long)-1) {
		nbuffer=opcBufferFastAllocate(nctx, ssize, ctx->chunk_alignment, NULL);
		memcpy(nbuffer,ctx->buffer + ctx->chunk_start, ssize);
		nctx->preptr=ctx->buffer+ctx->block_start+sizeof(unsigned long);
	    } else {
		nbuffer=opcBufferFastAllocate(nctx, ssize - HEADER_SIZE, ctx->chunk_alignment, NULL);
		memcpy(nbuffer,ctx->buffer + ctx->chunk_start + HEADER_SIZE, ssize - HEADER_SIZE);
		nctx->preptr = ctx->preptr;
	    }
	    ctx->buffer_pos = ctx->buffer_end;
	    
       	    return opcBufferFastAllocate(nctx, size, alignment, NULL);

/*
	    nbuffer=opcBufferFastAllocate(nctx, ssize - HEADER_SIZE, ctx->chunk_alignment, NULL);
	    if (!nbuffer) return nbuffer;
	
	    memcpy(nbuffer,buffer + ctx->chunk_start + HEADER_SIZE, ssize - HEADER_SIZE);
	    opcBufferCancel(ctx); 
	    
	    return opcBufferFastAllocate(nctx, size, alignment, NULL);*/
	}
    }

/* Isn't required!
    if (pos > ctx->buffer_pos)
	memset(buffer + ctx->buffer_pos, OPC_BUFFER_ALIGN_BYTE, pos - ctx->buffer_pos);
*/    
    ctx->buffer_pos = pos + size;

    return ctx->buffer + pos;
}

unsigned char *opcBufferFastAllocate(OPCBufferContext ctx, unsigned long size, unsigned char alignment, unsigned long *preallocated) {
    OPCBufferContext nctx;
    unsigned char *buffer, *nbuffer;
    unsigned long esize, ssize;
    unsigned long allocation_size;
    unsigned long new_size;
    unsigned long pos;
    unsigned char first_chunk=1;
    
    assert(ctx);
/*
    assert(preallocated);
    assert(*preallocated < alignment + HEADER_SIZE + size);
*/
//    printf("%p, %lu %lu %lu\n",ctx, ctx->buffer_start, ctx->buffer_end, ctx->buffer_pos);
    if (ctx->buffer_end == ctx->buffer_pos) {
	if (ctx->buffer_start != ctx->buffer_end) {
	    if (ctx->block_start==(unsigned long)-1) first_chunk=1;
	    else first_chunk=0;
	}
	pos = ctx->buffer_end;
	buffer = ctx->buffer;
	
	if (first_chunk) {
	    alignment = opc_binary_configuration->align_allocation;
	    new_size = pos + HEADER_SIZE;
	} else {
	    if (!alignment) alignment = opc_binary_configuration->align_allocation;
	    new_size = pos;
	}
	
	if ((alignment)&&(new_size&(alignment-1))) new_size = (new_size&~(alignment-1))+alignment;
	if (preallocated) *preallocated -= new_size + size - ctx->buffer_pos;
	ctx->buffer_pos = new_size + size;

	if (first_chunk) ctx->chunk_start = new_size - HEADER_SIZE;
	else ctx->chunk_start = new_size;

	ctx->chunk_alignment = alignment;

/*	
	if (ctx->chunk_start > pos)
	    memset(buffer+pos, OPC_BUFFER_ALIGN_BYTE, ctx->chunk_start - pos);
*/
//    printf(":%lu %lu %lu\n",ctx->buffer_start, ctx->buffer_end, ctx->buffer_pos);
	return buffer + new_size;
    } 
    
    pos = ctx->buffer_pos;
    if ((alignment)&&(pos&(alignment-1)))
	pos = (pos&~(alignment-1))+alignment;
    *preallocated -= pos+size - ctx->buffer_pos;
    ctx->buffer_pos = pos + size;

//    printf(";%lu %lu %lu\n",ctx->buffer_start, ctx->buffer_end, ctx->buffer_pos);
    return ctx->buffer + pos;
}



OPCBufferContext opcBufferFree(OPCBufferContext ctx, unsigned char *ptr) {
    OPCBufferContext next;
    unsigned char *buffer;
    unsigned long buffer_start;
    unsigned long buffer_size;
    unsigned long size;
    void *notlast=(void*)ctx;
    
    assert(ctx);

    buffer = ctx->buffer;
    buffer_start = ctx->buffer_start+HEADER_SIZE;
    buffer_size = ctx->buffer_size;

    ALIGN(buffer_start, opc_binary_configuration->align_allocation);
    assert((!ptr)||(ptr==buffer+buffer_start));

    while (notlast) {
	size = *((unsigned long*)(buffer + buffer_start - HEADER_SIZE));
	notlast = *((void**)(buffer + buffer_start - sizeof(void*)));
	
	if (notlast) {
	    if ((ctx->buffer_end)&&(ctx->buffer_end<buffer_start)) {
		buffer_start = HEADER_SIZE;
		ALIGN(buffer_start, opc_binary_configuration->align_allocation);
	    } else {
		next = ctx->next;
		opcBufferPoolReturnBuffer(ctx->pool, ctx);
		return opcBufferFree(next, NULL);
	    }
	} else {
	    buffer_start += size;
	    if (((buffer_size-buffer_start)<sizeof(unsigned long))||(*(unsigned long*)(buffer+buffer_start)==OPC_BUFFER_BREAK))
		buffer_start = 0;

	    if (buffer_start == ctx->buffer_end) {
		next = ctx->next;
		opcBufferPoolReturnBuffer(ctx->pool, ctx);
		return next;
	    }

	    if (ctx->last_allocation_end==buffer_start) {
		ctx->last_allocation_end==(unsigned long)-1;
	    }
	}
    }
    ctx->buffer_start = buffer_start;
    return ctx;
}

void opcBufferChunkCancel(OPCBufferContext ctx) {
    assert(ctx);
    ctx->buffer_pos = ctx->buffer_end;
}

unsigned char *opcBufferChunkAccept(OPCBufferContext ctx, unsigned long *ret_size) {
    unsigned char *buffer;
    unsigned long size;
    unsigned long chunk_start;
    
    assert(ctx);

    chunk_start = ctx->chunk_start;
    buffer = ctx->buffer;

    assert(buffer);
    assert(ctx->buffer_end != ctx->buffer_pos);
    assert(ctx->buffer_pos < ctx->buffer_size);
    

    if (ctx->buffer_pos<ctx->block_start) { /* (unsigned long)-1 = true */
	assert(ctx->buffer_pos > (chunk_start + HEADER_SIZE));

	if (ctx->buffer_end == ctx->buffer_start) ctx->buffer_start = chunk_start;

	if ((ctx->buffer_size-ctx->buffer_pos)<sizeof(unsigned long)) {
	    ctx->buffer_end=0;
	    ctx->buffer_pos=0;
	} else ctx->buffer_end = ctx->buffer_pos;

	size = ctx->buffer_pos - chunk_start - HEADER_SIZE;

	*((unsigned long*)(buffer + chunk_start)) = size;
	if (ret_size) *ret_size = size;

	if (ctx->block_start == (unsigned long)-1) {
	    if (ctx->preptr) *((unsigned char**)ctx->preptr)=buffer + chunk_start + HEADER_SIZE;
	} else {
	    *((unsigned char**)(buffer + ctx->block_start + sizeof(unsigned long))) = buffer + chunk_start + HEADER_SIZE;
	} 
	
	ctx->block_start = chunk_start;

	*((unsigned char**)(buffer + chunk_start + sizeof(unsigned long))) = NULL;

	return buffer + chunk_start + HEADER_SIZE;
    } else {
	assert(ctx->buffer_pos > chunk_start);

	if ((ctx->buffer_size-ctx->buffer_pos)<sizeof(unsigned long)) {
	    ctx->buffer_end=0;
	    ctx->buffer_pos=0;
	} else ctx->buffer_end = ctx->buffer_pos;

	if (ret_size) {
	    size = ctx->buffer_pos - chunk_start;
	    *ret_size = size;
	}
	*((unsigned long*)(buffer + ctx->block_start)) = ctx->buffer_pos - ctx->block_start - HEADER_SIZE;
	return buffer + chunk_start;
    }
}

unsigned char *opcBufferChunkSimple(OPCBufferContext ctx, unsigned long size, unsigned long *preallocated) {
    unsigned char *buffer;
    unsigned long alignment;
    unsigned long new_size;
    unsigned long pos;
    unsigned char first_chunk=1;
    
    assert(ctx);
    assert (ctx->buffer_end == ctx->buffer_pos);

    if (ctx->buffer_start != ctx->buffer_end) {
        if (ctx->block_start==(unsigned long)-1) first_chunk=1;
        else first_chunk=0;
    }
    
    buffer = ctx->buffer;
	
    alignment = opc_binary_configuration->align_allocation;
    if (first_chunk) {
	pos = ctx->buffer_end + HEADER_SIZE;
	new_size = size + HEADER_SIZE;
    } else { 
	pos = ctx->buffer_end;
	new_size = size;
    }
    if ((alignment)&&(pos&(alignment-1))) pos = (pos&~(alignment-1))+alignment;

    *preallocated -= new_size;
    
    if ((ctx->buffer_size-pos-size)<sizeof(unsigned long)) {
	ctx->buffer_pos=0;
	ctx->buffer_end=0;
    } else {
	ctx->buffer_pos = pos + size;
	ctx->buffer_end = ctx->buffer_pos;
    }

    if (pos<ctx->block_start) { /* (unsigned long)-1 = true */
	*((unsigned long*)(buffer + pos - HEADER_SIZE)) = size;

	if (ctx->block_start == (unsigned long)-1) {
	    if (ctx->preptr) *((unsigned char**)ctx->preptr)=buffer + pos;
	} else {
	    *((unsigned char**)(buffer + ctx->block_start + sizeof(unsigned long))) = buffer + pos;
	} 
	ctx->block_start = pos - HEADER_SIZE;
	*((unsigned char**)(buffer + pos - sizeof(void*))) = NULL;
    } else {
	*((unsigned long*)(buffer + ctx->block_start)) = ctx->buffer_pos - ctx->block_start - HEADER_SIZE;
    }

    return buffer + pos;
}

unsigned char *opcBufferSimpleAllocate(OPCBufferContext ctx, unsigned long size, unsigned long *preallocated, unsigned long *used_space) {
    unsigned char *buffer;
    unsigned long alignment;
    unsigned long new_size;
    unsigned long pos;
    
    assert(ctx);
    assert (ctx->buffer_end == ctx->buffer_pos);
    assert (ctx->block_start == (unsigned long)-1);

    buffer = ctx->buffer;

    alignment = opc_binary_configuration->align_allocation;
    pos = ctx->buffer_end + HEADER_SIZE;
    new_size = size + HEADER_SIZE;
    if ((alignment)&&(pos&(alignment-1))) pos = (pos&~(alignment-1))+alignment;
    *preallocated -= new_size;
    
    if ((ctx->buffer_size-pos-size)<sizeof(unsigned long)) {
	ctx->buffer_pos=0;
	ctx->buffer_end=0;
    } else {
	ctx->buffer_pos = pos + size;
	ctx->buffer_end = ctx->buffer_pos;
    }

    *((unsigned long*)(buffer + pos - HEADER_SIZE)) = size;
    *((unsigned char**)(buffer + pos - sizeof(void*))) = NULL;

    ctx->last_allocation_end = ctx->buffer_end;
    
    if (used_space) {
	size = ctx->buffer_end - ctx->buffer_start;
	if ((long)size<0) *used_space = ctx->buffer_size + (long)size;
	else *used_space = size;
    }

    return buffer + pos;
}


/*
opcBufferCancel
    Cancels current allocation
    
    ctx: First buffer of allocation!
    
*/

OPCBufferContext opcBufferCancel(OPCBufferContext ctx) {
    OPCBufferContext next, next2;
    
    assert(ctx);
    
    if (ctx->last_allocation_end==(unsigned long)-1) {
	if (ctx->next) {
	    next = ctx->next;
	    opcBufferPoolReturnBuffer(ctx->pool, ctx);
	    for (ctx=next;ctx->next;ctx=next) {
		next=ctx->next;
		opcBufferPoolReturnBuffer(ctx->pool, ctx);
	    }
	}
	
	ctx->buffer_start = 0;
	ctx->buffer_end = 0;
	ctx->buffer_pos = 0;
        ctx->block_start = (unsigned long)-1;
	ctx->preptr = NULL;
    } else {
	if (ctx->next) {
	    for (next=ctx->next;next->next;next=next2) {
		next2 = next->next;
		opcBufferPoolReturnBuffer(ctx->pool, next);
	    }
	    ctx->next = next;
	    
	    next->buffer_start = 0;
	    next->buffer_end = 0;
	    next->buffer_pos = 0;
	    next->block_start = (unsigned long)-1;
	    next->preptr = NULL;
	} else {
	    ctx->block_start = (unsigned long)-1;
	    ctx->preptr = NULL;
	}
	ctx->buffer_end = ctx->last_allocation_end;
	ctx->buffer_pos = ctx->buffer_end;
    }

    return ctx;
}


int opcBufferAccept(OPCBufferContext ctx, unsigned long *used_space) {
    long size;
    assert(ctx);
    
    if (ctx->buffer_end != ctx->buffer_pos) opcBufferChunkAccept(ctx,NULL);
/*    assert(ctx->block_start!=(unsigned long)-1); Can be if last chunk canceled */

/*
    if (ctx->block_start==(unsigned long)-1) {
	if (ctx->preptr) *((unsigned long*)(ctx->preptr-sizeof(unsigned long)))= -*((unsigned long*)(ctx->preptr-sizeof(unsigned long)));
	else return 1;
    } else {
	*((unsigned long*)(buffer + ctx->block_start)) = -*((unsigned long*)(buffer + ctx->block_start));
    }
*/

    ctx->last_allocation_end = ctx->buffer_end;
    ctx->block_start = (unsigned long)-1;

    
    if (used_space) {
	size = ctx->buffer_end - ctx->buffer_start;
	if (size<0) *used_space = ctx->buffer_size + size;
	else *used_space = size;
    }

    return 0;
}
