#ifndef _OPC_BUFFER_H
#define _OPC_BUFFER_H

typedef struct OPCBufferContextT *OPCBufferContext;

#include <opcbufferpool.h>

/*
OPCBufferContext:
    Contains undefined number of chunks. Chunks can be grouped into allocations (Buffer can 
    contain only part of allocation). In this case contigious chunks belonging to same allocation 
    grupped into blocks, first chunk of the block prepended by HEADER(size(uint32) + next_ptr).
    Theoretecally it is possible to have 2 blocks of the same allocation in the one buffer. 
*/

struct OPCBufferContextT {
    OPCBufferPoolContext pool;
    unsigned char *buffer;
    unsigned long buffer_size;


    OPCBufferContext next;

    unsigned long buffer_start;		/* First allocated byte (aligned on alloc_align) */
    unsigned long buffer_end;		/* Last allocated byte, belonging to completed chunk 
					(for finished blocks aligned on alloc_align) */
    unsigned long buffer_pos;		/* Current allocation pointer (first free byte) */
    
    
    unsigned long chunk_start;		/* Position of size field of currently allocating chunk */

    
    unsigned long block_start;		/* Position of size field of currently allocating block,
					-1 if no chunks allocated in allocation in current block */
    unsigned long last_allocation_end;	/* End of last completed allocation (for cancel),
					-1 if no allocation end in buffer */

    void *preptr;			/* Pointer on next header field of unfinished allocation
					in previous buffer (last block), NULL if no unfinished
					blocks of current allocation in previous buffers */
    
    unsigned char chunk_alignment;	/* Alignment of current chunk */
};


int opcBufferInitContext(OPCBufferContext ctx);
void opcBufferFreeContext(OPCBufferContext ctx);
void opcBufferRestoreContext(OPCBufferContext ctx);


unsigned char *opcBufferAllocate(OPCBufferContext ctx, unsigned long softmax, unsigned long size, unsigned char alignment);
unsigned char *opcBufferFastAllocate(OPCBufferContext ctx, unsigned long size, unsigned char alignment, unsigned long *preallocated);
OPCBufferContext opcBufferFree(OPCBufferContext ctx, unsigned char *ptr);
unsigned char *opcBufferChunkAccept(OPCBufferContext ctx, unsigned long *ret_size);
void opcBufferChunkCancel(OPCBufferContext ctx);
int opcBufferAccept(OPCBufferContext ctx, unsigned long *used_space);
OPCBufferContext opcBufferCancel(OPCBufferContext ctx);

unsigned char *opcBufferChunkSimple(OPCBufferContext ctx, unsigned long size, unsigned long *preallocated);
unsigned char *opcBufferSimpleAllocate(OPCBufferContext ctx, unsigned long size, unsigned long *preallocated, unsigned long *used_space);

unsigned long opcBufferReserve(OPCBufferContext ctx, unsigned long softmax, unsigned long size);
void opcBufferTrim(OPCBufferContext ctx, unsigned long size);

#endif /* _OPC_BUFFER_H */
