#ifndef _OPC_DATA_BUFFER_H
#define _OPC_DATA_BUFFER_H

#include <opcmalloc.h>

typedef struct OPCDataBufferContextT *OPCDataBufferContext;

#include <opcallocator.h>
#include <opcbuffer.h>
#include <opcbufferpool.h>
#include <opcuserdata.h>

typedef struct OPCDataBufferOptionsT *OPCDataBufferOptions;

struct OPCDataBufferContextT {
    struct OPCAllocatorContextT base;

    OPCBufferPoolContext pool;
    OPCBufferContext buffers;
    OPCBufferContext current;
    OPCBufferContext first_chunk;
    OPCAllocation allocation;
    
    unsigned long preallocated;
    unsigned long block_size;
    
    unsigned long allocation_size;

    unsigned long n;
    unsigned long max1, max2;
    unsigned long save;
    
    unsigned long maxfill;
    
    unsigned long minimal_size;
    unsigned long maximal_size;
    unsigned long fixed_allocation_size;
};

struct OPCDataBufferOptionsT {
    OPCBufferPoolContext pool;
    unsigned long minimal_size;
    unsigned long maximal_size;
    unsigned long block_size;
    unsigned long fixed_allocation_size;
};

OPCAllocator opcAllocatorDataBuffer();

OPCDataBufferContext opcDataBufferCreateContext(OPCMallocContext mctx);
int opcDataBufferInitContext(OPCDataBufferContext ctx, OPCDataBufferOptions options);
void opcDataBufferFreeContext(OPCDataBufferContext ctx);
void opcDataBufferDestroyContext(OPCDataBufferContext ctx);

OPCDataBufferOptions opcDataBufferGetDefaultOptions();
int opcDataBufferInitDefaultOptions(OPCDataBufferOptions options);
void opcDataBufferDestroyOptions(OPCDataBufferOptions options);

int opcDataBufferOptionsSetSize(OPCDataBufferOptions options, unsigned long minimal_size, unsigned long maximal_size);
int opcDataBufferOptionsSetBlockSize(OPCDataBufferOptions options, unsigned long block_size);
int opcDataBufferOptionsSetAllocationSize(OPCDataBufferOptions options, unsigned long fixed_allocation_size);
int opcDataBufferOptionsSetBufferPool(OPCDataBufferOptions options, OPCBufferPoolContext pool);


unsigned long opcDataBufferReserve(OPCDataBufferContext ctx, OPCAllocation notrequired, unsigned long size);
void opcDataBufferTrim(OPCDataBufferContext ctx, OPCAllocation notrequired, unsigned long size);
void *opcDataBufferAllocate(OPCDataBufferContext ctx, OPCAllocation notrequired, unsigned long size, unsigned char alignment);
void opcDataBufferFree(OPCDataBufferContext ctx, OPCAllocation buffer);

void *opcDataBufferAcceptChunk(OPCDataBufferContext ctx, OPCAllocation notrequired, unsigned long *req_size);
void opcDataBufferCancelChunk(OPCDataBufferContext ctx, OPCAllocation notrequired);
OPCAllocation opcDataBufferAccept(OPCDataBufferContext ctx, OPCAllocation notrequired);
void opcDataBufferCancel(OPCDataBufferContext ctx, OPCAllocation notrequired);

void *opcDataBufferSimpleChunk(OPCDataBufferContext ctx, OPCAllocation notrequired, unsigned long size);
/* OPCAllocation for DataBuffer is a pointer on the first buffer and therefore the pointer
returned by SimpleAllocate can be directly used to store data, - but it can be wrong for
other allocators! And normaly opcDataBufferGet call should be used */
OPCAllocation opcDataBufferSimpleAllocate(OPCDataBufferContext ctx, unsigned long size);

int opcDataBufferSend(OPCUserData user_data, OPCSendFunction func, void *attr);
int opcDataBufferCall(OPCDataBufferContext ctx, OPCAllocation allocation, OPCSendFunction func, void *attr, unsigned char align);
unsigned long opcDataBufferSizeof(OPCDataBufferContext ctx, OPCAllocation allocation, unsigned char align);
void* opcDataBufferGet(OPCDataBufferContext ctx, OPCAllocation allocation, unsigned long *size);
void* opcDataBufferGetNext(OPCDataBufferContext ctx, OPCAllocation allocation, void *current, unsigned long *size);

void opcDataBufferMantain(OPCDataBufferContext ctx);


#ifdef _OPC_DATA_BUFFER_C
static const struct OPCAllocatorT opc_allocator_databuffer = {
    "DataBuffer",
    (OPCAllocatorCreateFunction)opcDataBufferCreateContext,
    (OPCAllocatorInitFunction)opcDataBufferInitContext,
    (OPCAllocatorFreeFunction)opcDataBufferFreeContext,
    (OPCAllocatorDestroyFunction)opcDataBufferDestroyContext,
    (OPCAllocatorGetDefaultOptions)opcDataBufferGetDefaultOptions,
    (OPCAllocatorInitDefaultOptions)opcDataBufferInitDefaultOptions,
    (OPCAllocatorDestroyOptions)opcDataBufferDestroyOptions,
    (OPCAllocatorMantainFunction)opcDataBufferMantain,
    NULL,
    (OPCAllocatorAllocateMemFunction)opcDataBufferAllocate,
    (OPCAllocatorFreeMemFunction)opcDataBufferFree,
    (OPCAllocatorAcceptChunkFunction)opcDataBufferAcceptChunk,
    (OPCAllocatorCancelChunkFunction)opcDataBufferCancelChunk,
    (OPCAllocatorAcceptFunction)opcDataBufferAccept,
    (OPCAllocatorCancelFunction)opcDataBufferCancel,
    (OPCAllocatorAllocateSimpleChunkFunction)opcDataBufferSimpleChunk,
    (OPCAllocatorAllocateSimpleFunction)opcDataBufferSimpleAllocate,
    (OPCAllocatorReserveFunction)opcDataBufferReserve,
    (OPCAllocatorTrimFunction)opcDataBufferTrim,
    (OPCAllocatorGetFunction)opcDataBufferGet,
    (OPCAllocatorGetNextFunction)opcDataBufferGetNext,
    (OPCAllocatorCallFunction)opcDataBufferCall,
    (OPCAllocatorSizeofFunction)opcDataBufferSizeof
};

static const struct OPCDataBufferOptionsT opc_databuffer_options = {
    NULL,
    0,
    0,
    0,
    0
};
#endif /* _OPC_DATA_BUFFER_C */

#endif /* _OPC_DATA_BUFFER_H */
