#include <stdio.h>
#include <assert.h>

#include <dsgnuintl.h>

#define _OPC_C
#include <opcerr.h>
#include <opcconfig.h>

#include <opcmalloc.h>
#include <opcerror.h>
#include <opcdebug.h>

#include <opcsignal.h>
#include <opcbinaryconfiguration.h>

#include <opcserverinterface.h>

#include <opc.h>

int opcInit() {
    int err;

	// XML Libraries Initialisation
    xmlInitParser();
    xmlInitCharEncodingHandlers();

    err=opcErrorInit();
    if (!err) err=opcBinaryConfigurationInit();
    if (!err) err=opcSignalInit();
    return err;
}

void opcDestroy() {
    opcSignalDestroy();
    opcBinaryConfigurationDestroy();
    opcErrorDestroy();

	// XML Libraries Deintialization
    xmlCleanupCharEncodingHandlers();
    xmlCleanupParser();
}

static opcZeroContext(OPCContext ctx) {
    assert(ctx);

    ctx->version=NULL;
    ctx->state=NULL;
    ctx->start_time=NULL;
    ctx->locales=NULL;
    ctx->tree=NULL;
    ctx->holder=NULL;
    ctx->config=NULL;
    
    ctx->fd_opcbinary=NULL;
    ctx->formats=NULL;
    ctx->chunks=NULL;
    ctx->buffers=NULL;
    ctx->properties=NULL;
}

OPCContext opcCreateContext(OPCMallocContext mctx) {
    OPCContext ctx;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcCreateContext");

    ctx = (OPCContext)opcMalloc(mctx,sizeof(struct OPCContextT));
    if (ctx) {
	ctx->malloc = mctx;
	opcZeroContext(ctx);
    }
    return ctx;
}

void opcFreeContext(OPCContext ctx) {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcFreeContext");

    assert(ctx);
    if (ctx->tree) opcTreeDestroyContext(ctx->tree);
    if (ctx->holder) opcHolderDestroyContext(ctx->holder);
    if (ctx->fd_opcbinary) opcFDOPCBinaryDestroyContext(ctx->fd_opcbinary);
    if (ctx->chunks) opcChunkPoolDestroyContext(ctx->chunks);
    if (ctx->buffers) opcBufferPoolDestroyContext(ctx->buffers);
    if (ctx->formats) opcFormatsDestroyContext(ctx->formats);
    if (ctx->properties) opcPropertiesDestroyContext(ctx->properties);
    if (ctx->version) opcVersionDestroy(ctx->version);
    if (ctx->start_time) opcTimeDestroy(ctx->start_time);
    if (ctx->state) opcStateDestroy(ctx->state);
    if (ctx->locales) opcLocalesDestroyContext(ctx->locales);
    if (ctx->config) opcConfigurationDestroyContext(ctx->config);
    opcZeroContext(ctx);
}

void opcDestroyContext(OPCContext ctx) {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcDestroyContext");
    
    assert(ctx);
    opcFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}

int opcInitContext(OPCContext ctx, OPCConfigurationContext cfg, unsigned long scalability) {
    int err=0;
    unsigned long i;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcInitContext");    

    assert(ctx);
    
    opcZeroContext(ctx);
	// GetText-DS Library Initialisation
    if (!bindtextdomain(opc_text_domain,opc_message_catalog_dir)) return ERR_OPC_GETTEXT_INIT;
    if (!textdomain(opc_text_domain)) return ERR_OPC_GETTEXT_INIT;
    
	// Initialising server structures
    if (cfg) ctx->config=cfg;
    else {
	ctx->config = opcConfigurationCreateContext(ctx->malloc);
	if (ctx->config) err=opcConfigurationInitContext(ctx->config, scalability);
	else err=ERR_OPC_MEM;
    }
    if (!err) {
	ctx->holder = opcHolderCreateContext(ctx->malloc);
	if (ctx->holder) err=opcHolderInitContext(ctx->holder, ctx->config->holder_size);
	else err=ERR_OPC_MEM;
	if (!err) {
	    opcHolderSetMaxSize(ctx->holder, ctx->config->holder_maximal_size);
	    opcHolderSetTimeout(ctx->holder, ctx->config->resource_wait_timeout);
	}
    }
    if (!err) err=opcServerVersionSet(ctx,opc_version_major,opc_version_minor,opc_version_build);
    if (!err) err=opcServerLocalesAdd(ctx,opc_supported_locales);    
    if (!err) err=opcServerVendorInfoSet(ctx,opc_vendor_info);
    if (!err) err=opcServerStateSet(ctx,OPC_STATE_RUNNING,lgs("All OK"));
    if (!err) err=opcServerStartTimeSet(ctx);
    if (!err) err=opcServerInitTree(ctx);

	// Initialising properties
    if (!err) {
	ctx->buffers = opcBufferPoolCreateContext(ctx->malloc);
	if (ctx->buffers) err = opcBufferPoolInitContext(ctx->buffers, ctx->config->buffer_pool_smallest, ctx->config->buffer_pool_bins);
	else err=ERR_OPC_BUFFERPOOL_CREATE;
    }
    if (!err) {
	ctx->fd_opcbinary = opcFDOPCBinaryCreateContext(ctx->malloc);
	if (ctx->fd_opcbinary) err = opcFDOPCBinaryInitContext(ctx->fd_opcbinary,&ctx->config->namespaces,ctx->config->opcbinary_schema,ctx->config->opcbinary_dictionaries);
	else err=ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_CREATE;
    }
    if (!err) {
	ctx->formats = opcFormatsCreateContext(ctx->malloc);
	if (ctx->formats) err = opcFormatsInitContext(ctx->formats, ctx->config);
	else err=ERR_OPC_FORMATS_CREATE;
    }
    if (!err) {
	ctx->chunks = opcChunkPoolCreateContext(ctx->malloc);
	if (ctx->chunks) err = opcChunkPoolInitContext(ctx->chunks, ctx->config->chunk_pool_bins);
	else err=ERR_OPC_CHUNKPOOL_CREATE;
    }
    if (!err) {
	ctx->properties = opcPropertiesCreateContext(ctx->malloc);
	if (ctx->properties) err=opcPropertiesInitContext(ctx->properties, ctx->config->properties_hash_size, ctx->config->property_maxid, ctx->config->property_userid);
	else err=ERR_OPC_PROPERTIES_CREATE;
    }

    return err;
}

void opcMantain(OPCContext ctx) {
    assert(ctx);
    opcHolderMantain(ctx->holder);
    opcBufferPoolMantain(ctx->buffers);
}
