#ifndef _OPC_LIB_CONFIG_H
#define _OPC_LIB_CONFIG_H

#include <time.h>
#include <opcmalloc.h>

typedef struct OPCNamespacesT *OPCNamespaces;
struct OPCNamespacesT {
    OPCMallocContext malloc;
    char *xsi_namespace;
    char *opc_namespace;
    char *opcbinary_namespace;
};


typedef struct OPCConfigurationContextT *OPCConfigurationContext;
struct OPCConfigurationContextT {
    OPCMallocContext malloc;

    unsigned long max_elements_returned;

    unsigned long holder_size;
    unsigned long holder_maximal_size;
    
    unsigned long formats_hash_size;
    unsigned long properties_hash_size;
    unsigned long property_maxid;
    unsigned long property_userid;
    
    unsigned char chunk_pool_bins;
    
    unsigned long buffer_pool_smallest;
    unsigned char buffer_pool_bins;
    
    char *data_directory;
    char *schema_directory;
    char *opc_schema;
    char *opcbinary_schema;
    char *dictionary_directory;
    char *opcbinary_dictionaries;
    
    struct OPCNamespacesT namespaces;
    
    time_t resource_wait_timeout;
    time_t browse_timeout;
    time_t request_timeout;
};

OPCConfigurationContext opcConfigurationCreateContext(OPCMallocContext mctx);
int opcConfigurationInitContext(OPCConfigurationContext ctx, unsigned long scalability);
void opcConfigurationFreeContext(OPCConfigurationContext ctx);
void opcConfigurationDestroyContext(OPCConfigurationContext ctx);

int opcConfigurationSetBrowseTimeout(OPCConfigurationContext ctx, time_t browse_timeout);
int opcConfigurationSetResourceWaitTimeout(OPCConfigurationContext ctx, time_t resource_timeout);
int opcConfigurationSetRequestTimeout(OPCConfigurationContext ctx, time_t request_timeout);
int opcConfigurationSetHolderSize(OPCConfigurationContext ctx, unsigned long holder_size, unsigned long holder_maximal_size);
int opcConfigurationSetFormatsHashSize(OPCConfigurationContext ctx, unsigned long formats_hash_size);
int opcConfigurationSetPropertiesHashSize(OPCConfigurationContext ctx, unsigned long properties_hash_size);
int opcConfigurationSetPropertiesIDs(OPCConfigurationContext ctx, unsigned long maxid, unsigned long userid);
int opcConfigurationSetChunkPoolConfig(OPCConfigurationContext ctx, unsigned char bins);
int opcConfigurationSetBufferPoolConfig(OPCConfigurationContext ctx, unsigned long smallest, unsigned char bins);
int opcConfigurationSetDataDirectory(OPCConfigurationContext ctx, const char *data_directory);
int opcConfigurationSetSchemaNamespace(OPCConfigurationContext ctx, const char *xsi_namespace);
int opcNamespacesSetOPCBinaryNamespace(OPCNamespaces ctx, const char *opcbinary_namespace);
int opcNamespacesSetOPCNamespace(OPCNamespaces ctx, const char *opc_namespace);

void opcConfigurationDefault(OPCConfigurationContext ctx, unsigned long scalability);

#define OPC_CONFIGURATION_BROWSE_TIMEOUT 60
#define OPC_CONFIGURATION_RESOURCE_TIMEOUT 3
#define OPC_CONFIGURATION_REQUEST_TIMEOUT 60

#define OPC_CONFIGURATION_HOLDER_SIZE 2
#define OPC_CONFIGURATION_HOLDER_MAXIMAL_SIZE 10
#define OPC_CONFIGURATION_FORMATS_HASH_SIZE 10
#define OPC_CONFIGURATION_PROPERTY_MAXID 2048
#define OPC_CONFIGURATION_PROPERTY_USERID 1024
#define OPC_CONFIGURATION_PROPERTIES_HASH_SIZE 128
#define OPC_CONFIGURATION_CHUNK_POOL_BINS 4
#define OPC_CONFIGURATION_BUFFER_POOL_SMALLEST 512
#define OPC_CONFIGURATION_BUFFER_POOL_BINS 12


#define OPC_CONFIGURATION_DATA_DIRECTORY "/usr/share/opc"
#define OPC_CONFIGURATION_SCHEMA_DIRECTORY "schemas"
#define OPC_CONFIGURATION_OPCBINARY_SCHEMA "opcbinary.xsd"
#define OPC_CONFIGURATION_OPC_SCHEMA "opc.xsd"
#define OPC_CONFIGURATION_DICTIONARY_DIRECTORY "dictionaries"
#define OPC_CONFIGURATION_OPCBINARY_DICTIONARIES_DIRECTORY "opcbinary"
#define OPC_CONFIGURATION_NAMESPACE_XSI "http://www.w3.org/2001/XMLSchema-instance"



#endif /* _OPC_LIB_CONFIG_H */
