#define _OPC_DATA_C

#include <opcerr.h>
#include <opcassert.h>

#include <opcerror.h>
#include <opcdebug.h>
#include <opcdata.h>

#define OPC_DATA_ALLOCATE_PRESENTATIONS 2


OPCDataOptions opcDataGetDefaultOptions() {
    int err;
    OPCDataOptions options;

    options = (OPCDataOptions)malloc(sizeof(struct OPCDataOptionsT));
    if (!options) return options;
    
    err = opcDataInitDefaultOptions(options);
    if (err) {
	opcDataDestroyOptions(options);
	return NULL;
    }
    return options;
}

int opcDataInitDefaultOptions(OPCDataOptions options) {
    assert(options);
    memcpy(options, &opc_datamanager_data_options, sizeof(struct OPCDataOptionsT));
    return 0;
}

void opcDataDestroyOptions(OPCDataOptions options) {
    if (options) free(options);
}


int opcDataOptionsSetAcquireFunction(OPCDataOptions options, OPCDataManagerAcquireFunction acquire) {
    assert(options);
    options->acquire = acquire;
    return 0;
}

int opcDataOptionsSetBufferSize(OPCDataOptions options, unsigned long min, unsigned long max) {
    assert(options);
    options->binary_minbuf_size = min;
    options->binary_maxbuf_size = max;
    return 0;
}

int opcDataOptionsSetCacheSize(OPCDataOptions options, unsigned long min, unsigned long max) {
    assert(options);
    options->cache_min = min;
    options->cache_max = max;
    return 0;
}

int opcDataOptionsSetFormatsContext(OPCDataOptions options, OPCFormatsContext formats) {
    assert(options);
    options->formats = formats;
    return 0;
}

int opcDataOptionsSetBinaryAllocator(OPCDataOptions options, OPCAllocatorContext allocator) {
    assert(options);
    options->binary_allocator = allocator;
    return 0;
}

int opcDataOptionsSetBufferPool(OPCDataOptions options, OPCBufferPoolContext buffer_pool) {
    assert(options);
    options->buffer_pool = buffer_pool;
    return 0;
}

int opcDataOptionsSetChunkPool(OPCDataOptions options, OPCChunkPoolContext chunk_pool) {
    assert(options);
    options->chunk_pool = chunk_pool;
    return 0;
}

int opcDataOptionsSetDataSourceFlags(OPCDataOptions options, unsigned long flags) {
    assert(options);
    options->flags = flags;
    return 0;
}


static inline void opcDataZeroContext(OPCDataContext ctx) {
    assert(ctx);

    ctx->representation = NULL;
    ctx->repinfo = NULL;
    ctx->rephash = NULL;
    ctx->native_representation = NULL;
    ctx->opcxml_representation = NULL;
    
    ctx->chunks = NULL;
    ctx->current = NULL;
    ctx->lock = NULL;
    ctx->representation_lock = NULL;
    ctx->chunk_lock = NULL;
    
    ctx->nchunks = 0;
}

static void opcDataFreeContext(OPCDataContext ctx) {
    OPCDataChunkContext chunk, nchunk;
    long i;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcDataFreeContext: %p", ctx);    

    assert(ctx);
    
    if (ctx->lock) LockDestroyContext(ctx->lock);
    if (ctx->chunk_lock) LockDestroyContext(ctx->chunk_lock);
    if (ctx->representation_lock) LockDestroyContext(ctx->representation_lock);

    for (chunk=ctx->chunks;chunk;chunk=nchunk) {
        nchunk = chunk->next;
	opcDataChunkWaitUnref(chunk, 0);
        opcDataChunkDestroyContext(chunk);
    }

    if (ctx->rephash) HashTableDestroy(ctx->rephash);    
    if (ctx->representation) opcFree(ctx->malloc, ctx->representation);
    if (ctx->repinfo) opcFree(ctx->malloc, ctx->repinfo);
    
    opcDataZeroContext(ctx);    
}

static void opcDataDestroyContext(OPCDataContext ctx) {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcDataDestroyContext: %p", ctx);    
    opcDataFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}


OPCDataContext opcDataCreateContext(OPCMallocContext mctx) {
    OPCDataContext ctx;

    ctx = (OPCDataContext)opcMalloc(mctx,sizeof(struct OPCDataContextT));
    opcDebugServer(OPC_DEBUG_NOTICE, "opcDataCreateContext: %p", ctx);    
    if (ctx) {
	ctx->malloc=mctx;
	ctx->refs = 1;
	opcDataZeroContext(ctx);
	memcpy(&ctx->manager, &opc_datamanager_data, sizeof(struct OPCDataManagerContextT));
    }
    return ctx;
}

int opcDataInitContext(OPCDataContext ctx, OPCDataOptions options) {
    int err;
    unsigned short int i;
    OPCBufferPoolContext buffer_pool;
    OPCChunkPoolContext chunk_pool;
    OPCDataRepresentationContext rctx;
    OPCFormatsContext fctx;
    OPCAllocatorContext allocator;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcDataInitContext: %p", ctx);

    assert(ctx);
    
    if ((!ctx)||(!options)) return ERR_OPC_PARAM;
    if ((!options->chunk_pool)) return ERR_OPC_DATA_OPTIONS;
    
    ctx->manager.Acquire = options->acquire;
    ctx->formats = options->formats;
    ctx->binary_allocator = options->binary_allocator;
    ctx->buffer_pool = options->buffer_pool;
    ctx->chunk_pool = options->chunk_pool;
    
    if (options->cache_min) ctx->cache_min = options->cache_min;
    else ctx->cache_min = 1;
    ctx->cache_max = options->cache_max;
    
    ctx->binary_minbuf_size = options->binary_minbuf_size;
    ctx->binary_maxbuf_size = options->binary_maxbuf_size;
    
    ctx->flags = options->flags;
    
    opcDataZeroContext(ctx);    

    ctx->allocated_representations = OPC_DATA_ALLOCATE_PRESENTATIONS;

    ctx->representation = (OPCRepresentationContext*)opcMalloc(ctx->malloc, ctx->allocated_representations*sizeof(OPCDataRepresentationContext));
    ctx->repinfo = (OPCRepresentationInfo*)opcMalloc(ctx->malloc, ctx->allocated_representations*sizeof(OPCRepresentationInfo));
    ctx->rephash = HashTableInit(ctx->malloc, OPC_DATA_HASHTABLE_SIZE, HASH_TYPE_STATICPOINTER, NULL);
    if ((!ctx->representation)||(!ctx->repinfo)||(!ctx->rephash)) {
	opcDataFreeContext(ctx);
	return ERR_OPC_MEM;
    }
    HashTableSetDestructor(ctx->rephash, (HashTableDestructorFunction)opcRepresentationUnref);


    ctx->supported_representations = 2;
    for (i=0;i<ctx->supported_representations;i++)
	ctx->representation[i] = NULL;
    
    ctx->lock = LockCreateContext(ctx->malloc);
    ctx->chunk_lock = LockCreateContext(ctx->malloc);
    ctx->representation_lock = LockCreateContext(ctx->malloc);
    if ((!ctx->lock)||(!ctx->chunk_lock)||(!ctx->representation_lock)) {
	opcDataFreeContext(ctx);
	return ERR_OPC_LOCK_CREATE;
    }
    err = LockInitContext(ctx->lock);
    if (!err) err = LockInitContext(ctx->chunk_lock);
    if (!err) err = LockInitContext(ctx->representation_lock);
    if (err) {
	opcDataFreeContext(ctx);
	return err;
    }

    return 0;
}


int opcDataRef(OPCDataContext ctx) {
    assert(ctx);
    
    LockWrite(ctx->lock);
    ctx->refs++;
    LockReturn(ctx->lock);
}

void opcDataUnref(OPCDataContext ctx) {
    assert(ctx);
    if (!ctx->lock) {
	opcDataDestroyContext(ctx);
	return;
    }
    
    LockWrite(ctx->lock);
    if ((--ctx->refs)==0) {
	LockReturn(ctx->lock);
	opcDataDestroyContext(ctx);
	return;
    }
    LockReturn(ctx->lock);
}


int opcDataSetDataRepresentationOptions(OPCDataContext ctx, OPCDataRepresentationOptions options, OPCFormatContext format) {
    assert(ctx);
    assert(options);
    assert(format);
    
    if ((!ctx)||(!options)||(!format)) return ERR_OPC_PARAM;
    
    opcDataRepresentationOptionsSetFormat(options, format);
    opcDataRepresentationOptionsSetBufferSize(options, ctx->binary_minbuf_size, ctx->binary_maxbuf_size);
    opcDataRepresentationOptionsSetDefaultAllocatorOptions(options,ctx->buffer_pool);
    opcDataRepresentationOptionsSetAllocator(options, ctx->binary_allocator);
    opcDataRepresentationOptionsSetFlags(options, ctx->flags);
    return 0;
}

unsigned long opcDataGetId(OPCDataContext ctx, const char *repname) {
    OPCRepresentationContext rep;
    
    if ((!ctx)||(!repname)||(!ctx->rephash)) return (unsigned long)-1;
    
    LockRead(ctx->representation_lock);
    rep=HashTableFind(ctx->rephash, (void*)repname);
    LockReturn(ctx->representation_lock);
    
    if (!rep) return (unsigned long)-1;
    return rep->id;
}


static int opcDataRegisterRepresentationI(OPCDataContext ctx, OPCRepresentationContext rep, unsigned short int *repnum) {
    int err;
    OPCRepresentationContext *representations;
    unsigned short int repid;
    
    repid = *repnum;

    if (!rep->title) return ERR_OPC_DATA_REGISTER_REPRESENTATION_INCOMPLETE;
    
    LockWrite(ctx->representation_lock);
    if (repid==(unsigned short int)-1) {
	if (ctx->supported_representations==ctx->allocated_representations) {
	    representations = (OPCRepresentationContext*)opcRealloc(ctx->malloc, ctx->representation, 2*ctx->allocated_representations*sizeof(OPCDataRepresentationContext));
	    if (!representations) return ERR_OPC_MEM;
	    ctx->representation = representations;
	    ctx->allocated_representations*=2;
	}
	repid = ctx->supported_representations;	/* Isn't default representation and can't be accessed until in hash */
    } else {
        if (ctx->representation[repid]) return ERR_OPC_DATA_REGISTER_REPRESENTATION_ALREDY;
    }

    opcRepresentationSetId(rep, repid);
    if (HashTableAdd(ctx->rephash, (void*)rep, 1)) return ERR_OPC_MEM;
    ctx->representation[repid] = rep;
    ctx->repinfo[repid] = &rep->repinfo;
    ctx->supported_representations++;
    if (*repnum == (unsigned short int)-1) *repnum = repid;
    LockReturn(ctx->representation_lock);

    opcDebugServer(OPC_DEBUG_DEBUG, "opcData(%p): AddRepresentation(%s): Registered as %u.", ctx, rep->title, repid);

    return 0;
}

unsigned short int opcDataRegisterRepresentation(OPCDataContext ctx, OPCRepresentationContext representation) {
    int err;
    unsigned short int rep = (unsigned short int)-1;
    
    err = opcDataRegisterRepresentationI(ctx, representation, &rep);
    if (err) {
	opcDebugServer(OPC_DEBUG_WARNING, "opcData(%p): AddRepresentation(%p): Failed: %s.", ctx, representation, opcError(err, NULL));
	return (unsigned short int)-1;
    }
    
    return rep;
}

int opcDataRegisterNativeRepresentation(OPCDataContext ctx, OPCType type, OPCDataRepresentationOptions options) {
    int err;
    OPCDataRepresentationOptions opts;
    OPCDataRepresentationContext rep;
    OPCFormatsContext formats;
    unsigned short int repid = OPC_REPRESENTATION_NATIVE;

    if ((!ctx)||(!type)) return ERR_OPC_PARAM;

    formats = ctx->formats;
    if ((!formats)||(!formats->native_format)) return ERR_OPC_DATA_INCOMPLETE_FORMATS;
    

    rep = opcDataRepresentationCreateContext(ctx->malloc, "Native");
    if (!rep) return ERR_OPC_DATA_CREATE_NATIVE_REPRESENTATION;
    
    if (options) opts=options;
    else {
	opts = opcDataRepresentationGetDefaultOptions();
	opcDataSetDataRepresentationOptions(ctx, opts, (OPCFormatContext)formats->native_format);
	opcDataRepresentationOptionsSetType(opts, type);
	opcDataRepresentationOptionsSetBase(opts, NULL);
    }

    err = opcDataRepresentationInitContext(rep, opts);
    if (err) opcDataRepresentationUnref(rep);
    else {
	err = opcDataRegisterRepresentationI(ctx, (OPCRepresentationContext)rep, &repid);
	if (err) opcDataRepresentationUnref(rep);
	else ctx->native_representation = rep;
    }

    if (!options) opcDataRepresentationDestroyOptions(opts);
    return err;
}

int opcDataRegisterOPCXMLRepresentation(OPCDataContext ctx, OPCType type, OPCDataRepresentationOptions options) {
    int err;
    OPCDataRepresentationOptions opts;
    OPCDataRepresentationContext rep;
    OPCFormatsContext formats;
    unsigned short int repid = OPC_REPRESENTATION_OPCXML;
    
    if ((!ctx)||(!type)) return ERR_OPC_PARAM;
    if (!ctx->native_representation) return ERR_OPC_DATA_MISSING_NATIVE_REPRESENTATION;

    formats = ctx->formats;
    if ((!formats)||(!formats->opcxml_format)) return ERR_OPC_DATA_INCOMPLETE_FORMATS;

    rep = opcDataRepresentationCreateContext(ctx->malloc, "OPCXML");
    if (!rep) return ERR_OPC_DATA_CREATE_OPCXML_REPRESENTATION;
    
    if (options) opts=options;
    else {
	opts = opcDataRepresentationGetDefaultOptions();
	opcDataSetDataRepresentationOptions(ctx, opts, formats->opcxml_format);
	opcDataRepresentationOptionsSetType(opts, type);
	opcDataRepresentationOptionsSetBase(opts, (OPCRepresentationContext)ctx->native_representation);
    }

    err = opcDataRepresentationInitContext(rep, opts);
    if (err) opcDataRepresentationUnref(rep);
    else {
	err = opcDataRegisterRepresentationI(ctx, (OPCRepresentationContext)rep, &repid);
	if (err) opcDataRepresentationUnref(rep);
	else ctx->opcxml_representation = rep;
    }

    if (!options) opcDataRepresentationDestroyOptions(opts);

    return 0;
}


static inline void opcDataClear(OPCDataContext ctx) {
    LockContext chunk_lock;
    OPCChunkPoolContext chunk_pool;
    OPCDataChunkContext chunk, nchunk;
    unsigned long buffer_min, buffer_max;
    unsigned long ref = 0;

	/* After chunk_lock locked number of referers can decrease, but not increase! */

    chunk_lock = ctx->chunk_lock;
    chunk_pool = ctx->chunk_pool;
    chunk = ctx->chunks;

    buffer_min = ctx->cache_min;
    buffer_max = ctx->cache_max;

    nchunk = chunk;

    if (buffer_max) 
        for (;((ctx->nchunks>buffer_max)&&(!ref));chunk=nchunk) {
            nchunk = chunk->next;
	    ctx->chunks=nchunk;
	    ctx->nchunks--;
	    LockReturn(chunk_lock);
	    opcChunkPoolReturnChunk(chunk_pool, chunk);
	    LockWrite(chunk_lock);
	    ref=opcDataChunkGetRefs(nchunk);
	}
	
    for (;((!ref)&&(ctx->nchunks>buffer_min));chunk=nchunk) {
	nchunk=chunk->next;
	ctx->chunks=nchunk;
	ctx->nchunks--;
	LockReturn(chunk_lock);
	opcChunkPoolReturnChunk(chunk_pool, chunk);
	LockWrite(chunk_lock);
	ref = opcDataChunkGetRefs(nchunk);
	if (!ref) ref = opcDataChunkGetDelayedRefs(nchunk);
    }
    
}

OPCAllocatorContext opcDataGetAllocator(OPCDataContext ctx, unsigned long repid, OPCDataInfo info) {
    int err;
    OPCRepresentationContext rctx;

    if ((!ctx)||(repid>=ctx->supported_representations)) return NULL;

    LockRead(ctx->representation_lock);
    assert(ctx->representation);
    rctx = ctx->representation[repid];
    LockReturn(ctx->representation_lock);
    assert(rctx);
    
    if (!rctx) return NULL;

    return opcRepresentationGetAllocator(rctx, info);
}

void opcDataReturnAllocator(OPCDataContext ctx, unsigned long repid, OPCAllocatorContext allocator) {
    int err;
    OPCRepresentationContext rctx;

    if ((!ctx)||(repid>=ctx->supported_representations)) return;

    LockRead(ctx->representation_lock);
    assert(ctx->representation);
    rctx = ctx->representation[repid];
    LockReturn(ctx->representation_lock);
    assert(rctx);
    
    if (!rctx) return;
    
    return opcRepresentationReturnAllocator(rctx, allocator);
}

int opcDataPushAllocation(OPCDataContext ctx, unsigned long repid, OPCAllocatorContext allocator, OPCDataInfo info, OPCAllocation allocation) {
    int err;
    OPCRepresentationContext rctx;
    OPCDataChunkContext chunk;

    if ((!ctx)||(!allocation)||(repid>=ctx->supported_representations)) return ERR_OPC_DATA_PARAMS;

    LockRead(ctx->representation_lock);
    assert(ctx->representation);
    rctx = ctx->representation[repid];
    LockReturn(ctx->representation_lock);
    assert(rctx);

    if (!rctx) return ERR_OPC_DATA_DISABLED_REPRESENTATION;

    chunk = opcChunkPoolGetChunk(ctx->chunk_pool, ctx->repinfo, ctx->supported_representations);
    if (!chunk) return ERR_OPC_CHUNKPOOL_GET_CHUNK;
    opcDataChunkCopyInfo(chunk, info);

    LockWrite(ctx->lock);
    err = opcRepresentationPushAllocation(rctx, chunk, allocator, allocation);
	// ToDo: prelock datachunk for all subscriptions may need it
    LockReturn(ctx->lock);

    if (err) {
	opcChunkPoolReturnChunk(ctx->chunk_pool, chunk);
	return err;
    }
    
    LockWrite(ctx->chunk_lock);
    ctx->nchunks++;
    if (ctx->current) {
	/* What to do with OOB items, with time earlier than current one?
	For Fast items can be assumed only one sequential data source? For items
	with multiple or/and non-sequential data sources OPCDataContext should be used
	if (opcTimeCmp(ctx->current->info.time, info->time)>0) {
	}
	*/
	ctx->current->next = chunk;
	ctx->current=chunk;

	if ((ctx->nchunks>ctx->cache_min)&&(!opcDataChunkGetRefs(ctx->chunks))) 
	    opcDataClear(ctx);
    } else {
	ctx->chunks = chunk;
	ctx->current = chunk;
    }
    LockReturn(ctx->chunk_lock);
    

    return 0;
    
}



int opcDataPushData(OPCDataContext ctx, unsigned long repid, OPCDataInfo info, const void *data, unsigned long size) {
    int err;
    OPCRepresentationContext rctx;
    OPCDataChunkContext chunk;

    assert(ctx);
    assert(data);

    if ((!ctx)||(!data)||(repid>=ctx->supported_representations)) return ERR_OPC_DATA_PARAMS;

    LockRead(ctx->representation_lock);
    assert(ctx->representation);
    rctx = ctx->representation[repid];
    LockReturn(ctx->representation_lock);
    
    if (!rctx) return ERR_OPC_DATA_DISABLED_REPRESENTATION;

    opcDebugServer(OPC_DEBUG_DEBUG, "opcData(%p): Push (representation: %u).", ctx, repid);
    
    chunk = opcChunkPoolGetChunk(ctx->chunk_pool, ctx->repinfo, ctx->supported_representations);
    if (!chunk) return ERR_OPC_CHUNKPOOL_GET_CHUNK;
    opcDataChunkCopyInfo(chunk, info);

    LockWrite(ctx->lock);
    err = opcRepresentationPushData(rctx, chunk, data, size);
	// ToDo: prelock datachunk for all subscriptions may need it
    LockReturn(ctx->lock);

    if (err) {
	opcChunkPoolReturnChunk(ctx->chunk_pool, chunk);
	return err;
    }
    
    LockWrite(ctx->chunk_lock);
    ctx->nchunks++;
    if (ctx->current) {
	/* What to do with OOB items, with time earlier than current one?
	For Fast items can be assumed only one sequential data source? For items
	with multiple or/and non-sequential data sources OPCDataContext should be used
	if (opcTimeCmp(ctx->current->info.time, info->time)>0) {
	}
	*/
	ctx->current->next = chunk;
	ctx->current=chunk;

	if ((ctx->nchunks>ctx->cache_min)&&(!opcDataChunkGetRefs(ctx->chunks))) 
	    opcDataClear(ctx);
    } else {
	ctx->chunks = chunk;
	ctx->current = chunk;
    }
    LockReturn(ctx->chunk_lock);
    

    return 0;
}

int opcDataGetData(OPCDataContext ctx, unsigned long repid, OPCDataInfo *info, OPCRepresentationData *data) {
    int err;
    OPCRepresentationContext rep;
    OPCDataChunkContext current;

    assert(ctx);
    assert(ctx->current);
    assert(ctx->representation);

    if ((!ctx)||(repid>=ctx->supported_representations)) return ERR_OPC_PARAM; 
    if ((!ctx->current)||(!ctx->representation)) return ERR_OPC_DATA_NOTINITIALIZED;

    LockRead(ctx->chunk_lock);
    current = ctx->current;
    opcDataChunkRef(current);
    LockReturn(ctx->chunk_lock);
    
    if (data) {    
	LockRead(ctx->representation_lock);
	rep = ctx->representation[repid];
	if (rep) err = opcRepresentationGetData(rep, current, data);
	else err = ERR_OPC_DATA_DISABLED_REPRESENTATION;
	LockReturn(ctx->representation_lock);
    }

    if (!err) {
	if (info) *info=opcDataChunkGetInfo(current);
        opcDataRef(ctx);
    }
    
    return err;
}


void opcDataReturnData(OPCDataContext ctx, OPCDataInfo info) {
    assert(ctx);
    if ((!ctx)||(!info)) return;

    LockWrite(ctx->chunk_lock);
    if ((!opcDataChunkUnrefByInfo(info))&&(ctx->chunks == opcDataChunkGetChunkByInfo(info)))
	opcDataClear(ctx);
    LockReturn(ctx->chunk_lock);
    
    opcDataUnref(ctx);
}

void opcDataMantain(OPCDataContext ctx) {
    OPCRepresentationContext *representation;
    long i;

    assert(ctx);
    if ((!ctx)||(!ctx->representation)) return;
    
    LockRead(ctx->representation_lock);
    representation = ctx->representation;
    for (i=ctx->supported_representations;i>=0;i--) 
	if (representation[i]) opcRepresentationMantain(representation[i]);
    LockReturn(ctx->representation_lock);
}
