#ifndef _OPC_DATA_FAST_H
#define _OPC_DATA_FAST_H

#include <lock.h>
#include <opcdatabuffer.h>
#include <opcformats.h>
#include <opcformatnative.h>
#include <opcchunkpool.h>
#include <opcdatamanager.h>
#include <opcuserdata.h>
#include <opcrepresentation.h>

typedef struct OPCDataFastContextT *OPCDataFastContext;
typedef struct OPCDataFastOptionsT *OPCDataFastOptions;

struct OPCDataFastInfoT {
    OPCRepresentationInfoStr repinfo;
    OPCFormatsContext formats;
    OPCDataFastContext datafast;
};
typedef struct OPCDataFastInfoT *OPCDataFastInfo;
typedef struct OPCDataFastInfoT OPCDataFastInfoStr;

struct OPCDataFastContextT {
    struct OPCDataManagerContextT manager;
    OPCDataFastInfoStr repinfo;
    
    OPCMallocContext malloc;
    OPCDataBufferContext allocator;
    OPCFormatNativeContext format;
    OPCChunkPoolContext chunk_pool;
    OPCDataChunkContext chunks;
    OPCDataChunkContext current;

    OPCType type;

    LockContext lock;
    LockContext allocator_lock;
    LockContext chunk_lock;
    unsigned long refs;
    unsigned long nchunks;
    
    unsigned long buffer_min;
    unsigned long buffer_max;    
    
    unsigned char grabed_allocator;
};

struct OPCDataFastOptionsT {
    OPCDataManagerAcquireFunction acquire;
    OPCRepresentationGetUserDataFunction get;
    OPCFormatNativeContext format;
    OPCChunkPoolContext chunk_pool;
    unsigned long buffer_min;
    unsigned long buffer_max;
    OPCFormatsContext formats;
    struct OPCDataBufferOptionsT allocator;
};

OPCDataFastContext opcDataFastCreateContext(OPCMallocContext mctx);
int opcDataFastInitContext(OPCDataFastContext ctx, OPCType type, OPCDataFastOptions options);

OPCDataFastOptions opcDataFastGetDefaultOptions();
int opcDataFastInitDefaultOptions(OPCDataFastOptions options);
void opcDataFastDestroyOptions(OPCDataFastOptions options);
int opcDataFastOptionsSetAcquireFunction(OPCDataFastOptions options, OPCDataManagerAcquireFunction acquire);
int opcDataFastOptionsSetUserizeFunction(OPCDataFastOptions options, OPCRepresentationGetUserDataFunction get);
int opcDataFastOptionsSetBufferSize(OPCDataFastOptions options, unsigned long min, unsigned long max);
int opcDataFastOptionsSetCacheSize(OPCDataFastOptions options, unsigned long min, unsigned long max);


int opcDataFastGetData(OPCDataFastContext ctx, unsigned long repid, OPCDataInfo *info, OPCRepresentationData *data);
void opcDataFastReturnData(OPCDataFastContext ctx, OPCDataInfo info);
void opcDataFastMantain(OPCDataFastContext ctx);

int opcDataFastPushData(OPCDataFastContext ctx, unsigned long repid, OPCDataInfo info, const void *data, unsigned long size);
//int opcDataFastPushAllocation(OPCDataFastContext ctx, OPCDataInfo info);
OPCAllocatorContext opcDataFastGetAllocator(OPCDataFastContext ctx);
void opcDataFastReturnAllocator(OPCDataFastContext ctx);

int opcDataFastRef(OPCDataFastContext ctx);
void opcDataFastUnref(OPCDataFastContext ctx);

int opcDataFastSetAcquireFunction(OPCDataFastContext ctx, OPCDataManagerAcquireFunction acquire);
int opcDataFastSetUserizeFunction(OPCDataFastContext ctx, OPCRepresentationGetUserDataFunction get);

#ifdef _OPC_DATA_FAST_C
static const struct OPCDataManagerContextT opc_datamanager_fast = {
    NULL,							/* GetID */
    NULL,							/* Acquire (Costumized by user) */
    NULL,							/* PushData */
    NULL, 							/* PushAllocation */
    NULL,							/* GetAllocator */
    NULL,							/* ReturnAllocator */
    (OPCDataManagerGetDataFunction)opcDataFastGetData,		/* GetData */
    NULL,							/* GetDataFirst */
    NULL,							/* GetDataNext */
    (OPCDataManagerReturnDataFunction)opcDataFastReturnData,	/* ReturnData */
    (OPCDataManagerRefFunction)opcDataFastRef,			/* Ref */
    (OPCDataManagerUnrefFunction)opcDataFastUnref,		/* Unref */
    (OPCDataManagerMantainFunction)opcDataFastMantain		/* Mantain */
};

static const struct OPCDataFastOptionsT opc_datamanager_fast_options = {
    opcAcquireNotApplicable,
    NULL,
    NULL,
    NULL,
    1,								/* Buffer Min */
    0,								/* Buffer Max */
    NULL,
    {}
};

#endif /* _OPC_DATA_FAST_C */

#endif /* _OPC_DATA_FAST_H */
