#include <stdio.h>
#include <assert.h>

#include <opcerr.h>
#include <opcconfig.h>

#include <hashtable.h>
#include <opcstringlist.h>

#include <opctime.h>
#include <opctools.h>
#include <opctree.h>
#include <opcitem.h>

#include <opcinterface.h>


OPCInterface opcCreateInterface(OPCContext ctx) {
    OPCInterface interface;
    assert(ctx);
    
    interface = (OPCInterface)malloc(sizeof(struct OPCInterfaceT));
    if (!interface) return NULL;
    interface->sctx = ctx;

    interface->start_time = NULL;
    interface->locale = NULL;
    interface->current_locale = NULL;
    interface->last_error = 0;

    return interface;
}

void opcDestroyInterface(OPCInterface ctx) {
    assert(ctx);

    if (ctx->start_time) free(ctx->start_time);
    if (ctx->locale) opcLocaleDestroy(ctx->locale);
    free(ctx);
}


const char *opcInterfaceGetVersion(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    
    return opcVersionGet(ctx->sctx->version);
}

const char *opcInterfaceGetStartTime(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    
    ctx->start_time = opcTimeGetDateTime(ctx->sctx->start_time,NULL);
    return ctx->start_time;
}

const char *opcInterfaceSetLocale(OPCInterface ctx, char *attr) {
    assert(ctx);
    assert(ctx->sctx);
    
    ctx->locale=opcLocalesGetLocale(ctx->sctx->locales, attr);
    if (ctx->locale) {
	ctx->current_locale = ctx->locale->locale;
	return ctx->current_locale;
    } else {
	ctx->current_locale = opc_default_locale;
	return opc_default_locale;
    }
}

const char *opcInterfaceGetCharset(OPCInterface ctx) {
    assert(ctx);
    if (ctx->locale) return ctx->locale->charset;
    return NULL;
}

const lchar opcInterfaceGetStatusInfo(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->current_locale);
    return opcStateGetStatusInfo(ctx->sctx->state,ctx->current_locale);
}

const lchar opcInterfaceGetVendorInfo(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->current_locale);
    return opcStateGetVendorInfo(ctx->sctx->state,ctx->current_locale);
}

const lchar opcInterfaceGetStatus(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->current_locale);
    return opcStateGetStatus(ctx->sctx->state,ctx->current_locale);
}

int opcInterfaceListSupportedLocales(OPCInterface ctx, OPCLocalesListFunctionT list_function, void *attr) {
    assert(ctx);
    assert(ctx->sctx);
    return opcLocalesList(ctx->sctx->locales, list_function, attr);
}

int opcInterfaceListSupportedInterfaces(OPCInterface ctx, OPCVersionListFunctionT list_function, void *attr) {
    assert(ctx);
    assert(ctx->sctx);
    return opcVersionList(ctx->sctx->version, list_function, attr);
}

OPCConfigurationContext opcInterfaceGetConfiguration(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    return ctx->sctx->config;
}

OPCTreeContext opcInterfaceGetServerTree(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    return ctx->sctx->tree;
}

OPCFormatsContext opcInterfaceGetFormats(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    
    return ctx->sctx->formats;
}

OPCFormatContext opcInterfaceGetNativeFormat(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    assert(ctx->sctx->formats);

    return (OPCFormatContext)ctx->sctx->formats->native_format;
}

OPCFormatContext opcInterfaceGetOPCXMLFormat(OPCInterface ctx) {
    assert(ctx);
    assert(ctx->sctx);
    assert(ctx->sctx->formats);

    return (OPCFormatContext)ctx->sctx->formats->opcxml_format;
}

OPCItemContext opcInterfaceGetItem(OPCInterface ctx, const char *path, const char *name) {
    OPCTreeContext tree;
    OPCItemContext ictx;
    int size = 2;
    char *tmp;
    
    assert(ctx);
    tree = opcInterfaceGetServerTree(ctx);
    if (!tree) return NULL;
    

    if (name) size+=xmlStrlen(name);
    if (path) size+=xmlStrlen(path);

    tmp = (xmlChar*)malloc((size)*sizeof(char));
    if (!tmp) return NULL;
    
    if ((name)&&(path)) {
	if (name[0]=='/') sprintf(tmp,"%s%s",path,name);
	else sprintf(tmp,"%s/%s",path,name);
    } else if (name) {
	if ((name[0]=='/')||(name[0]==0)) tmp=(char*)name;
	else {
	    sprintf(tmp,"/%s",name);
	}
    } else if (path) tmp=(char*)path;
    else return NULL;

    ictx = opcTreeGetItem(tree,tmp);
    if ((tmp!=name)&&(tmp!=path)) free(tmp);

    return ictx;
}

void opcInterfaceReturnItem(OPCInterface ctx, OPCItemContext ictx) {
    assert(ictx);
    opcItemReturn(ictx);
}
