#ifndef _OPC_ITEM_H
#define _OPC_ITEM_H

typedef enum OPCItemBrowseFilterT OPCItemBrowseFilter;
enum OPCItemBrowseFilterT {
    ALL = 0,
    BRANCH,
    ITEM
};

typedef struct OPCItemContextT *OPCItemContext;

#include <lock.h>
#include <condition.h>

#include <opctype.h>

#include <opcdatamanager.h>
#include <opcdata.h>
#include <opcdatafast.h>

#include <opcstringlist.h>
//#include <opcproperty.h>
#include <opctree.h>
#include <opctime.h>
#include <opcquality.h>
#include <lchar.h>

#define OPC_ITEM_LINKED_FLAG 1
#define OPC_ITEM_CONNECTED_FLAG 2

typedef int (*OPCVendorFilter)(OPCItemContext, const char *);



struct OPCItemContextT {
    char *fullname;
    char *name;
    lchar title;
    
    OPCMallocContext malloc;

    LockContext lock;
    ConditionContext client_cond;
    
    OPCStringList subitems;
    
    OPCTreeContext tree;

    OPCVendorFilter vendor_filter;

    struct OPCTimeT timestamp;

    OPCDataManagerContext dm;
    
    unsigned long clients;
    
    unsigned short representation;

    unsigned char status;	/*  1 Bit: Linked in tree,
				    2 Bit: Real Item */
			

    unsigned char manual;	/* By default all items in automatic mode, this means
				if parrent item are unregistered???? */
    
    unsigned char mark;		/* Marked from deletion. If in FreeContext this flag set then
				Item is locked and alredy unlinked from tree */
};

// Locking functions
OPCItemContext opcItemCreateContext(OPCMallocContext mctx);
int opcItemInitContext(OPCItemContext ctx, const char *fullname, const char *title);
int opcItemFreeContext(OPCItemContext ctx);
void opcItemDestroyContext(OPCItemContext ctx);

int opcItemLock(OPCItemContext ctx);
int opcItemUnlock(OPCItemContext ctx);
int opcItemGet(OPCItemContext ctx); /* Should be called only inderect truth opcTreeGetItem */
int opcItemReturn(OPCItemContext ctx);

int opcItemSetAssociation(OPCItemContext ctx);
int opcItemSetTitle(OPCItemContext ctx, const char *title);
int opcItemSetManualMode(OPCItemContext ctx);
int opcItemResetManualMode(OPCItemContext ctx);
int opcItemSetVendorFilter(OPCItemContext ctx, OPCVendorFilter vendor_filter);

char *opcItemGetFullName(OPCItemContext ctx);
int opcItemBrowse(OPCItemContext ctx, OPCItemBrowseFilter browse_filter, const char *name_filter, const char *vendor_filter, OPCStringList *result);

// Non-Locking functions. Should be locked from caller.

int opcItemHaveChilds(OPCItemContext ctx);
int opcItemIsConnected(OPCItemContext ctx);
const char *opcItemGetTitle(OPCItemContext ctx, const char *locale);

// Internal functions called from opcTree & opcItem and synchronised from caller functions 
int opcItemBrowseFilter(OPCItemContext ctx, OPCItemBrowseFilter filter);
int opcItemNameFilter(OPCItemContext ctx, const char *filter);

int opcItemLink(OPCItemContext ctx, OPCItemContext ictx);
int opcItemUnlink(OPCItemContext ctx, OPCItemContext ictx);

int opcItemSetTree(OPCItemContext ctx, OPCTreeContext tree);
int opcItemHaveManualSubChilds(OPCItemContext ctx);

void opcItemMark(OPCItemContext ctx);
void opcItemMarkSubTree(OPCItemContext ctx);
void opcItemUnMark(OPCItemContext ctx);
void opcItemUnMarkSubTree(OPCItemContext ctx);
int opcItemIsMarked(OPCItemContext ctx);

int opcItemMarkChildsCallback(OPCItemContext ctx, OPCItemContext ictx);
int opcItemMarkParrentsCallback(OPCItemContext ctx, OPCItemContext ictx);

//int opcItemSetDataType(OPCItemContext ctx, OPCType type);
//int opcItemCheckConnection(OPCItemContext ctx);

OPCDataFastContext opcItemRegisterFastDataManager(OPCItemContext ctx, OPCType type, OPCDataFastOptions options, unsigned long flags);
OPCDataContext opcItemRegisterLegacyDataManager(OPCItemContext ctx, OPCType type, OPCDataOptions options, unsigned long flags);
OPCDataManagerContext opcItemRegisterDataManager(OPCItemContext ctx, OPCDataManagerContext dm);
void opcItemRemoveDataManager(OPCItemContext ctx);

void opcItemEnableDataManager(OPCItemContext ctx);
void opcItemDisableDataManager(OPCItemContext ctx);

/*
int opcItemInitNativeDataContext(OPCItemContext ctx, OPCTime time, OPCQuality quality, void *data);

int opcItemPushNativeData(OPCItemContext ctx, OPCTime time, OPCQuality quality, void *data);
int opcItemGetOPCXMLData(OPCItemContext ctx, OPCDataInfo *info, OPCData *data);
*/
const OPCTime opcItemGetTimestamp(OPCItemContext ctx);

int opcItemGetData(OPCItemContext ctx, OPCDataInfo *info, OPCRepresentationData *data);
void opcItemReturnData(OPCItemContext ctx, OPCDataInfo info);


// Properties
//int opcItemRegisterProperty(OPCItemContext ctx, "propertyname");
//int opcItemRegisterNewProperty(OPCItemContext ctx, 
// int opcItemRegisterStandardFormats(OPCItemContext ctx)
//int opcItemRegisterFormat(OPCItemContext ctx, 
//


#endif /* _OPC_ITEM_H */
