#include <opcerr.h>
#include <opcassert.h>
#include <opcdebug.h>

#include <opcproperty.h>


void opcPropertyZeroContext(OPCPropertyContext ctx) {
    assert(ctx);
    ctx->id=0;    /* 0 - property by default (unconnected) */
    ctx->name=NULL;
    ctx->description=NULL;
    ctx->xmlnamespace=NULL;
}

OPCPropertyContext opcPropertyCreateContext(OPCMallocContext mctx) {
    OPCPropertyContext ctx;

    ctx = (OPCPropertyContext)opcMalloc(mctx,sizeof(struct OPCPropertyContextT));
    if (ctx) {
	ctx->malloc=mctx;
	opcPropertyZeroContext(ctx);
    }

    opcDebugServer(OPC_DEBUG_DEBUG, "opcPropertyCreateContext(%p)", ctx);    

    return ctx;
}


int opcPropertyInitContext(OPCPropertyContext ctx, const char *name,  OPCStandardType datatype, const void *attr) {
    int err;
    OPCType type;
    
    assert(ctx);
    assert(name);

    opcDebugServer(OPC_DEBUG_NOTICE, "opcPropertyInitContext(%p): %s", ctx, name);    

    type = opcStandardType(ctx->malloc, datatype, attr);
    if (!type) return ERR_OPC_STANDARDTYPE_CREATE;

    err = opcPropertyInitContextByType(ctx, name, type);
    if (err) opcTypeDestroy(type);
    return err;
}


int opcPropertyInitContextByType(OPCPropertyContext ctx, const char *name, OPCType type) {
    assert(ctx);
    assert(name);
    

    opcPropertyZeroContext(ctx);

    if (opcExchangeString(ctx->malloc,&ctx->name,name)) return ERR_OPC_MEM;
    ctx->type = type;
    
    opcDebugServer(OPC_DEBUG_DEBUG, "opcPropertyInitContext(%p): %s Ok", ctx, name);    
    return 0;
}

void opcPropertyFreeContext(OPCPropertyContext ctx) {
    opcDebugServer(OPC_DEBUG_DEBUG, "opcPropertyFreeContext(%p)", ctx);    
    assert(ctx);
    if (ctx->name) opcExchangeString(ctx->malloc,&ctx->name,NULL);
    if (ctx->xmlnamespace) opcExchangeString(ctx->malloc,&ctx->xmlnamespace,NULL);
}

void opcPropertyDestroyContext(OPCPropertyContext ctx) {
    opcDebugServer(OPC_DEBUG_DEBUG, "opcPropertyDestroyContext(%p)", ctx);    
    opcPropertyFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}

int opcPropertySetNamespace(OPCPropertyContext ctx, const char *xmlnamespace) {
    assert(ctx);
    if (opcExchangeString(ctx->malloc,&ctx->xmlnamespace,xmlnamespace)) return ERR_OPC_MEM;
    return 0;
}

int opcPropertySetId(OPCPropertyContext ctx, unsigned long id) {
    assert(ctx);
    ctx->id = id;    
}

int opcPropertySetDescription(OPCPropertyContext ctx, const char *description) {
    assert(ctx);
    if (opcLCharExchangeString(ctx->malloc, &ctx->description, description)) return ERR_OPC_MEM;
    return 0;
}


//opcPropertyCreateInstance()

//opcPropertyDestroyInstance()
