#ifndef _OPC_QUALITY_H
#define _OPC_QUALITY_H

#include <opclib.h>
#include <opcmalloc.h>

typedef struct OPCQualityT *OPCQuality;
typedef enum OPCQualityFieldT OPCQualityField;
typedef enum OPCLimitFieldT OPCLimitField;

enum OPCQualityFieldT {
    OPC_ITEM_QUALITY_GOOD = 0,
    OPC_ITEM_QUALITY_GOOD_LOCAL_OVERRIDE,
    OPC_ITEM_QUALITY_UNCERTAIN,
    OPC_ITEM_QUALITY_UNCERTAIN_LAST_USABLE_VALUE,
    OPC_ITEM_QUALITY_UNCERTAIN_SENSOR_NOT_ACCURATE,
    OPC_ITEM_QUALITY_UNCERTAIN_EU_EXCEEDED,
    OPC_ITEM_QUALITY_UNCERTAIN_SUB_NORMAL,
    OPC_ITEM_QUALITY_BAD,
    OPC_ITEM_QUALITY_BAD_CONFIGURATION_ERROR,
    OPC_ITEM_QUALITY_BAD_NOT_CONNECTED,
    OPC_ITEM_QUALITY_BAD_DEVICE_FAILURE,
    OPC_ITEM_QUALITY_BAD_SENSOR_FAILURE,
    OPC_ITEM_QUALITY_BAD_LAST_KNOWN_VALUE,
    OPC_ITEM_QUALITY_BAD_COMM_FAILURE,
    OPC_ITEM_QUALITY_BAD_OUT_OF_SERVICE
};

enum OPCLimitFieldT {
    OPC_ITEM_LIMITSTATUS_NONE = 0,
    OPC_ITEM_LIMITSTATUS_LOW,
    OPC_ITEM_LIMITSTATUS_HIGH,
    OPC_ITEM_LIMITSTATUS_CONSTANT
};

struct OPCQualityT {
    OPCQualityField quality;
    OPCLimitField limit;
    uint8 vendor;
};


OPCQuality opcQualityCreate(OPCMallocContext mctx, OPCQualityField quality, OPCLimitField limit, uint8 vendor);
int opcQualitySet(OPCQuality ctx, OPCQualityField quality, OPCLimitField limit, uint8 vendor);
int opcQualityCopy(OPCQuality dst, OPCQuality src);


#ifdef _OPC_QUALITY_C
struct OPCQualityT OPCQualityGoodT = {
    OPC_ITEM_QUALITY_GOOD,
    OPC_ITEM_LIMITSTATUS_NONE,
    0
};

struct OPCQualityT OPCQualityUncertainT = {
    OPC_ITEM_QUALITY_UNCERTAIN,
    OPC_ITEM_LIMITSTATUS_NONE,
    0
};

struct OPCQualityT OPCQualityBadT = {
    OPC_ITEM_QUALITY_BAD,
    OPC_ITEM_LIMITSTATUS_NONE,
    0
};

OPCQuality OPCQualityGood = &OPCQualityGoodT;
OPCQuality OPCQualityUncertain = &OPCQualityUncertainT;
OPCQuality OPCQualityBad = &OPCQualityBadT;
#endif /* _OPC_QUALITY_C */

extern OPCQuality OPCQualityGood;
extern OPCQuality OPCQualityUncertain;
extern OPCQuality OPCQualityBad;


#endif /* _OPC_QUALITY_H */
