#include <stdio.h>
#include <assert.h>

#include <opcerr.h>
#include <opcconfig.h>

#include <hashtable.h>
#include <opcstringlist.h>

#include <lchar.h>
#include <opctime.h>
#include <opcstate.h>
#include <opctools.h>
#include <opclocale.h>
#include <opctree.h>

#include <opcserverinterface.h>

int opcServerStartTimeSet(OPCContext ctx) {
    assert(ctx);
    
    if (!ctx->start_time) {
	ctx->start_time=opcTimeCreate(ctx->malloc);
	if (!ctx->start_time) return ERR_OPC_MEM;
    }
    opcTimeCurrentTime(ctx->start_time);
    return 0;
}

int opcServerStateSet(OPCContext ctx, unsigned int state, const char *status_str) {
    int err;
    assert(ctx);
    
    if (!ctx->state) {
	ctx->state=opcStateCreate(ctx->malloc);
	if (!ctx->state) return ERR_OPC_MEM;
	err=opcStateInit(ctx->state,ctx);
	if (err) return err;
    }
    return opcStateSetStatus(ctx->state,state,status_str);
}

int opcServerVendorInfoSet(OPCContext ctx, const char *vi) {
    int err;
    assert(ctx);

    if (!ctx->state) {
	ctx->state=opcStateCreate(ctx->malloc);
	if (!ctx->state) return ERR_OPC_MEM;
	err=opcStateInit(ctx->state,ctx);
	if (err) return err;
    }
    
    return opcStateSetVendor(ctx->state,vi);
}

int opcServerVersionSet(OPCContext ctx, const char *major, const char *minor, unsigned long build) {
    int err=0;
    int i=0;
    
    assert(ctx);
    
    if (!ctx->version) {
        ctx->version=opcVersionInit(ctx->malloc,ctx);
	if (!ctx->version) return ERR_OPC_MEM;
    } else {
	opcVersionInterfacesClear(ctx->version);
    }
    return opcVersionSet(ctx->version,major,minor,build);
}

int opcServerInterfaceAdd(OPCContext ctx, const char *interface, const char **extensions) {
    opcVersionInterfaceAdd(ctx->version,interface,extensions);
}


int opcServerLocalesAdd(OPCContext ctx, const char **locales) {
    unsigned long i;
    int err;
    
    assert(ctx);
    
    if (!ctx->locales) {
	ctx->locales=opcLocalesCreateContext(ctx->malloc);
	if (!ctx->locales) return ERR_OPC_MEM;
	err=opcLocalesInitContext(ctx->locales);
	if (err) return err;
    }
    
    return opcLocalesAddLocales(ctx->locales,locales);
}

int opcServerInitTree(OPCContext ctx) {
    int err;
    OPCTreeContext tree;
    assert(ctx);
    
    tree = opcTreeCreateContext(ctx->malloc);
    if (!tree) return ERR_OPC_MEM;
    
    err = opcTreeInitContext(tree,ctx);
    if (err) opcTreeDestroyContext(tree);
    else ctx->tree=tree;
    return err;
}


int opcServerRegisterItem(OPCContext ctx, OPCItemContext *ictx, const char *path, const char *name) {
    int err;
    OPCItemContext item;
    char *fullname, *title;
    unsigned long i;
    unsigned char flag;
    char *fp;
        
    assert(ctx);
    assert(ctx->tree);
    assert(name);
    
    if (path) {
	for (i=strlen(path);((i>0)&&(path[i-1]=='/'));i--) flag=1;
    } else {
	flag=1;
	i=0;
    }
    
    if (flag) {
	fp = (char*)malloc((i+1)*sizeof(char));
	if (!fp) return ERR_OPC_MEM;
	strncpy(fp,path,i);
	fp[i]=0;
	path=fp;
    } 

     
    item=opcItemCreateContext(ctx->malloc);
    fullname=(char*)malloc((i+strlen(name)+1)*sizeof(char));
    if ((!fullname)||(!item)) {
	if (fullname) free(fullname);
	if (item) opcItemDestroyContext(item);
	if (flag) free(path);
	return ERR_OPC_ITEM_INIT;
    }

    sprintf(fullname,"%s/%s",path,name);
    title = fullname + i + 1;

    err=opcItemInitContext(item, fullname, title);
    free(fullname);
    
    if (!err) err=opcTreeRegisterItem(ctx->tree,item,path);
    if (flag) free(path);
    
    if (!err) *ictx=item;
    else {
	opcItemDestroyContext(item);	
	*ictx=NULL;
    }
    return err;
}

int opcServerDestroyItem(OPCContext ctx, OPCItemContext ictx) {
    assert(ctx);
    assert(ictx);
    
    opcItemDestroyContext(ictx);
    return 0;
}

