#ifndef _OPC_STANDARD_PROPERTIES_H
#define _OPC_STANDARD_PROPERTIES_H

#include <opcproperties.h>
#include <opctype.h>
#include <opcconfig.h>

typedef struct OPCStandardPropertyT *OPCStandardProperty;

struct OPCStandardPropertyT {
    char *name;
    char *xmlnamespace;
    unsigned short int id;
    OPCSimpleType type;
    void *attr;				/* Enumerations, OPCBinary */
    char *description;
};

int opcPropertiesRegisterStandard(OPCPropertiesContext ctx);

#ifdef _OPC_STANDARD_PROPERTIES_C
static const char *opc_property_enumeration_quality[] = { 
    "bad", "badConfigurationError", "badNotConnected", 
    "badDeviceFailure", "badSensorFailure", "badLastKnownValue", 
    "badCommFailure", "badOutOfService", "uncertain",
    "uncertainLastUsableValue", "uncertainSensorNotAccurate", "uncertainEUExceeded",
    "uncertainSubNormal", "good", "goodLocalOverride", NULL
};


static const char *opc_property_enumeration_limitstatus[] = {
    "none","low","high","constant", NULL
};


static const char * opc_property_enumeration_accessrights[] = {
    "unknown", "readable", "writable", "readWritable", NULL
};

static const char * opc_property_enumeration_eutype[] = {
    "noEnum","analog","enumerated",NULL
};

static struct OPCStandardPropertyT opc_property[] = {
    {"dataType",NULL,1,OPC_DATATYPE_QNAME,NULL,lgs("")},
    {"value",NULL,2,0,NULL,lgs("")},
    {"quality",NULL,3,OPC_DATATYPE_ENUMERATION,opc_property_enumeration_quality,lgs("")},
    {"timestamp",NULL,4,OPC_DATATYPE_DATETIME,NULL,lgs("")},
    {"accessRights",NULL,5,OPC_DATATYPE_ENUMERATION,opc_property_enumeration_accessrights,lgs("")},
    {"scanRate",NULL,6,OPC_DATATYPE_FLOAT,NULL,lgs("")},
    {"euType",NULL,7,OPC_DATATYPE_ENUMERATION,opc_property_enumeration_eutype,lgs("")},
    {"euInfo",NULL,8,OPC_DATATYPE_STRINGLIST,NULL,lgs("")},
    {"engineeringUnits",NULL,100,OPC_DATATYPE_STRING,NULL,lgs("")},
    {"description",NULL,101,OPC_DATATYPE_STRING,NULL,lgs("")},
    {"highEU",NULL,102,OPC_DATATYPE_DOUBLE,NULL,lgs("")},
    {"lowEU",NULL,103,OPC_DATATYPE_DOUBLE,NULL,lgs("")},
    {"highIR",NULL,104,OPC_DATATYPE_DOUBLE,NULL,lgs("")},
    {"lowIR",NULL,105,OPC_DATATYPE_DOUBLE,NULL,lgs("")},
    {"closeLabel",NULL,106,OPC_DATATYPE_STRING,NULL,lgs("")},
    {"openLabel",NULL,107,OPC_DATATYPE_STRING,NULL,lgs("")},
    {"timeZone",NULL,108,OPC_DATATYPE_UNSIGNED,NULL,lgs("")},
    {NULL,NULL,0,0,NULL,NULL}
};
#endif /* _OPC_STANDARD_PROPERTIES_C */

#endif /* _OPC_STANDARD_PROPERTIES_H */

