#define _OPC_STATE_C

#include <stdio.h>
#include <assert.h>

#include <dsgnuintl.h>

#include <opcerr.h>
#include <opcconfig.h>
#include <opcdebug.h>

#include <opcmalloc.h>

#include <opc.h>
#include <lchar.h>
#include <opcstate.h>

static void opcStateZero(OPCState ctx) {
    assert(ctx);
    ctx->lock = NULL;
    ctx->status_info = NULL;
    ctx->vendor_info = NULL;
}

OPCState opcStateCreate(OPCMallocContext mctx) {
    OPCState state;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcStateCreateContext");    

    state = (OPCState)opcMalloc(mctx,sizeof(struct OPCStateT));
    if (state) {
	state->malloc=mctx;
	opcStateZero(state);
    }
    return state;
}

int opcStateInit(OPCState ctx, OPCContext sctx) {
    int err = 0;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcStateInitContext");    

    assert(ctx);
    
    ctx->lock = LockCreateContext(ctx->malloc);
    if (!ctx->lock) return ERR_OPC_MEM;
    err = LockInitContext(ctx->lock);
    if (!err) err=opcLCharExchangeString(ctx->malloc,&ctx->vendor_info,"");
    if (!err) err=opcLCharExchangeString(ctx->malloc,&ctx->status_info,"");
    if (err) opcStateFree(ctx);
    return err;
}

void opcStateFree(OPCState ctx) {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcStateFreeContext");    

    assert(ctx);
    if (ctx->vendor_info) opcLCharExchangeString(ctx->malloc,&ctx->vendor_info,NULL);
    if (ctx->status_info) opcLCharExchangeString(ctx->malloc,&ctx->status_info,NULL);
    if (ctx->lock) LockDestroyContext(ctx->lock);
    opcStateZero(ctx);
}

void opcStateDestroy(OPCState ctx) {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcStateDestroyContext");    

    assert(ctx);
    opcStateFree(ctx);
    opcFree(ctx->malloc,ctx);
}

int opcStateSetStatus(OPCState ctx, unsigned int state, const char *status) {
    int err = 0;
    
    assert(ctx);
    assert(ctx->lock);
    
    LockWrite(ctx->lock);
    if (state>OPC_STATE_LASTSTATE) err=ERR_OPC_I_INVALIDSTATE;
    if (!err) {
	ctx->state=state;
	err=opcLCharExchangeString(ctx->malloc,&ctx->status_info,status);
    }
    LockReturn(ctx->lock);
    return err;
}

int opcStateSetVendor(OPCState ctx, const char *vendor) {
    int err;

    assert(ctx);
    assert(ctx->lock);
    
    LockWrite(ctx->lock);
    err=opcLCharExchangeString(ctx->malloc,&ctx->vendor_info,vendor);
    LockReturn(ctx->lock);
    return err;
}


const lchar opcStateGetStatusInfo(OPCState ctx, const char *locale) {
    char *tmp;
    
    assert(ctx);
    assert(ctx->lock);
    assert(ctx->status_info);
    assert(locale);
    
    LockRead(ctx->lock);
    tmp=dsgettext(ctx->status_info,locale);
    LockReturn(ctx->lock);
    return tmp;
}

const lchar opcStateGetVendorInfo(OPCState ctx, const char *locale) {
    char *tmp;

    assert(ctx);
    assert(ctx->lock);
    assert(ctx->vendor_info);
    assert(locale);

    LockRead(ctx->lock);
    tmp=dsgettext(ctx->vendor_info,locale);
    LockReturn(ctx->lock);
    return tmp;
}

const lchar opcStateGetStatus(OPCState ctx, const char *locale) {
    assert(ctx);
    return opc_states[ctx->state];
}
