#include <stdio.h>
#include <time.h>
#include <assert.h>

#include <opcmalloc.h>

#include <opctime.h>



OPCTime opcTimeCreate(OPCMallocContext mctx) {
    OPCTime time;
    
    time = (OPCTime)opcMalloc(mctx, sizeof(struct OPCTimeT));
    if (!time) return NULL;
    time->malloc=mctx;
    return time;
}

void opcTimeDestroy(OPCTime time) {
    assert(time);
    
    opcFree(time->malloc, time);
}

OPCDuration opcDurationCreate(OPCMallocContext mctx) {
    OPCDuration duration;
    
    duration = (OPCDuration)opcMalloc(mctx, sizeof(struct OPCDurationT));
    if (!duration) return NULL;
    duration->malloc=mctx;
    return duration;
}

void opcDurationDestroy(OPCDuration duration) {
    assert(duration);
    opcFree(duration->malloc, duration);
}
OPCTime opcTimeCurrentTime(OPCTime time) {
    struct tm *t;

    assert(time);
        
    gettimeofday((struct timeval*)time,NULL);
    t = localtime(&time->s);
    time->offset=t->tm_gmtoff;
    return time;
}

OPCTime opcTimeImportUnixTime(OPCTime time, struct timeval *newtime) {
    struct tm *t;
    
    memcpy(time, newtime, sizeof(struct timeval));
    t = localtime(&time->s);
    time->offset=t->tm_gmtoff;
    return time;
}

OPCDuration opcDurationImportMS(OPCDuration duration, unsigned long ms) {
    assert(duration);
    
    duration->s = ms/1000;
    duration->us = (ms-1000*duration->s)*1000;
    return duration;
}

OPCTime opcTimeSetOffset(OPCTime time, long offset) {
    time->offset = offset;
}

int opcTimeCopy(OPCTime dst, OPCTime src) {
    assert(dst);
    assert(src);
    
    memcpy(dst, src, sizeof(struct OPCTimeT));
    return 0;
}

char *opcTimeGetTime(OPCTime time, char *buffer) {
    struct tm *t;
    int hoffset, moffset;
    char *str;
    
    assert(time);
    
    if (buffer) str=buffer;
    else {
	str=(char*)malloc(23*sizeof(char));
	if (!str) return NULL;
    }
    
    t = localtime(&time->s);
    
    sprintf(str,"%02u:%02u:%02u.%07u",
	t->tm_hour,t->tm_min,t->tm_sec,
	time->us,time->offset/3600);

    if (time->offset==0)
	strcpy(str+27,"Z");
    else {
	hoffset=time->offset/3600;
	moffset=abs(time->offset/60-hoffset*60);
	sprintf(str+27,"%+02i:%02u",hoffset,moffset);
    }
    
    return str;
}

char *opcTimeGetDate(OPCTime time, char *buffer) {
    struct tm *t;
    int hoffset, moffset;
    char *str;
    
    assert(time);
    
    if (buffer) str=buffer;
    else {
	str=(char*)malloc(11*sizeof(char));
	if (!str) return NULL;
    }
    
    t = localtime(&time->s);
    
    sprintf(str,"%04u-%02u-%02u",
	t->tm_year+1900,t->tm_mon+1,t->tm_mday);
    
    return str;

}

char *opcTimeGetDateTime(OPCTime time, char *buffer) {
    struct tm *t;
    int hoffset, moffset;
    char *str;
    
    assert(time);
    
    if (buffer) str=buffer;
    else {
	str=(char*)opcMalloc(NULL, 34*sizeof(char));
	if (!str) return NULL;
    }
    
    t = localtime(&time->s);
    
    sprintf(str,"%04u-%02u-%02uT%02u:%02u:%02u.%07u",
	t->tm_year+1900,t->tm_mon+1,t->tm_mday,
	t->tm_hour,t->tm_min,t->tm_sec,
	time->us*10,time->offset/3600);

    if (time->offset==0)
	strcpy(str+27,"Z");
    else {
	hoffset=time->offset/3600;
	moffset=abs(time->offset/60-hoffset*60);
	sprintf(str+27,"%+02i:%02u",hoffset,moffset);
    }
    
    return str;
}

OPCTime opcTimeSetDateTime(OPCTime time, const char *buffer) {
    char *ch, chs;
    char *tmp = (char*)buffer;
    struct tm tms;
    time_t result;
    unsigned long day,month,year;
    unsigned long hour,minute,second;
    unsigned long us;
    long offset;
    unsigned char uslen;
    
    assert(time);
    assert(buffer);
    
    ch = strchr(tmp,'-');
    if (!ch) return NULL;
    *ch=0;year = atol(tmp);*ch='-';
    tmp=ch+1;

    ch = strchr(tmp,'-');
    if (!ch) return NULL;
    *ch=0;month = atol(tmp);*ch='-';
    tmp=ch+1;

    ch = strchr(tmp,'T');
    if (!ch) return NULL;
    *ch=0;day = atol(tmp);*ch='T';
    tmp=ch+1;

    ch = strchr(tmp,':');
    if (!ch) return NULL;
    *ch=0;hour = atol(tmp);*ch=':';
    tmp=ch+1;

    ch = strchr(tmp,':');
    if (!ch) return NULL;
    *ch=0;minute = atol(tmp);*ch=':';
    tmp=ch+1;


    ch = strchr(tmp,'.');
    if (!ch) ch = strchr(tmp,'Z');
    if (!ch) ch = strchr(tmp,'+');
    if (!ch) ch = strchr(tmp,'-');

    chs=*ch;
    if (chs) *ch=0;
    second = atol(tmp);
    if (chs) *ch=chs;
    tmp=ch+1;
    
    if (chs=='.') {
	ch = strchr(tmp,'Z');
	if (!ch) ch = strchr(tmp,'+');
	if (!ch) ch = strchr(tmp,'-');
	chs=*ch;

	if (chs) *ch=0;
	uslen = strlen(tmp);
	if (uslen>6) {
	    uslen=tmp[6];
	    tmp[6]=0;
	    us = atol(tmp);
	    tmp[6]=uslen;
	} else {
	    us = atol(tmp);
	    for (uslen=6-uslen;uslen>0;uslen--) us *= 10;
	}
	if (chs) *ch=chs;
	tmp=ch+1;
    } else us=0;
    
    if ((chs)&&(chs!='Z'))  {
	ch = strchr(tmp,':');
	if (!ch) return NULL;
	*ch=0;offset=atol(tmp);*ch=':';
	tmp=ch+1;

	if (chs=='+') offset = (offset*60+atol(tmp))*60;
	else offset = -(offset*60+atol(tmp))*60;
    } else offset=0;
    
    tms.tm_sec = second;
    tms.tm_min = minute;
    tms.tm_hour = hour;
    tms.tm_mday = day;
    if (month>0) tms.tm_mon = month-1;
    else return NULL;
    if (year>1899) tms.tm_year = year-1900;
    else tms.tm_year = year;
    tms.tm_isdst = -1;
    
    result=mktime(&tms);
    if (result==(time_t)-1) return NULL;
    
    time->s = result;
    time->us = us;
    time->offset = offset;

/*
    printf("%u-%u-%u\n",year,month,day);
    printf("%u:%u:%u.%u\n",hour,minute,second,us);
    printf("%u\n",offset);
*/

    return time;
}

/*
  Format: PxYxMxDTxHxMxS,
    any of YMDHMS can be omited, if HMS specified T must presist,
    x can be any number, i.e. 120S - correct
*/

//ToDo: offsets
char *opcTimeGetDuration(OPCTime stime, OPCTime etime, char *buffer) {
    unsigned long s,us;
    char *str;
    
    assert(stime);
    assert(etime);
    
    if (buffer) str=buffer;
    else {
	str=(char*)malloc(34*sizeof(char));
	if (!str) return NULL;
    }
    
    if (etime->s>stime->s) {
	s=etime->s-stime->s;
	if (etime->us>stime->us) us=etime->us-stime->us;
	else {
	    s--;
	    us=etime->us-stime->us+1000000;
	}
	sprintf(str,"P%02u.%07u",s,us);
    } else {
	s=stime->s-etime->s;
	if (etime->us<stime->us) us=stime->us-etime->us;
	else {
	    s--;
	    us=stime->us-etime->us+1000000;
	}
	sprintf(str,"-P%02u.%07u",s,us);
    }
    
    return str;    
}

OPCDuration opcTimeCalcDuration(OPCDuration duration, OPCTime stime, OPCTime etime) {
    long diff;
    
    assert(duration);
    assert(stime);
    assert(etime);
    
    diff = stime->offset - etime->offset;
    
    if ((etime->s+diff)<stime->s) {
	duration->s=0;
	duration->us=0;
    } else {
	duration->s=diff+etime->s-stime->s;
	if (etime->us>stime->us) duration->us=etime->us-stime->us;
	else {
	    duration->s--;
	    duration->us=etime->us-stime->us+1000000;
	}
    }
    return duration;
}

OPCTime opcTimeAdd(OPCTime time, OPCDuration duration) {
    assert(time);
    assert(duration);
    
    time->s+=duration->s;
    time->us+=duration->us;
    if (time->us>999999) {
	time->s++;
	time->us-=1000000;
    }
    return time;
}

OPCTime opcTimeAddSeconds(OPCTime time, time_t sec) {
    assert(time);
    
    time->s+=sec;
    return time;
}

OPCTime opcTimeSub(OPCTime time, OPCDuration duration) {
    assert(time);
    assert(duration);
    
    time->s-=duration->s;
    
    if (time->us<duration->us) {
	time->s--;
	time->us+=1000000-duration->us;
    } else {
	time->us-=duration->us;
    }
	
    return time;
}

int opcTimeCmp(OPCTime time1, OPCTime time2) {
    long diff;
    
    assert(time1);
    assert(time2);
    
    diff = time1->offset - time2->offset;
    
    if (time1->s>(time2->s+diff)) return 1;
    else if (time1->s<(time2->s+diff)) return -1;
    else {
	if (time1->us>time2->us) return 1;
	else if (time1->us<time2->us) return -1;
    }
    return 0;
}

int opcDurationCmp(OPCDuration duration1, OPCDuration duration2) {
    assert(duration1);
    assert(duration2);
    
    if (duration1->s > duration2->s) return 1;
    else if (duration1->s < duration2->s) return -1;
    else {
	if (duration1->us > duration2->us) return 1;
	else if (duration2->us < duration2->us) return -1;
    }
    return 0;
}
