#ifndef _OPC_TIME_H
#define _OPC_TIME_H

#include <opcmalloc.h>
#include <sys/time.h>
#include <time.h>

typedef struct OPCDurationT *OPCDuration;
typedef struct OPCTimeT *OPCTime;

struct OPCDurationT {
	// This part of structure must correspond to system "struct timeval"
    time_t s;
    long us;

	// Additional Information
    OPCMallocContext malloc;
};

struct OPCTimeT {
	// This part of structure must correspond to system "struct timeval"
    time_t s;
    long us;
    
	// Additional information
    unsigned long offset;
    OPCMallocContext malloc;
};


OPCTime opcTimeCreate(OPCMallocContext mctx);
void opcTimeDestroy(OPCTime time);
OPCDuration opcDurationCreate(OPCMallocContext mctx);
void opcDurationDestroy(OPCDuration duration);

OPCTime opcTimeCurrentTime(OPCTime time);
OPCTime opcTimeSetOffset(OPCTime time, long offset);

OPCTime opcTimeImportUnixTime(OPCTime time, struct timeval *newtime);
OPCDuration opcDurationImportMS(OPCDuration duration, unsigned long ms);

OPCDuration opcTimeCalcDuration(OPCDuration duration, OPCTime stime, OPCTime etime);
OPCTime opcTimeAdd(OPCTime time, OPCDuration duration);
OPCTime opcTimeAddSeconds(OPCTime time, time_t sec);
OPCTime opcTimeSub(OPCTime time, OPCDuration duration);

int opcTimeCopy(OPCTime dst, OPCTime src);


OPCTime opcTimeSetDateTime(OPCTime time, const char *buffer);
char *opcTimeGetDateTime(OPCTime time, char *buffer);
char *opcTimeGetDate(OPCTime time, char *buffer);
char *opcTimeGetTime(OPCTime time, char *buffer);
char *opcTimeGetDuration(OPCTime stime, OPCTime etime, char *buffer);

int opcTimeCmp(OPCTime time1, OPCTime time2);
int opcDurationCmp(OPCDuration duration1, OPCDuration duration2);


#endif /* _OPC_TIME_H */
