#ifndef _OPC_TREE_H
#define _OPC_TREE_H

#include <opcmalloc.h>
#include <hashtable.h>
#include <lock.h>
#include <lchar.h>

typedef struct OPCTreeContextT *OPCTreeContext;
typedef int (*OPCTreeWalkT)(const void *, void*);

#include <opc.h>
#include <opcitem.h>


#define OPC_TREE_DESTROYING_FLAG 1

struct OPCTreeContextT {
    OPCMallocContext malloc;
    OPCContext sctx;
    unsigned char status; /* 1 - Destroying Tree Hash */
    HashTable item;
    LockContext lock;
};

OPCTreeContext opcTreeCreateContext(OPCMallocContext mctx);
int opcTreeInitContext(OPCTreeContext ctx, OPCContext sctx);
void opcTreeFreeContext(OPCTreeContext ctx);
void opcTreeDestroyContext(OPCTreeContext ctx);
void opcTreeZeroContext(OPCTreeContext ctx);

int opcTreeGetStatus(OPCTreeContext ctx);
int opcTreeLock(OPCTreeContext ctx);
int opcTreeUnlock(OPCTreeContext ctx);

OPCItemContext opcTreeFindItem(OPCTreeContext ctx, const char *fullname);
OPCItemContext opcTreeGetItem(OPCTreeContext ctx, const char *fullname);
int opcTreeWalk(OPCTreeContext ctx, OPCTreeWalkT func, void *attr);

OPCItemContext opcTreeCreatePath(OPCTreeContext ctx, const char *path);
int opcTreeLinkItems(OPCTreeContext ctx, OPCItemContext ictx, OPCItemContext child);
int opcTreeUnlinkItems(OPCTreeContext ctx, OPCItemContext ictx, OPCItemContext child);
int opcTreeRegisterItem(OPCTreeContext ctx, OPCItemContext ictx, const char *path);
int opcTreeUnregisterItem(OPCTreeContext ctx, OPCItemContext ictx);

int opcTreeBrowse(OPCTreeContext ctx, const char *path, OPCItemBrowseFilter browse_filter, const char *name_filter, const char *vendor_filter, OPCStringList *result);

// Internal functions called from opcTree & opcItem and synchronised from caller functions 
int opcTreeMarkUnused(OPCTreeContext tree, OPCItemContext ctx);
int opcTreeCleanMarked(OPCTreeContext tree);

#endif /* _OPC_TREE_H */
