#include <stdio.h>
#include <assert.h>

#include <opcerr.h>
#include <opcconfig.h>
#include <opcdebug.h>

#include <opcmalloc.h>

#include <opc.h>
#include <opcstringlist.h>
#include <opctools.h>
#include <opcversion.h>

OPCVersion opcVersionInit(OPCMallocContext mctx, OPCContext ctx) {
    OPCVersion ver;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcVersionInit");    

    ver = (OPCVersion)opcMalloc(mctx,sizeof(struct OPCVersionT));
    if (ver) {
	ver->malloc=mctx;
	ver->major=NULL;
	ver->minor=NULL;
	ver->version=NULL;
	ver->SupportedInterfaceVersions=NULL;
    }
    return ver;
}

void opcVersionInterfacesClear(OPCVersion ctx) {
    assert(ctx);
    
    if (ctx->SupportedInterfaceVersions) {
	opcStringListDestroy(ctx->SupportedInterfaceVersions);
	ctx->SupportedInterfaceVersions=NULL;
    }
}

void opcVersionDestroy(OPCVersion ctx) {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcVersionDestroy");    

    assert(ctx);
    
    opcVersionInterfacesClear(ctx);
    if (ctx->major) opcExchangeString(ctx->malloc,&ctx->major,NULL);
    if (ctx->minor) opcExchangeString(ctx->malloc,&ctx->minor,NULL);
    if (ctx->version) opcExchangeString(ctx->malloc,&ctx->version,NULL);
    opcFree(ctx->malloc,ctx);
}

int opcVersionSet(OPCVersion ctx, const char *major, const char *minor, unsigned long build) {
    assert(ctx);
    assert(major);
    assert(minor);
    
    if (opcExchangeString(ctx->malloc,&ctx->major,major)) return ERR_OPC_MEM;
    if (opcExchangeString(ctx->malloc,&ctx->minor,minor)) return ERR_OPC_MEM;
    ctx->build=build;

    if (ctx->version) opcExchangeString(ctx->malloc,&ctx->version,NULL);
    ctx->version=(char*)opcMalloc(ctx->malloc,(strlen(major)+strlen(minor)+16)*sizeof(char));
    if (!ctx->version) return ERR_OPC_MEM;
    
    if (ctx->build)
	sprintf(ctx->version,"%s.%s.%u",ctx->major,ctx->minor,ctx->build);
    else
	sprintf(ctx->version,"%s.%s",ctx->major,ctx->minor);
    
    return 0;
}

int opcVersionInterfaceAdd(OPCVersion ctx, const char *interface, const char **extensions) {
    struct OPCInterfaceVersionT tmp;
    
    assert(ctx);
    assert(interface);
    
    if (!ctx->SupportedInterfaceVersions) {
	ctx->SupportedInterfaceVersions=opcStringListCreate(ctx->malloc);
	if (!ctx->SupportedInterfaceVersions) return ERR_OPC_MEM;
    }

    tmp.interface=interface;
    tmp.extensions=extensions;

    if (opcStringListAdd(ctx->SupportedInterfaceVersions,(void*)&tmp,sizeof(struct OPCInterfaceVersionT)))
	return ERR_OPC_MEM;
    return 0;    
}

const char *opcVersionGet(OPCVersion ctx) {
    assert(ctx);
    assert(ctx->version);
    
    return ctx->version;
}

int opcVersionList(OPCVersion ctx, OPCVersionListFunctionT list_function, void *attr) {
    OPCStringList siv;
    assert(ctx);
    assert(list_function);
    
    for (siv=ctx->SupportedInterfaceVersions;siv;siv=siv->next) {
	list_function(((OPCInterfaceVersion)siv->item)->interface,attr);
    }
    return 0;
}
