#include <assert.h>
#include <opcdebug.h>

#include <opcerr.h>
#include <opcformat.h>


OPCFormatContext opcFormatCreateContext(OPCFormatContext format, OPCMallocContext mctx) {
    OPCFormatContext ctx;

    assert(format);
    
    if (format->CreateContext) ctx=format->CreateContext(mctx);
    else ctx = (OPCFormatContext)opcMalloc(mctx,sizeof(struct OPCFormatContextT));

    if (ctx) {
	memcpy(ctx,format,sizeof(struct OPCFormatContextT));
	ctx->malloc=mctx;
    }
    return ctx;
}

int opcFormatInitContext(OPCFormatContext ctx,  const void *options) {
    assert(ctx);

    	// Check consistence
    if (!ctx->title) return ERR_OPC_FORMAT_INCOMPLETE_FORMAT_STRUCTURE;
    
    opcDebugServer(OPC_DEBUG_NOTICE, "opcFormatInitContext: %s (%p)", ctx->title, ctx);    

    if (ctx->InitContext) return ctx->InitContext(ctx, options);
    return 0;
}

void opcFormatFreeContext(OPCFormatContext ctx) {
    assert(ctx);
    assert(ctx->title);

    opcDebugServer(OPC_DEBUG_NOTICE, "opcFormatFreeContext: %s", ctx->title);    

    if (ctx->FreeContext) ctx->FreeContext(ctx);
}

void opcFormatDestroyContext(OPCFormatContext ctx) {
    assert(ctx);
    assert(ctx->title);

    opcDebugServer(OPC_DEBUG_NOTICE, "opcFormatDestroyContext: %s", ctx->title);    

    if (ctx->DestroyContext) ctx->DestroyContext(ctx);
    else {
	if (ctx->FreeContext) ctx->FreeContext(ctx);
	opcFree(ctx->malloc, ctx);
    }
}


