#define _OPC_FORMAT_OPCXML_C

#include <opcerr.h>
#include <opcassert.h>
#include <opcdebug.h>
#include <opclib.h>

#include <opcformatopcxml.h>

OPCFormatContext opcFormatOPCXML() {
    return (OPCFormatContext)&opc_format_opcxml;
}



int opcFormatOPCXMLImportData(OPCFormatOPCXMLContext ctx, OPCType type, OPCAllocatorContext allocator, const void *data, unsigned long size) {
    // ToDo: check if xml in text form and parse
}


static const char *opcxml_subtypes[] = {
    "byte", "short", "int", "long",
    "unsignedByte", "unsignedShort", "unsignedInt", "unsignedLong",
    "float", "double", "decimal", "boolean",
    "string", "dateTime"
};


const char *opcFormatOPCXMLSetType(OPCFormatOPCXMLContext ctx, xmlNsPtr xsins, OPCType type, xmlNodePtr node) {
    unsigned char flag = 0;
    
    if ((type->element_count>1)||(type->element_count_ref)||(type->terminator)) flag = 1;

    switch(type->type) {
	case OPC_TYPE_COMPLEX:
	    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfAnyType");
	    return NULL;
	case OPC_TYPE_INTEGER:
	    switch (((OPCTypeInteger)type)->bytes) {
		case 1:
		    if (((OPCTypeInteger)type)->is_signed) {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfByte");
			    return opcxml_subtypes[0];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "byte");
			    return NULL;
			}
		    } else {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfUnsignedByte");
			    return opcxml_subtypes[4];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "unsignedByte");
			    return NULL;
			}
		    }
		case 2:
		    if (((OPCTypeInteger)type)->is_signed) {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfShort");
			    return opcxml_subtypes[1];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "short");
			    return NULL;
			}
		    } else {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfUnsignedShort");
			    return opcxml_subtypes[5];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "unsignedShort");
			    return NULL;
			}
		    }
		case 4:
		    if (((OPCTypeInteger)type)->is_signed) {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfInt");
			    return opcxml_subtypes[2];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "int");
			    return NULL;
			}
		    } else {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfUnsignedInt");
			    return opcxml_subtypes[6];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "unsignedInt");
			    return NULL;
			}
		    }
		case 8:
		    if (((OPCTypeInteger)type)->is_signed) {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfLong");
			    return opcxml_subtypes[3];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "long");
			    return NULL;
			}
		    } else {
			if (flag) {
			    if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfUnsignedLong");
			    return opcxml_subtypes[7];
		        } else {
			    if (node) xmlSetNsProp(node, xsins, "type", "unsignedLong");
			    return NULL;
			}
		    }
		default:
		    if (flag) {
			if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfDecimal");
			return opcxml_subtypes[10];
		    } else {
			if (node) xmlSetNsProp(node, xsins, "type", "decimal");
			return NULL;
		    }
	    }
	case OPC_TYPE_FLOAT:
	    switch (((OPCTypeInteger)type)->bytes) {
		case 4:
		    if (flag) {
			if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfFloat");
			return opcxml_subtypes[8];
		    } else {
			if (node) xmlSetNsProp(node, xsins, "type", "float");
			return NULL;
		    }
		case 8:
		    if (flag) {
			if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfDouble");
			return opcxml_subtypes[9];
		    } else {
			if (node) xmlSetNsProp(node, xsins, "type", "double");
			return NULL;
		    }
		default:
		    if (flag) {
			if (node) xmlSetNsProp(node, xsins, "type", "ArrayOfDecimal");
			return opcxml_subtypes[10];
		    } else {
			if (node) xmlSetNsProp(node, xsins, "type", "decimal");
			return NULL;
		    }
	    }
    }

    return NULL;
}

static inline int opcFormatOPCXMLinteger(OPCTypeInteger type, const void *source, char *stmp) {
//    printf("Bytes: %u\n",type->bytes);
    if (type->is_signed) {
        switch(type->bytes) {
	    case sizeof(char):
	        sprintf(stmp, "%hhi", *(char*)source);
	    break;
	    case sizeof(short):
		sprintf(stmp, "%hi", *(short*)source);
	    break;
	    case sizeof(long):
	        sprintf(stmp, "%li", *(long*)source);
	    break;
	    case sizeof(long long):
	        sprintf(stmp, "%lli", *(long long*)source);
	    break;
	    default:
	        return ERR_OPC_TYPE_INVALID_SIZE;
	}
    } else {
	switch(type->bytes) {
	    case sizeof(char):
	        sprintf(stmp, "%hhu", *(unsigned char*)source);
	    break;
	    case sizeof(short):
	        sprintf(stmp, "%hu", *(unsigned short*)source);
	    break;
	    case sizeof(long):
	        sprintf(stmp, "%lu", *(unsigned long*)source);
	    break;
	    case sizeof(long long):
	        sprintf(stmp, "%llu", *(unsigned long long*)source);
	    break;
	    default:
	        return ERR_OPC_TYPE_INVALID_SIZE;
	}
    }
    return 0;
}

static inline int opcFormatOPCXMLfloat(OPCTypeFloat type, const void *source, char *stmp) {
//    printf("Bytes: %u\n",type->bytes);
    switch(type->bytes) {
	case sizeof(float):
	    sprintf(stmp, "%e", *(float*)source);
	break;
	case sizeof(double):
	    sprintf(stmp, "%le", *(double*)source);
	break;
	case sizeof(long double):
	    sprintf(stmp, "%Le", *(long double*)source);
	break;
	default:
	    return ERR_OPC_TYPE_INVALID_SIZE;
    }
    return 0;
}


/*
opcFormatReadNative
    Convert Native representation to OPCXML    

    ctx:	OPCXML Format Context
    type: 	Type to read
    allocator:	allocator for OPCXML representation
    data:	data in native format    
*/
int opcFormatOPCXMLReadNative(OPCFormatOPCXMLContext ctx, OPCType type, OPCAllocatorContext allocator, OPCData data, OPCData native) {
    int err;
    struct OPCConversionOPCXMLT conversion;
    xmlNodePtr node;
    xmlNsPtr xsins;
    xmlAttrPtr attr;
    
    node = xmlNewNode(NULL,"Value");
    xsins=xmlNewNs(node,opc_defines_xsi_namespace,"xsi");

    opcFormatOPCXMLSetType(ctx, xsins, type, node);

    conversion.node = node;
    conversion.xsins = xsins;

    err=opcTypeWrite(type, ctx, allocator, native->data, (OPCConversion)&conversion);
    if (!err) {
	data->data = (void*)node;
	data->size = 0;
	data->type = OPC_DATA_TYPE_XMLNODEPTR;
    }
    
    return err;
}


/*
opcFormatWriteNative
    Convert OPCXML representation to Native format

    ctx:	OPCXML Format Context
    type: 	Type to read
    allocator:	allocator for Native representation
    data:	data in OPCXML format
*/
int opcFormatOPCXMLWriteNative(OPCFormatOPCXMLContext ctx, OPCType type, OPCAllocatorContext allocator, OPCData native, OPCData data) {
    xmlNodePtr node;

    node=(xmlNodePtr)data;
    return opcTypeRead(type, ctx, allocator, (OPCConversion)node);
}


int opcFormatOPCXMLFree(OPCFormatOPCXMLContext ctx, OPCType type, OPCAllocatorContext allocator, OPCData data) {
    if (data&&data->data) xmlFreeNode((xmlNodePtr)data->data);
}


int opcFormatOPCXMLWriteInteger(OPCFormatOPCXMLContext ctx, OPCType type, OPCConversionOPCXML conversion, const void *source, unsigned long *bytes) {
    int err;
    xmlNodePtr node,cnode;
    unsigned long elements, i;    
    char stmp[64];

    node = conversion->node;
    
    elements=type->element_count;
    if (elements) {
	if (elements>1) {
	    puts("elems");
	    return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;
    	    for (i=type->element_count;i>0;i--) {
	    }
	} else {
	    err = opcFormatOPCXMLinteger((OPCTypeInteger)type, source, stmp);
	    if (err) return err;

	    if (type->options&OPC_TYPE_OPTION_MASTER) {
		puts("add");
		xmlNodeAddContentLen(node, stmp, strlen(stmp));
	    } else {
		cnode=xmlNewChild(node, NULL, "anyType", stmp);
		opcFormatOPCXMLSetType(ctx,conversion->xsins,type,cnode);
	    }
	    
	    if (bytes) *bytes = ((OPCTypeInteger)type)->bytes;
	}
    } else if (type->element_count_ref) {
	puts("cref");
	return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;
    } else if (type->terminator) {
	puts("term");
	return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;
    } else return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;
    return 0;
}



int opcFormatOPCXMLWriteFloat(OPCFormatOPCXMLContext ctx, OPCType type, OPCConversionOPCXML conversion, const void *source, unsigned long *bytes) {
    int err;
    xmlNodePtr node,cnode;
    unsigned long elements, i;    
    char stmp[64];
    const char *subtype;

    node = conversion->node;
    
    elements=type->element_count;
    if (elements) {
	if (elements>1) {
	    if (type->options&OPC_TYPE_OPTION_MASTER) {
		subtype=opcFormatOPCXMLSetType(ctx,NULL,type,NULL);
	    } else {
		cnode=xmlNewChild(node, NULL, "anyType", NULL);
		subtype=opcFormatOPCXMLSetType(ctx,conversion->xsins,type,cnode);
	    }
    	    for (i=type->element_count;i>0;i--) {
		err = opcFormatOPCXMLfloat((OPCTypeFloat)type, source, stmp);
		if (err) return err;
		node=xmlNewChild(cnode,NULL,subtype,stmp);
		source+=((OPCTypeFloat)type)->bytes;
	    }
	    if (bytes) *bytes = ((OPCTypeFloat)type)->bytes * type->element_count;
	} else {
	    err = opcFormatOPCXMLfloat((OPCTypeFloat)type, source, stmp);
	    if (err) return err;

	    if (type->options&OPC_TYPE_OPTION_MASTER) {
		puts("add");
		xmlNodeAddContentLen(node, stmp, strlen(stmp));
	    } else {
		cnode=xmlNewChild(node, NULL, "anyType", stmp);
		opcFormatOPCXMLSetType(ctx,conversion->xsins,type,cnode);
	    }
	    
	    if (bytes) *bytes = ((OPCTypeFloat)type)->bytes;
	}
    } else if (type->element_count_ref) {
	puts("cref");
	return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;
    } else if (type->terminator) {
	puts("term");
	return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;
    } else return ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION;

    return 0;
}

