#define _OPC_FORMAT_DESCRIPTION_OPC_BINARY_DICTIONARY_C
#include <assert.h>
#include <libxml/tree.h>

#include <opcerr.h>
#include <opcdebug.h>
#include <opcxml.h>

#include <opcbinaryconfiguration.h>

#include <opcsimpletypes.h>
#include <opccomplextype.h>

#include <opcfdopcbinarydictionary.h>

void opcFDOPCBinaryDictionaryZeroContext(OPCFDOPCBinaryDictionaryContext ctx) {
    assert(ctx);

    ctx->xpath_ctx = NULL;
    ctx->xml_doc = NULL;
    ctx->lock = NULL;
    ctx->types = NULL;
    ctx->default_string_encoding = NULL;
    ctx->default_float_format = NULL;
}

OPCFDOPCBinaryDictionaryContext opcFDOPCBinaryDictionaryCreateContext(OPCMallocContext mctx) {
    OPCFDOPCBinaryDictionaryContext ctx;

    ctx = (OPCFDOPCBinaryDictionaryContext)opcMalloc(mctx,sizeof(struct OPCFDOPCBinaryDictionaryContextT));
    if (ctx) {
	ctx->malloc=mctx;
	opcFDOPCBinaryDictionaryZeroContext(ctx);
    }
    return ctx;
}

int opcFDOPCBinaryDictionaryInitContext(OPCFDOPCBinaryDictionaryContext ctx, OPCNamespaces namespaces, xmlDocPtr doc) {
    int err;
    xmlNodePtr node;
    xmlAttrPtr attr;
    
    assert(ctx);
    assert(doc);

    opcFDOPCBinaryDictionaryZeroContext(ctx);

    ctx->lock = LockCreateContext(ctx->malloc);
    if (!ctx->lock) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return ERR_OPC_MEM;
    }
    err = LockInitContext(ctx->lock);
    if (err) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return err;
    }
    
    ctx->types = opcEnumerationCreateContext(ctx->malloc);
    if (!ctx->types) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return ERR_OPC_MEM;
    }
    err = opcEnumerationInitContext(ctx->types, (OPCEnumerationBucket)&opc_fd_opcbinary_types_enumeration);
    if (err) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return err;
    }
	
    ctx->xml_doc = doc;

    ctx->xpath_ctx = xmlXPathNewContext(doc);
    if (!ctx->xpath_ctx) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_XPATH_CONTEXT;
    }
    
    if (xmlXPathRegisterNs(ctx->xpath_ctx, "OPCBinary", namespaces->opcbinary_namespace)) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_XPATH_REGISTER_NAMESPACE;
    }

    ctx->xsins = xmlSearchNsByHref(doc, xmlDocGetRootElement(doc), namespaces->xsi_namespace);
    if (!ctx->xsins) {
	opcFDOPCBinaryDictionaryFreeContext(ctx);
	return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_FIND_XSI_NAMESPACE;
    }

    node = xmlDocGetRootElement(doc);
    attr=xmlHasProp(node, "DefaultBigEndian");
    if ((attr)&&((!xmlStrcmp(attr->children->content,"false"))||(!xmlStrcmp(attr->children->content,"0"))))
	ctx->default_big_endian=0;
    else ctx->default_big_endian=1;
    attr=xmlHasProp(node, "DefaultCharWidth");
    if (attr) ctx->default_char_width = atoi(attr->children->content);
    else ctx->default_char_width = 2;
    attr=xmlHasProp(node, "DefaultStringEncoding");
    if (attr) opcExchangeString(ctx->malloc, &ctx->default_string_encoding, attr->children->content);
    else opcExchangeString(ctx->malloc, &ctx->default_string_encoding, "UCS-2");
    attr=xmlHasProp(node, "DefaultFloatFormat");
    if (attr) opcExchangeString(ctx->malloc, &ctx->default_float_format, attr->children->content);
    else opcExchangeString(ctx->malloc, &ctx->default_float_format, "IEEE-754");

    ctx->namespaces = namespaces;

    return 0;        
}

void opcFDOPCBinaryDictionaryFreeContext(OPCFDOPCBinaryDictionaryContext ctx) {
    assert(ctx);

	// ToDo: Wait for lock
    if (ctx->lock) LockDestroyContext(ctx->lock);
    if (ctx->default_string_encoding) opcExchangeString(ctx->malloc,&ctx->default_string_encoding, NULL);
    if (ctx->default_float_format) opcExchangeString(ctx->malloc, &ctx->default_float_format, NULL);
    if (ctx->types) opcEnumerationDestroyContext(ctx->types);
    if (ctx->xpath_ctx) xmlXPathFreeContext(ctx->xpath_ctx);
    if (ctx->xml_doc) xmlFreeDoc(ctx->xml_doc);
    opcFDOPCBinaryDictionaryZeroContext(ctx);
}

void opcFDOPCBinaryDictionaryDestroyContext(OPCFDOPCBinaryDictionaryContext ctx) {
    opcFDOPCBinaryDictionaryFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}


#define XPATH_LOCALIZATION_EXPRESSION "//OPCBinary:TypeDescription[@TypeID]"

int opcFDOPCBinaryDictionaryLocalise(OPCFDOPCBinaryDictionaryContext ctx) {
    int err;
    OPCFDOPCBinaryTypes field_type;
    xmlAttrPtr attr;
    xmlXPathObjectPtr obj;
    xmlNodePtr pnode, node;
    xmlNodeSetPtr node_set;
    unsigned long nnodes;
    const char *stmp;
    unsigned long i;
    
    assert(ctx);
    
    puts("Localising");
    
    obj = xmlXPathEvalExpression(XPATH_LOCALIZATION_EXPRESSION, ctx->xpath_ctx);
    if (!obj) return 0;
    node_set = obj->nodesetval;
    if (!node_set) return 0;
    nnodes = node_set->nodeNr;
    
    for (i=0;i<nnodes;i++) {
	pnode = node_set->nodeTab[i];
        
	printf("Node: %s\n", opcXmlGetPropString(pnode, "TypeID", NULL));

	if (opc_binary_configuration->endianess)
	    xmlSetProp(pnode, "DefaultBigEndian", "true");
	else
	    xmlSetProp(pnode, "DefaultBigEndian", "false");
	
	
	for (node=pnode->children;node;node=node->next) {
	    attr=xmlHasNsProp(node, "type", ctx->xsins->href);
	    if (!attr) continue;
	    printf(" Type: %s\n", attr->children->content);

	    field_type = opcEnumerationGetID(ctx->types, attr->children->content);
	    switch(field_type) {
		case OPC_FD_OPCBINARY_INTEGER:
		break;
		case OPC_FD_OPCBINARY_FLOATINGPOINT:
		    stmp=opcXmlGetPropString(node, "Format", NULL);
		    if (stmp) break;

		    if (!xmlStrcmp(stmp,"float")) {
		        if (opc_binary_configuration->align_float>1)
			    opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_float);
			else {
			    attr = xmlHasProp(node, "Alignment");
			    if (attr) xmlRemoveProp(attr);
			}
			if (opc_binary_configuration->float_format==FLOAT_FORMAT_IEEE) {
			    if (opc_binary_configuration->float_endianess == opc_binary_configuration->endianess)
				xmlSetProp(node, "FloatFormat", "IEEE-754");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_LITTLE_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-LE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_BIG_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-BE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_MIXED_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-ME");
    			    else 
				return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_ENDIANESS;
			} else {
			    return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_FLOATING_FORMAT;
			}
			opcXmlSetPropInt(node, "Length", opc_binary_configuration->float_size);
			break;
		    }

		    if (!xmlStrcmp(stmp,"double")) {
		        if (opc_binary_configuration->align_float>1)
			    opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_double);
			else {
			    attr = xmlHasProp(node, "Alignment");
			    if (attr) xmlRemoveProp(attr);
			}
			if (opc_binary_configuration->float_format==FLOAT_FORMAT_IEEE) {
			    if (opc_binary_configuration->float_endianess == opc_binary_configuration->endianess)
				xmlSetProp(node, "FloatFormat", "IEEE-754");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_LITTLE_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-LE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_BIG_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-BE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_MIXED_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-ME");
    			    else 
				return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_ENDIANESS;
			} else {
			    return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_FLOATING_FORMAT;
			}
			opcXmlSetPropInt(node, "Length", opc_binary_configuration->double_size);
			break;
		    }
		break;
		case OPC_FD_OPCBINARY_CHARSTRING:
		break;
		case OPC_FD_OPCBINARY_TYPEREFERENCE:
		break;
		case OPC_FD_OPCBINARY_INT8:
		    if (opc_binary_configuration->align_int8>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int8);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_INT16:
		    if (opc_binary_configuration->align_int16>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int16);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_INT32:
		    if (opc_binary_configuration->align_int32>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int32);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_INT64:
		    if (opc_binary_configuration->align_int64>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int64);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_UINT8:
		    if (opc_binary_configuration->align_int8>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int8);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_UINT16:
		    if (opc_binary_configuration->align_int16>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int16);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_UINT32:
		    if (opc_binary_configuration->align_int32>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int32);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_UINT64:
		    if (opc_binary_configuration->align_int64>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_int64);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		break;
		case OPC_FD_OPCBINARY_SINGLE:
		    if (opc_binary_configuration->align_float>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_float);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		    if (opc_binary_configuration->float_format==FLOAT_FORMAT_IEEE) {
			if (opc_binary_configuration->float_endianess == opc_binary_configuration->endianess) {
			    if (opc_binary_configuration->float_size != 4) {
				xmlNodeSetName(node, "FloatingPoint");
				xmlSetProp(node, "FloatFormat", "IEEE-754");
				xmlSetProp(node, "Format", "float");
				opcXmlSetPropInt(node, "Length", opc_binary_configuration->float_size);
			    }
			} else {
			    xmlNodeSetName(node, "FloatingPoint");
			    if (opc_binary_configuration->float_endianess==ENDIANESS_LITTLE_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-LE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_BIG_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-BE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_MIXED_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-ME");
			    else 
				return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_ENDIANESS;
			    xmlSetProp(node, "Format", "float");
			    opcXmlSetPropInt(node, "Length", opc_binary_configuration->float_size);
			}
		    } else {
			return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_FLOATING_FORMAT;
		    }
		break;
		case OPC_FD_OPCBINARY_DOUBLE:
		    if (opc_binary_configuration->align_double>1)
			opcXmlSetPropInt(node, "Alignment", opc_binary_configuration->align_double);
		    else {
			attr = xmlHasProp(node, "Alignment");
			if (attr) xmlRemoveProp(attr);
		    }
		    if (opc_binary_configuration->float_format==FLOAT_FORMAT_IEEE) {
			if (opc_binary_configuration->float_endianess == opc_binary_configuration->endianess) {
			    if (opc_binary_configuration->double_size != 8) {
				xmlNodeSetName(node, "FloatingPoint");
				xmlSetProp(node, "FloatFormat", "IEEE-754");
				xmlSetProp(node, "Format", "double");
				opcXmlSetPropInt(node, "Length", opc_binary_configuration->double_size);
			    }
			} else {
			    xmlNodeSetName(node, "FloatingPoint");
			    if (opc_binary_configuration->float_endianess==ENDIANESS_LITTLE_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-LE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_BIG_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-BE");
			    else if (opc_binary_configuration->float_endianess==ENDIANESS_MIXED_ENDIAN)
				xmlSetProp(node, "FloatFormat", "IEEE-754-ME");
			    else 
				return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_ENDIANESS;
			    xmlSetProp(node, "Format", "double");
			    opcXmlSetPropInt(node, "Length", opc_binary_configuration->double_size);
			}
		    } else {
			return ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_FLOATING_FORMAT;
		    }
		break;
		case OPC_FD_OPCBINARY_ASCII:
		    err=1;
		break;
		case OPC_FD_OPCBINARY_UNICODE:
		    err=1;
		break;
		default:
		    err=1;
		break;
	    
	    }
	}	
    }
    
    xmlXPathFreeObject(obj);
    return 0;    
}


#define XPATH_EXPRESSION "//OPCBinary:TypeDescription[@TypeID=\"%s\"]"

inline static void process_node_basics(OPCType ctype, xmlNodePtr node) {
    xmlAttrPtr attr;

    attr=xmlHasProp(node, "ElementCount");
    if (attr) opcTypeSetElementCount(ctype, atol(attr->children->content));
//    attr=xmlHasProp(node, "ElementCountRef");
//    if (attr) opcTypeSetElementCount(ctype, ref);
    attr=xmlHasProp(node, "FieldTerminator");
    if (attr) opcTypeSetTerminator(ctype, attr->children->content);
    
    opcTypeSetAlignment(ctype, opcXmlGetPropInt(node, "Alignment", 1));
}

OPCType opcFDOPCBinaryDictionaryCreateType(OPCFDOPCBinaryDictionaryContext ctx, const char *type) {
    int err = 0;
    char *xpath,*stmp;
    unsigned long nnodes;
    OPCMallocContext mctx;
    xmlNodePtr pnode, node;
    xmlNodeSetPtr node_set;
    xmlXPathObjectPtr obj;
    xmlAttrPtr attr;
    OPCType ptype;
    OPCType ctype;
    OPCFDOPCBinaryTypes field_type;
    unsigned char big_endian;
    unsigned char char_width;
    char *string_encoding;
    char *float_format;
    
    unsigned long size;
    
    assert(ctx);
    assert(ctx->xpath_ctx);
    assert(type);
    
    opcDebugServer(OPC_DEBUG_NOTICE, "opcFormatDescriptionOPCBinaryDictionaryCreateType: %s", type);
    xpath = (char*)malloc((sizeof(XPATH_EXPRESSION)+strlen(type))*sizeof(char));
    if (!xpath) return NULL;
    sprintf(xpath, XPATH_EXPRESSION, type);
    
    LockWrite(ctx->lock);
    obj = xmlXPathEvalExpression(xpath,ctx->xpath_ctx);
    LockReturn(ctx->lock);
    if (!obj) return NULL;
    node_set = obj->nodesetval;
    if (!node_set) {
	xmlXPathFreeObject(obj);
	return NULL;
    }
    nnodes = node_set->nodeNr;
    if (nnodes!=1) {
        xmlXPathFreeObject(obj);
	return NULL;
    }
    pnode = node_set->nodeTab[0];
    if (pnode->ns) printf("Node: %s:%s\n",pnode->ns->href,pnode->name);
    else printf("Node: %s\n",pnode->name);
    
    mctx = ctx->malloc;
    attr=xmlHasProp(pnode, "DefaultBigEndian");
    if (attr) {
	if ((!xmlStrcmp(attr->children->content,"false"))||(!xmlStrcmp(attr->children->content,"0")))
	    big_endian = 0;
	else
	    big_endian = 1;
    } else big_endian=ctx->default_big_endian;
    attr=xmlHasProp(pnode, "DefaultCharWidth");
    if (attr) char_width = atoi(attr->children->content);
    else char_width=ctx->default_char_width;
    attr=xmlHasProp(pnode, "DefaultStringEncoding");
    if (attr) string_encoding = attr->children->content;
    else string_encoding = ctx->default_string_encoding;
    attr=xmlHasProp(pnode, "DefaultFloatFormat");
    if (attr) float_format = attr->children->content;
    else float_format = ctx->default_float_format;

    ptype = (OPCType)opcTypeComplexInit(ctx->malloc);
    if (!ptype) {
	xmlXPathFreeObject(obj);
	return NULL;
    }
    opcTypeSetMaster(ptype);

    size=0;
    for (node=pnode->children;node;node=node->next) {
	if ((node->type==XML_ELEMENT_NODE)&&(!xmlStrcmp(node->name, "Field"))) {
	    attr=xmlHasNsProp(node, "type", ctx->xsins->href);
	    if ((err)||(!attr)) {
		xmlXPathFreeObject(obj);
		return NULL;
	    }
	    field_type = opcEnumerationGetID(ctx->types, attr->children->content);
	    printf("New Node: %s(%u)\n",node->name,field_type);
	    
	    ctype = NULL;
	    switch(field_type) {
		case OPC_FD_OPCBINARY_INTEGER:
		    err=1;
		break;
		case OPC_FD_OPCBINARY_FLOATINGPOINT:
		    err=1;
		break;
		case OPC_FD_OPCBINARY_CHARSTRING:
		    err=1;
		break;
		case OPC_FD_OPCBINARY_TYPEREFERENCE:
		    err=1;
		break;
		case OPC_FD_OPCBINARY_INT8:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 1, 1, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_INT16:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 1, 2, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_INT32:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 1, 4, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_INT64:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 1, 8, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_UINT8:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 0, 1, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_UINT16:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 0, 2, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_UINT32:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 0, 4, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_UINT64:
		    ctype = (OPCType)opcTypeIntegerInit(mctx, 0, 8, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_SINGLE:
		    ctype = (OPCType)opcTypeFloatInit(mctx, OPC_FLOAT_IEEE, 4, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_DOUBLE:
		    ctype = (OPCType)opcTypeFloatInit(mctx, OPC_FLOAT_IEEE, 8, big_endian);
		    if (ctype) {
			process_node_basics(ctype, node);
			opcTypePush(ptype,ctype);
		    } else err=2;
		break;
		case OPC_FD_OPCBINARY_ASCII:
		    err=1;
		break;
		case OPC_FD_OPCBINARY_UNICODE:
		    err=1;
		break;
		default:
		    err=1;
		break;
	    }
	    if ((ctype)&&(size!=(unsigned long)-1)) {
		printf("SubSize: %u\n",ctype->size);
		printf("Alignment: %u\n",ctype->align);
		if (ctype->size) {
    		    if ((ctype->align>1)&&(size&(ctype->align-1))) 
			size=(size&~(ctype->align-1))+ctype->align;
		    size+=ctype->size;
		}
		else size=(unsigned long)-1;
	    }
	}
    }

    if (size) ptype->size=size;
    printf("UltimateSize: %lu\n",ptype->size);
    
    xmlXPathFreeObject(obj);
    return ptype;
}
