#ifndef _OPC_FORMAT_DESCRIPTION_OPC_BINARY_DICTIONARY_H
#define _OPC_FORMAT_DESCRIPTION_OPC_BINARY_DICTIONARY_H

#include <libxml/tree.h>
#include <libxml/xpath.h>

#include <opcconfiguration.h>
#include <opcmalloc.h>
#include <opcenumeration.h>
#include <lock.h>

typedef struct OPCFDOPCBinaryDictionaryContextT *OPCFDOPCBinaryDictionaryContext;

#include <opcfdopcbinary.h>
#include <opctype.h>

struct OPCFDOPCBinaryDictionaryContextT {
    OPCMallocContext malloc;
    OPCNamespaces namespaces;
    OPCEnumerationContext types;
    
    LockContext lock;

    xmlXPathContextPtr xpath_ctx;    
    xmlDocPtr xml_doc;
    xmlNsPtr xsins;
    
    unsigned char default_big_endian;
    unsigned char default_char_width;
    char *default_string_encoding;
    char *default_float_format;
};

OPCFDOPCBinaryDictionaryContext opcFDOPCBinaryDictionaryCreateContext(OPCMallocContext mctx);
int opcFDOPCBinaryDictionaryInitContext(OPCFDOPCBinaryDictionaryContext ctx, OPCNamespaces namespaces, xmlDocPtr doc);
void opcFDOPCBinaryDictionaryFreeContext(OPCFDOPCBinaryDictionaryContext ctx);
void opcFDOPCBinaryDictionaryDestroyContext(OPCFDOPCBinaryDictionaryContext ctx);

int opcFDOPCBinaryDictionaryLocalise(OPCFDOPCBinaryDictionaryContext ctx);

OPCType opcFDOPCBinaryDictionaryCreateType(OPCFDOPCBinaryDictionaryContext ctx, const char *type);

/* ToDo:
int opcFDOPCBinaryDictionaryAddType(OPCFDOPCBinaryDictionaryContext ctx, xmlDocPtr doc);
int opcFDOPCBinaryDictionaryLocalize(OPCFDOPCBinaryDictionaryContext ctx);
int opcFDOPCBinaryDictionaryPublish(OPCFDOPCBinaryDictionaryContext ctx);
*/


/* ToDo: in case of problems rewrite truth set of defines */
enum OPCFDOPCBinaryTypesT {
    OPC_FD_OPCBINARY_INTEGER,
    OPC_FD_OPCBINARY_FLOATINGPOINT,
    OPC_FD_OPCBINARY_CHARSTRING,
    OPC_FD_OPCBINARY_TYPEREFERENCE,
    OPC_FD_OPCBINARY_INT8,
    OPC_FD_OPCBINARY_INT16,
    OPC_FD_OPCBINARY_INT32,
    OPC_FD_OPCBINARY_INT64,
    OPC_FD_OPCBINARY_UINT8,
    OPC_FD_OPCBINARY_UINT16,
    OPC_FD_OPCBINARY_UINT32,
    OPC_FD_OPCBINARY_UINT64,
    OPC_FD_OPCBINARY_SINGLE,
    OPC_FD_OPCBINARY_DOUBLE,
    OPC_FD_OPCBINARY_ASCII,
    OPC_FD_OPCBINARY_UNICODE
};

typedef enum OPCFDOPCBinaryTypesT OPCFDOPCBinaryTypes;


struct OPCFDOPCBinaryTypesEnumerationT {
    char *label;
    unsigned long type;
};

#ifdef _OPC_FORMAT_DESCRIPTION_OPC_BINARY_DICTIONARY_C
struct OPCFDOPCBinaryTypesEnumerationT opc_fd_opcbinary_types_enumeration[] = {
    {"Integer", OPC_FD_OPCBINARY_INTEGER},
    {"FloatingPoint", OPC_FD_OPCBINARY_FLOATINGPOINT},
    {"CharString", OPC_FD_OPCBINARY_CHARSTRING},
    {"TypeReference", OPC_FD_OPCBINARY_TYPEREFERENCE},
    {"Int8", OPC_FD_OPCBINARY_INT8},
    {"Int16", OPC_FD_OPCBINARY_INT16},
    {"Int32", OPC_FD_OPCBINARY_INT32},
    {"Int64", OPC_FD_OPCBINARY_INT64},
    {"UInt8", OPC_FD_OPCBINARY_UINT8},
    {"UInt16", OPC_FD_OPCBINARY_UINT16},
    {"UInt32", OPC_FD_OPCBINARY_UINT32},
    {"UInt64", OPC_FD_OPCBINARY_UINT64},
    {"Single", OPC_FD_OPCBINARY_SINGLE},
    {"Double", OPC_FD_OPCBINARY_DOUBLE},
    {"Ascii", OPC_FD_OPCBINARY_ASCII},
    {"Unicode", OPC_FD_OPCBINARY_UNICODE},
    {NULL, 0}
};
#endif /* _OPC_FORMAT_DESCRIPTION_OPC_BINARY_DICTIONARY_C */

#endif /* _OPC_FORMAT_DESCRIPTION_OPC_BINARY_DICTIONARY_H */
