#include <opcassert.h>
#include <opcerr.h>

#include <opclib.h>

#include <opctools.h>

#include <opctreenode.h>
#include <opctype.h>

static inline void opcTypeZero(OPCType ctx) {
    ctx->options = 0;
    ctx->terminator = NULL;
    ctx->element_count_ref = NULL;
    ctx->element_count = 1;
    ctx->is_pointer = 0;
    ctx->size = 0;
    ctx->align = 0;
}

OPCType opcTypeCreate(OPCMallocContext mctx) {
    OPCType ctx;
    
    ctx = opcMalloc(mctx, sizeof(struct OPCTypeT));
    if (ctx) {
	((OPCTreeNodeContext)ctx)->malloc = mctx;
	opcTypeZero(ctx);
    }
    return ctx;
}

int opcTypeInit(OPCType ctx, OPCSimpleType type, OPCDestroyContextFunction destructor) {
    assert(ctx);
    opcTypeZero(ctx);
    if (!destructor) destructor=(OPCDestroyContextFunction)opcTypeDestroy;
    opcTreeComplexNodeInitContext((OPCTreeComplexNodeContext)ctx, destructor);
    ctx->type = type;
    return 0;
}

void opcTypeFree(OPCType ctx) {
    assert(ctx);
    if (ctx->terminator) opcExchangeString(((OPCTreeNodeContext)ctx)->malloc, &ctx->terminator, NULL);
    opcTypeZero(ctx);
}

void opcTypeDestroy(OPCType ctx) {
    opcTypeFree(ctx);
    opcFree(((OPCTreeNodeContext)ctx)->malloc,ctx);
}

void opcTypeSetElementCount(OPCType ctx, unsigned long element_count) {
    assert(ctx);
    assert(!ctx->terminator);
    assert(!ctx->element_count_ref);
    
    ctx->element_count = element_count;
    
    if ((element_count>1)&&(ctx->size)) ctx->size*=element_count;
}

void opcTypeSetElementCountRef(OPCType ctx, OPCType ref) {
    assert(ctx);
    assert(!ctx->terminator);
    assert(!ctx->element_count);
    
    ctx->element_count = 0;
    ctx->element_count_ref = ref;
}

void opcTypeSetTerminator(OPCType ctx, const char *terminator) {
    assert(ctx);
    assert(!ctx->element_count);
    assert(!ctx->element_count_ref);

    ctx->element_count = 0;
    opcExchangeString(((OPCTreeNodeContext)ctx)->malloc, &ctx->terminator, terminator);
}

void opcTypeSetExternal(OPCType ctx) {
    assert(ctx);
    
    ctx->is_pointer = 1;
}

void opcTypeSetMaster(OPCType ctx) {
    assert(ctx);
    
    ctx->options|=OPC_TYPE_OPTION_MASTER;
}

void opcTypeResetMaster(OPCType ctx) {
    assert(ctx);
    
    ctx->options&=~OPC_TYPE_OPTION_MASTER;
}

void opcTypeSetAlignment(OPCType ctx, unsigned char align) {
    assert(ctx);
    
    ctx->align = align;
}

int opcTypePush(OPCType ctx, OPCType child) {
    assert(ctx);
    assert(child);
    assert(ctx->type == OPC_TYPE_COMPLEX);
    return opcTreeComplexNodeAddChild((OPCTreeComplexNodeContext)ctx,(OPCTreeNodeContext)child);
}


int opcTypeWrite(OPCType ctx, OPCFormatContext format, OPCAllocatorContext allocator, const void *source, OPCConversion conversion) {
    int err;
    unsigned long bytes;
//    struct OPCConversionContextT conversion;
    OPCTreeNodeContext node;
    unsigned long pos = 0;
    char *source_ptr;
    unsigned char align;
        
    source_ptr = (char*)source;
    
//    format->CreateConversionContext();
    
    for (node=((OPCTreeComplexNodeContext)ctx)->first_child;node;node=node->next) {

	align = ((OPCType)node)->align;
	if ((align>1)&&(pos&(align-1))) pos=(pos&~(align-1))+align;

	switch (((OPCType)node)->type) {
	    case OPC_TYPE_COMPLEX:
		// source_ptr viravnivaem
//		format->WriteStructureStart(format, ctx, (void*)source_ptr, conversion);
//		opcTypeWrite((OPCType)node, ctx, source_ptr, conversion)
//		format->WriteStructureEnd(format, ctx, (void*)..., conversion);
		return ERR_OPC_TYPE_UNSUPPORTED;
	    break;
	    case OPC_TYPE_INTEGER:
		printf("Writing Integer: %lu position\n", pos);
		if (format->WriteInteger)
		    err=format->WriteInteger(format, (OPCType)node, conversion, (void*)(source_ptr+pos), &bytes);
		else 
		    return ERR_OPC_TYPE_UNSUPPORTED_BY_FORMAT;

		if (err) return err;
		
		printf("Writed Integer: %lu bytes\n", bytes);
		pos+=bytes;
	    break;
	    case OPC_TYPE_FLOAT:
		printf("Writing Float: %lu\n", pos);
		if (format->WriteFloat)
	    	    err=format->WriteFloat(format, (OPCType)node, conversion, (void*)(source_ptr+pos), &bytes);
		else
		    return ERR_OPC_TYPE_UNSUPPORTED_BY_FORMAT;
		
		if (err) return err;
		
		printf("Writed Float: %lu bytes\n", bytes);
		pos+=bytes;
	    break;
	    default:
		return ERR_OPC_TYPE_UNSUPPORTED;
	    break;
	}
    }
    
//    format->DestroyConversionContext();
    puts("The END");
    return 0;
}

// Constructing local binary representation
int opcTypeRead(OPCType ctx, OPCFormatContext format, void *source) {
/*    OPCTreeNodeContext node;
    unsigned long pos;
    char *source_ptr;
        
    if (ctx->type!=OPC_TYPE_COMPLEX) return 0; // ToDo: ...
    
    source_ptr = (char*)source;
    
    for (node=((OPCTreeComplexNodeContext)ctx)->first_child;node;node=node->next) {
	switch (((OPCType)node)->type) {
	    case OPC_TYPE_COMPLEX:
		//
	    break;
	    case OPC_TYPE_ALIGN:
//		source_ptr+=node->element_count;
	    break;
	    case OPC_TYPE_INTEGER:
		format->ReadInteger(format, ctx, (void*)source_ptr, allocator);
	    break;
	    case OPC_TYPE_FLOAT:
		format->ReadFloat(format, ctx, (void*)source_ptr, allocator);
	    break;
	    default:
		//
	    break;
	}
    }


    switch (ctx->type) {
	case OPC_TYPE_COMPLEX:
	case OPC_TYPE_ALIGN:
	
	case OPC_TYPE_INTEGER:
	    format->ReadInteger(format, ctx, source, 
	
	case OPC_TYPE_FLOAT:    
    }    */

    return 0;
}


/*
int opcTypeWrite(OPCType ctx, OPCFormatContext format, void *source) {
    return 0;
}
*/

/*
int opcTypeCopy(OPCtype ctx, OPCFormatInstanceContext format, void *source) {
    void *ptr;

//    copy source representation into OPCDataBufferContext
    if (ctx->size) {
	ptr = opcDataBufferAllocate(dbctx,ctx->size);
	if (!ptr) ...
	memcpy(ptr, source, ctx->size);
	
	for (node=ctx->node.first_child;node;node=node->next) {
	    if ((OPCType)node)->type==OPC_TYPE_POINTER) {
		// copy recursive
	    }
	}
    } else {
	// go by items
    }

}
*/

/*
int opcTypeCopy(OPCType ctx, void *source, OPCDataBufferContext dbctx) {
    void *ptr;
//    copy source representation into OPCDataBufferContext
    if (ctx->size) {
	ptr = opcDataBufferAllocate(dbctx,ctx->size);
	if (!ptr) ...
	memcpy(ptr, source, ctx->size);
	
	for (node=ctx->node.first_child;node;node=node->next) {
	    if ((OPCType)node)->type==OPC_TYPE_POINTER) {
		// copy recursive
	    }
	}
    } else {
	// go by items
    }
}
// copy source representation from OPCDataBufferContext?
// Pointers should be normalized... Etc.

// 
int opcTypeSend(OPCType ctx, void *source, int socket) {
}
*/
