#ifndef _OPC_DATA_CHUNK_H
#define _OPC_DATA_CHUNK_H

#include <opcmalloc.h>

#include <lock.h>
#include <condition.h>
#include <opctime.h>
#include <opcquality.h>
#include <opcuserdata.h>


typedef struct OPCDataInfoT *OPCDataInfo;
typedef struct OPCDataInfoHolderT *OPCDataInfoHolder;
typedef struct OPCDataChunkContextT *OPCDataChunkContext;
typedef void (*OPCDataChunkRunFunction)(void *ctx, OPCRepresentationData data);


struct OPCDataInfoHolderT {
    struct OPCTimeT time;
    struct OPCQualityT quality;
};

struct OPCDataInfoT {
    OPCTime time;
    OPCQuality quality;
};

struct OPCDataChunkContextT {
    struct OPCDataInfoHolderT info;

    OPCRepresentationData data;
    
    OPCMallocContext malloc;
    LockContext lock;
    ConditionContext cond;
    OPCDataChunkContext next;

    unsigned long referers;
    unsigned long delayed_referers;

    unsigned short int supported_representations;
};

OPCDataChunkContext opcDataChunkCreateContext(OPCMallocContext mctx);
int opcDataChunkInitContext(OPCDataChunkContext ctx, OPCRepresentationInfo *definfo, unsigned short int supported_representations);
int opcDataChunkRestoreContext(OPCDataChunkContext ctx, OPCRepresentationInfo *definfo);
void opcDataChunkFreeContext(OPCDataChunkContext ctx);
void opcDataChunkDestroyContext(OPCDataChunkContext ctx);

int opcDataChunkSet(OPCDataChunkContext ctx, unsigned short int format, void *data, unsigned long size);

OPCRepresentationData opcDataChunkGet(OPCDataChunkContext ctx, unsigned short int format);
OPCRepresentationData opcDataChunkGetData(OPCDataChunkContext ctx, unsigned short int format);
unsigned short opcDataChunkFindData(OPCDataChunkContext ctx);

int opcDataChunkCopyInfo(OPCDataChunkContext ctx, const OPCDataInfo info);
int opcDataChunkSetInfo(OPCDataChunkContext ctx, const OPCTime time, const OPCQuality quality);
OPCDataInfo opcDataChunkGetInfo(OPCDataChunkContext ctx);
OPCDataChunkContext opcDataChunkGetChunkByInfo(OPCDataInfo ctx);

unsigned long opcDataChunkRef(OPCDataChunkContext ctx);
unsigned long opcDataChunkUnref(OPCDataChunkContext ctx);
unsigned long opcDataChunkGetRefs(OPCDataChunkContext ctx);
unsigned long opcDataChunkDelayedRef(OPCDataChunkContext ctx);
unsigned long opcDataChunkRefDelayed(OPCDataChunkContext ctx);
unsigned long opcDataChunkDelayedUnref(OPCDataChunkContext ctx);
unsigned long opcDataChunkGetDelayedRefs(OPCDataChunkContext ctx);

unsigned long opcDataChunkRefByInfo(OPCDataInfo ctx);
unsigned long opcDataChunkUnrefByInfo(OPCDataInfo ctx);
unsigned long opcDataChunkGetRefsByInfo(OPCDataInfo ctx);
unsigned long opcDataChunkDelayedRefByInfo(OPCDataInfo ctx);
unsigned long opcDataChunkRefDelayedByInfo(OPCDataInfo ctx);
unsigned long opcDataChunkDelayedUnrefByInfo(OPCDataInfo ctx);
unsigned long opcDataChunkGetDelayedRefsByInfo(OPCDataInfo ctx);


void opcDataChunkLock(OPCDataChunkContext ctx);
void opcDataChunkUnlock(OPCDataChunkContext ctx);

void opcDataChunkRun(OPCDataChunkContext ctx, OPCDataChunkRunFunction func, void *attr);

/*
#define opcDataChunkGetRepresentationDestructorContext(ctx, id) \
    opcDataDestructorGet(ctx->destructor, id)

#define opcDataChunkGetRepresentationDestructor(ctx, id) \
    opcDataDestructorGetDestructor(ctx->destructor, id)
    
#define opcDataChunkGetRepresentationAllocator(ctx, id) \
    (((ctx)&&(ctx->data)&&(id<=ctx->supported_representations)&&(ctx->data[i].allocator))?ctx->data[i].allocator:opcDataDestructorGetAllocator(ctx->destructor, id))

#define opcDataChunkGetRepresentationAllocatorFromDestructorContext(ctx, id) \
    opcDataDestructorGetAllocator(ctx->destructor, id)
*/

#endif /* _OPC_DATA_CHUNK_H */
