#define _OPC_DATA_REPRESENTATION_C

#include <opcerr.h>
#include <opcassert.h>
#include <opclib_types.h>

#include <opcassert.h>
#include <opcdebug.h>
#include <opcdatabuffer.h>
#include <opcallocatorsimple.h>

#include <opcdatarepresentation.h>

OPCDataRepresentationOptions opcDataRepresentationGetDefaultOptions() {
    int err;
    OPCDataRepresentationOptions options;

    options = (OPCDataRepresentationOptions)malloc(sizeof(struct OPCDataRepresentationOptionsT));
    if (!options) return options;
    
    err = opcDataRepresentationInitDefaultOptions(options);
    if (err) {
	opcDataRepresentationDestroyOptions(options);
	return NULL;
    }
    return options;
}

int opcDataRepresentationInitDefaultOptions(OPCDataRepresentationOptions options) {
    assert(options);
    memcpy(options, &opc_data_representation_options, sizeof(struct OPCDataRepresentationOptionsT));
    return 0;
}

void opcDataRepresentationDestroyOptions(OPCDataRepresentationOptions options) {
    if (options) free(options);
}

int opcDataRepresentationOptionsSetUserDataFunctions(OPCDataRepresentationOptions options, OPCRepresentationGetUserDataFunction GetUserData) {
    assert(options);
    options->GetUserData = GetUserData;
    return 0;
}

int opcDataRepresentationOptionsSetFormat(OPCDataRepresentationOptions options, OPCFormatContext format) {
    assert(options);
    options->format = format;
    return 0;
}

int opcDataRepresentationOptionsSetType(OPCDataRepresentationOptions options, OPCType type) {
    assert(options);
    options->type = type;
    return 0;
}

int opcDataRepresentationOptionsSetBase(OPCDataRepresentationOptions options, OPCRepresentationContext base) {
    assert(options);
    options->base = base;
    return 0;
}

int opcDataRepresentationOptionsSetId(OPCDataRepresentationOptions options, unsigned short int id) {
    assert(options);
    options->id = id;
    return 0;
}

int opcDataRepresentationOptionsSetBufferSize(OPCDataRepresentationOptions options, unsigned long minbuf_size, unsigned long maxbuf_size) {
    assert(options);

    options->maxbuf_size = maxbuf_size;
    options->minbuf_size = minbuf_size;
    return 0;
}

int opcDataRepresentationOptionsSetDefaultAllocatorOptions(OPCDataRepresentationOptions options, OPCBufferPoolContext buffer_pool) {
    assert(options);
    
    options->buffer_pool = buffer_pool;
    return 0;
}

int opcDataRepresentationOptionsSetAllocator(OPCDataRepresentationOptions options, OPCAllocatorContext allocator) {
    assert(options);
    
    options->allocator = allocator;
    return 0;
}

int opcDataRepresentationOptionsSetBaseRW(OPCDataRepresentationOptions options, OPCDataRepresentationReadBaseFunction ReadBase, OPCDataRepresentationWriteBaseFunction WriteBase) {
    assert(options);
    
    options->ReadBase = ReadBase;
    options->WriteBase = WriteBase;
    
    return 0;
}

int opcDataRepresentationOptionsSetFlags(OPCDataRepresentationOptions options, unsigned long flags) {
    assert(options);
    
    options->flags = flags;
    return 0;
}

int opcDataRepresentationOptionsSetSequentialMode(OPCDataRepresentationOptions options) {
    assert(options);
    
    options->flags |= OPC_DATA_FLAG_SEQUENTIAL;
    return 0;
}

static int opcDataRepresentationSetDefaultAllocator(OPCDataRepresentationContext ctx, unsigned char sequential) {
    int err;
    OPCDataBufferOptions options;
    OPCAllocatorContext data;
    OPCFormatContext format;
    OPCType type;


    assert(ctx);
    if (!ctx) return ERR_OPC_PARAM;
    
    format = ctx->format;
    type = ctx->type;
    if ((!format)||(!type)) return ERR_OPC_DATAREPRESENTATION_INCOMPLETE_CONTEXT;

    switch(format->allocator) {
        case OPC_ALLOCATOR_TYPE_FULL:
	    if ((ctx->buffer_pool)&&(sequential)) {
		options=opcDataBufferGetDefaultOptions(ctx->buffer_pool);
		if (!options) return ERR_OPC_MEM;
		if ((ctx->maxbuf_size)||(ctx->minbuf_size))
		    opcDataBufferOptionsSetSize(options,ctx->minbuf_size, ctx->maxbuf_size);
		if (type->size)
		    opcDataBufferOptionsSetAllocationSize(options,type->size);
		opcDataBufferOptionsSetBufferPool(options,ctx->buffer_pool);

		data = (OPCAllocatorContext)opcDataBufferCreateContext(ctx->malloc);
		if (!data) {
		    opcDataBufferDestroyOptions(options);
		    return ERR_OPC_DATAREPRESENTATION_DATABUFFER_CREATE;
		}
	    
		err = opcDataBufferInitContext((OPCDataBufferContext)data, options);
		opcDataBufferDestroyOptions(options);
		
		if (err) opcDataBufferDestroyContext((OPCDataBufferContext)data);
		else ctx->allocator = data;

		break;
	    }
	
	case OPC_ALLOCATOR_TYPE_MINIMAL:
	    data = (OPCAllocatorContext)opcAllocatorSimpleCreateContext(ctx->malloc);
	    if (!data) return ERR_OPC_DATAREPRESENTATION_ALLOCATOR_CREATE;
	    err = opcAllocatorSimpleInitContext((OPCAllocatorSimpleContext)data, NULL);
	    if (err) opcAllocatorSimpleDestroyContext((OPCAllocatorSimpleContext)data);
	    else ctx->allocator = data;
	break;
	
	case OPC_ALLOCATOR_TYPE_ABSENT:
	    ctx->allocator = NULL;
	    err = 0;
	break;

	default:
	    err = ERR_OPC_DATAREPRESENTATION_UNSUPPORTED_ALLOCATOR;
    }
    return err;
}

static int opcDataRepresentationSendAllocation(OPCUserData user_data, OPCSendFunction func, void *attr) {
    OPCDataRepresentationContext ctx;
    int err;
    
    assert(ctx);
    assert(user_data);
    assert(func);
    assert(attr);
    
    if ((!user_data)||(!user_data->info)||(!func)||(!attr)) return ERR_OPC_PARAM;
    ctx = user_data->info->ctx;    
    if (!ctx) return ERR_OPC_PARAM;

    err = opcFormatSend(ctx->format, ctx->type, ctx->allocator, (OPCData)user_data->data, func, attr);
    if (err==ERR_OPC_FORMAT_MISSING_METHOD_SEND) {
	if ((user_data->type==OPC_DATA_TYPE_ALLOCATION)||(user_data->type==OPC_DATA_TYPE_DATABUFFER))
	    return opcAllocatorCall(ctx->allocator, (OPCAllocation)user_data->data, func, attr, 1);
    }
    return err;
}

static void opcDataRepresentationFreeData(OPCDataRepresentationContext ctx, OPCRepresentationData allocation) {
    assert(ctx);

    if (ctx->format->FreeData)
	opcFormatFreeData(ctx->format, ctx->type, ctx->allocator, (OPCData)allocation);
    else
	opcAllocatorFree(ctx->allocator, allocation->data);
    
    opcDataRepresentationUnref(ctx);
}

static inline void opcDataRepresentationZeroContext(OPCDataRepresentationContext ctx) {
    assert(ctx);
    ctx->allocator = NULL;
    ctx->format = NULL;
    ctx->representation.id = 0;
    ctx->base = NULL;
    ctx->type = NULL;
    ctx->lock = NULL;
    ctx->allocator_lock = NULL;
    ctx->grabed_allocator = 0;
}

static void opcDataRepresentationFreeContext(OPCDataRepresentationContext ctx) {
    assert(ctx);
    
    if (ctx->grabed_allocator) opcDataRepresentationReturnAllocator(ctx, ctx->allocator);
    if (ctx->lock) LockDestroyContext(ctx->lock);
    if (ctx->allocator_lock) LockDestroyContext(ctx->allocator_lock);
    if (ctx->allocator) opcAllocatorDestroyContext(ctx->allocator);

    opcDataRepresentationZeroContext(ctx);
}


static void opcDataRepresentationDestroyContext(OPCDataRepresentationContext ctx) {
    opcDataRepresentationFreeContext(ctx);
    opcFree(ctx->malloc, ctx);
}

OPCDataRepresentationContext opcDataRepresentationCreateContext(OPCMallocContext mctx, const char *title) {
    unsigned long len;
    OPCDataRepresentationContext ctx;

    if (title) {
	len = strlen(title);
	ctx = (OPCDataRepresentationContext)opcMalloc(mctx,sizeof(struct OPCDataRepresentationContextT)+len+1);
    } else {
	ctx = (OPCDataRepresentationContext)opcMalloc(mctx,sizeof(struct OPCDataRepresentationContextT));
    }
    
    if (ctx) {
	if (title) {
	    strcpy((char*)ctx+sizeof(struct OPCDataRepresentationContextT), title);
	    ctx->representation.title = (char*)ctx + sizeof(struct OPCDataRepresentationContextT);
	} else ctx->representation.title = NULL;
	ctx->refs = 1;
	ctx->malloc=mctx;
	opcDataRepresentationZeroContext(ctx);
	memcpy(&ctx->representation, &opc_data_representation, sizeof(struct OPCRepresentationContextT));
    }
    return ctx;
}

int opcDataRepresentationInitContext(OPCDataRepresentationContext ctx, OPCDataRepresentationOptions options) {
    int err;
    
    assert(ctx);
    assert(options);
    
    if ((!ctx)||(!options)) return ERR_OPC_PARAM;
    if ((!options->type)||(!options->format)) return ERR_OPC_DATAREPRESENTATION_INCOMPLETE_OPTIONS;
    
    opcDataRepresentationZeroContext(ctx);


    ctx->representation.id = options->id;
    ctx->base = options->base;
    ctx->type = options->type;
    ctx->format = options->format;
    
    switch (ctx->format->type) {
	case OPC_FORMAT_TYPE_OPCXML:
	    ctx->representation.type = OPC_OPCXML_REPRESENTATION;
	break;
	case OPC_FORMAT_TYPE_NATIVE:
	    ctx->representation.type = OPC_NATIVE_REPRESENTATION;
	break;
    }

    if (!ctx->representation.title)
	ctx->representation.title = ctx->format->title;

    ctx->WriteBase = options->WriteBase;
    ctx->ReadBase = options->ReadBase;

    if (options->allocator) ctx->allocator = options->allocator;
    else {
	ctx->maxbuf_size = options->maxbuf_size;
	ctx->minbuf_size = options->minbuf_size;
	ctx->buffer_pool = options->buffer_pool;
	err = opcDataRepresentationSetDefaultAllocator(ctx, options->flags&OPC_DATA_FLAG_SEQUENTIAL);
	if (err) return err;
    }
    
    ctx->representation.repinfo.ctx = ctx;
    ctx->representation.repinfo.malloc = ctx->malloc;

    if ((ctx->format->Send)||(ctx->format->type==OPC_FORMAT_TYPE_NATIVE)||(ctx->format->type==OPC_FORMAT_TYPE_BINARY))
	ctx->representation.repinfo.Send = opcDataRepresentationSendAllocation;
    else
	ctx->representation.repinfo.Send = NULL;
	
    ctx->representation.repinfo.Destroy = NULL;
    ctx->representation.repinfo.destructor_type = OPC_DATA_DESTRUCTOR_RETURN_DATA;
    ctx->representation.repinfo.Destructor = (OPCReturnFunction)opcDataRepresentationFreeData;
    if (options->GetUserData) ctx->representation.repinfo.GetUserData = options->GetUserData;
    else ctx->representation.repinfo.GetUserData = NULL;

    ctx->lock = LockCreateContext(ctx->malloc);
    if ((!ctx->lock)) {
	opcDataRepresentationFreeContext(ctx);
	return ERR_OPC_MEM;
    }
    err=LockInitContext(ctx->lock);
    if (err) {
	opcDataRepresentationFreeContext(ctx);
	return err;
    }
    
    if ((ctx->allocator)&&(!strcmp(ctx->allocator->allocator.name, "DataBuffer"))) {
	ctx->allocator_lock = LockCreateContext(ctx->malloc);
	if (!ctx->allocator_lock) {
	    opcDataRepresentationFreeContext(ctx);
	    return ERR_OPC_MEM;
	}
	
	err = LockInitContext(ctx->allocator_lock);
	if (err) {
	    opcDataRepresentationFreeContext(ctx);
	    return err;
	}
    }

    return 0;
}


int opcDataRepresentationPushData(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk, const void *data, unsigned long size) {
    int err;
    OPCRepresentationData data_alloc;

    assert(ctx);
    assert(chunk);
    assert(data);
    
    if ((!ctx)||(!chunk)||(!data)) return ERR_OPC_PARAM;
    
    data_alloc = opcDataChunkGetData(chunk, ctx->representation.id);

    opcDataChunkLock(chunk);
    if (ctx->allocator_lock) LockWrite(ctx->allocator_lock);
    err = opcFormatImportData(ctx->format, ctx->type, ctx->allocator, (OPCData)data_alloc, data, size);
    if (ctx->allocator_lock) LockReturn(ctx->allocator_lock);
    opcDataChunkUnlock(chunk);
    
    if (!err) opcDataRepresentationRef(ctx);

    return err;
}

int opcDataRepresentationPushAllocation(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk, OPCAllocatorContext allocator, OPCAllocation allocation) {
    OPCRepresentationData data;

    assert(ctx);
    assert(chunk);
    assert(allocation);

    if ((!ctx)||(!chunk)||(!allocation)||(!allocator)||(ctx->allocator!=allocator)) return ERR_OPC_PARAM;
    
    data = opcDataChunkGetData(chunk, ctx->representation.id);

    opcDataChunkLock(chunk);
    data->type = OPC_DATA_TYPE_ALLOCATION;
    data->data = (void*)allocation;
    data->size = opcAllocatorSizeof(ctx->allocator, allocation, 1);
    opcDataChunkUnlock(chunk);

    opcDataRepresentationRef(ctx);

    return 0;
}

int opcDataRepresentationRef(OPCDataRepresentationContext ctx) {
    assert(ctx);

    if (!ctx) return ERR_OPC_PARAM;
    
    LockWrite(ctx->lock);
    ctx->refs++;
    LockReturn(ctx->lock);

    return 0;
}

void opcDataRepresentationUnref(OPCDataRepresentationContext ctx) {
    assert(ctx);

    if (!ctx) return;
    if (!ctx->lock) {
	opcDataRepresentationDestroyContext(ctx);
	return;
    }
    
    LockWrite(ctx->lock);
    if ((--ctx->refs)==0) {
	LockReturn(ctx->lock);
	opcDataRepresentationDestroyContext(ctx);
	return;
    }
    LockReturn(ctx->lock);
}

OPCAllocatorContext opcDataRepresentationGetAllocator(OPCDataRepresentationContext ctx, OPCDataInfo info) {
    assert(ctx);
    if (!ctx) return;    

    if (ctx->allocator) {
	if (ctx->allocator_lock) LockWrite(ctx->allocator_lock);
/*	opcDataRepresentationRef(ctx);*/
	ctx->grabed_allocator++;
	return ctx->allocator;
    }
    return NULL;
}


void opcDataRepresentationReturnAllocator(OPCDataRepresentationContext ctx, OPCAllocatorContext allocator) {
    assert(ctx);
    assert(allocator==ctx->allocator);
    
    if (ctx->grabed_allocator>0) {
	if ((!ctx)||(allocator)||(allocator!=ctx->allocator)) return;
	
	ctx->grabed_allocator--;
        if (ctx->allocator_lock) LockReturn(ctx->allocator_lock);
/*	opcDataRepresentationUnref(ctx);*/
    }
}


int opcDataRepresentationConvertData(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk) {
    int err;
    unsigned short int id, baseid;
    OPCRepresentationContext base;
    OPCRepresentationData data, basedata;
    OPCRepresentationType type, basetype;
    long i;
    
    assert(ctx);
    assert(chunk);
    
    if ((!ctx)||(!chunk)) return ERR_OPC_PARAM;
    
    id = ctx->representation.id;
    data = opcDataChunkGetData(chunk, id);
    if (data->data) return 0;
    base = ctx->base;
    if (base) {
	baseid = base->id;
	basedata = opcDataChunkGet(chunk, baseid);
        if (!basedata) {
	    err=opcRepresentationConvertData(base,chunk);
	    if (err) return err;
	    basedata = opcDataChunkGet(chunk, baseid);
	}

	opcDebugServer(OPC_DEBUG_DEBUG, "opcDataRepresentation: Convert: Rep%u reading from Rep%u",id, base->id);
	
	type = ctx->representation.type;
	
	if (basetype == OPC_NATIVE_REPRESENTATION) {
	    opcDataChunkLock(chunk);
	    if (ctx->allocator_lock) LockWrite(ctx->allocator_lock);
	    err = opcFormatReadNative(ctx->format, ctx->type, ctx->allocator, (OPCData)data, (OPCData)basedata);
	    if (ctx->allocator_lock) LockReturn(ctx->allocator_lock);
	    opcDataChunkUnlock(chunk);
	} else if (ctx->ReadBase) {
	    err = ctx->ReadBase(ctx, chunk);
	} else {
	    err = ERR_OPC_DATAREPRESENTATION_CONVERT;
	}
    } else {
	for (i=chunk->supported_representations;i>=0;i++) {
	    basedata = opcDataChunkGet(chunk, i);
	    if ((basedata)&&(basedata->info)) {
		base = (OPCRepresentationContext)basedata->info->ctx;
		if (!base) continue;
		basetype = base->type;
		if ((basetype!=OPC_DATA_REPRESENTATION)&&(basetype!=OPC_NATIVE_REPRESENTATION)&&(basetype!=OPC_OPCXML_REPRESENTATION)) continue;
		if (((OPCDataRepresentationContext)base)->base != (OPCRepresentationContext)ctx) continue;
		
		opcDebugServer(OPC_DEBUG_DEBUG, "opcDataRepresentation: Convert: Rep%u write to Rep%u",base->id, id);
		
		type = ctx->representation.type;
		if (type == OPC_NATIVE_REPRESENTATION) {
		    opcDataChunkLock(chunk);
		    if (ctx->allocator_lock) LockWrite(ctx->allocator_lock);
		    err = opcFormatWriteNative(((OPCDataRepresentationContext)base)->format, ((OPCDataRepresentationContext)base)->type, ctx->allocator, (OPCData)data, (OPCData)basedata);
		    if (ctx->allocator_lock) LockReturn(ctx->allocator_lock);
		    opcDataChunkUnlock(chunk);
		} else if (((OPCDataRepresentationContext)base)->WriteBase) {
		    err = ((OPCDataRepresentationContext)base)->WriteBase((OPCDataRepresentationContext)base, chunk);
		} else {
		    err = ERR_OPC_DATAREPRESENTATION_CONVERT;
		}
		break;
	    }
	}
	
	if (i<0) {
	    for (i=chunk->supported_representations;i>=0;i++) {
		basedata = opcDataChunkGetData(chunk, i);
		if ((!basedata)||(!basedata->info)) continue;
		base = (OPCRepresentationContext)basedata->info->ctx;
		if (!base) continue;

		basetype = base->type;
		if ((basetype!=OPC_DATA_REPRESENTATION)&&(basetype!=OPC_NATIVE_REPRESENTATION)&&(basetype!=OPC_OPCXML_REPRESENTATION)) continue;
		if (((OPCDataRepresentationContext)base)->base != (OPCRepresentationContext)ctx) continue;

	        if (!basedata->data) {
		    err=opcRepresentationConvertData(base,chunk);
		    if (err) continue;
		    basedata = opcDataChunkGet(chunk, baseid);
		}
		
		opcDebugServer(OPC_DEBUG_DEBUG, "opcDataRepresentation: Convert: Rep%u write to Rep%u",base->id, id);
		
		type = ctx->representation.type;
		if (type == OPC_NATIVE_REPRESENTATION) {
		    opcDataChunkLock(chunk);
		    if (ctx->allocator_lock) LockWrite(ctx->allocator_lock);
		    err = opcFormatWriteNative(((OPCDataRepresentationContext)base)->format, ((OPCDataRepresentationContext)base)->type, ctx->allocator, (OPCData)data, (OPCData)basedata);
		    if (ctx->allocator_lock) LockReturn(ctx->allocator_lock);
		    opcDataChunkUnlock(chunk);
		} else if (((OPCDataRepresentationContext)base)->WriteBase) {
		    err = ((OPCDataRepresentationContext)base)->WriteBase((OPCDataRepresentationContext)base, chunk);
		} else {
		    err = ERR_OPC_DATAREPRESENTATION_CONVERT;
		}
		break;
	    }
	    if (i<0) err = ERR_OPC_DATAREPRESENTATION_CONVERT;
	}
    }

    if (!err) opcDataRepresentationRef(ctx);

    return err;
}

int opcDataRepresentationGet(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk, OPCRepresentationData *rep) {
    int err;
    unsigned short int id;
    OPCRepresentationData data;
    
    assert(ctx);
    assert(chunk);
    assert(rep);

    if ((!ctx)||(!chunk)||(!rep)) return ERR_OPC_PARAM;
    
    id = ctx->representation.id;
    data=opcDataChunkGet(chunk, id);
    if (!data) {
	err=opcDataRepresentationConvertData(ctx,chunk);
	if (err) return err;
	data=opcDataChunkGet(chunk, id);
    }
            
    *rep = data;
    return 0;
}


int opcDataRepresentationMantainAllocator(OPCDataRepresentationContext ctx) {
    assert(ctx);
    opcAllocatorMantain(ctx->allocator);
}
