#ifndef _OPC_DATA_REPRESENTATION_H
#define _OPC_DATA_REPRESENTATION_H

#include <opcformat.h>
#include <opcallocator.h>
#include <opcrepresentation.h>

typedef struct OPCDataRepresentationContextT *OPCDataRepresentationContext;
typedef struct OPCDataRepresentationOptionsT *OPCDataRepresentationOptions;

typedef int (*OPCDataRepresentationReadBaseFunction)(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk);
typedef int (*OPCDataRepresentationWriteBaseFunction)(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk);



struct OPCDataRepresentationContextT {
    struct OPCRepresentationContextT representation;
    
    OPCRepresentationContext base;
    
    OPCDataRepresentationReadBaseFunction ReadBase;
    OPCDataRepresentationWriteBaseFunction WriteBase;

    LockContext lock;
    LockContext allocator_lock;

    OPCMallocContext malloc;
    OPCAllocatorContext allocator;
    OPCFormatContext format;
    OPCType type;
    unsigned long refs;

    OPCBufferPoolContext buffer_pool;
    unsigned long minbuf_size;
    unsigned long maxbuf_size;
    
    unsigned char grabed_allocator;
};


struct OPCDataRepresentationOptionsT {
    OPCAllocatorContext allocator;
    OPCDataRepresentationReadBaseFunction ReadBase;
    OPCDataRepresentationWriteBaseFunction WriteBase;
    OPCRepresentationGetUserDataFunction GetUserData;
    OPCFormatContext format;
    OPCType type;
    OPCRepresentationContext base;
    unsigned short int id;
    OPCBufferPoolContext buffer_pool;
    unsigned long minbuf_size;
    unsigned long maxbuf_size;
    unsigned long flags;
};


OPCDataRepresentationOptions opcDataRepresentationGetDefaultOptions();
int opcDataRepresentationInitDefaultOptions(OPCDataRepresentationOptions options);
void opcDataRepresentationDestroyOptions(OPCDataRepresentationOptions options);

int opcDataRepresentationOptionsSetUserDataFunctions(OPCDataRepresentationOptions options, OPCRepresentationGetUserDataFunction GetUserData);
int opcDataRepresentationOptionsSetFormat(OPCDataRepresentationOptions options, OPCFormatContext format);
int opcDataRepresentationOptionsSetType(OPCDataRepresentationOptions options, OPCType type);
int opcDataRepresentationOptionsSetBase(OPCDataRepresentationOptions options, OPCRepresentationContext base);
int opcDataRepresentationOptionsSetId(OPCDataRepresentationOptions options, unsigned short int id);
int opcDataRepresentationOptionsSetBufferSize(OPCDataRepresentationOptions options, unsigned long minbuf_size, unsigned long maxbuf_size);
int opcDataRepresentationOptionsSetDefaultAllocatorOptions(OPCDataRepresentationOptions options, OPCBufferPoolContext buffer_pool);
int opcDataRepresentationOptionsSetAllocator(OPCDataRepresentationOptions options, OPCAllocatorContext allocator);
int opcDataRepresentationOptionsSetBaseRW(OPCDataRepresentationOptions options, OPCDataRepresentationReadBaseFunction ReadBase, OPCDataRepresentationWriteBaseFunction WriteBase);
int opcDataRepresentationOptionsSetFlags(OPCDataRepresentationOptions options, unsigned long flags);
int opcDataRepresentationOptionsSetSequentialMode(OPCDataRepresentationOptions options);


OPCDataRepresentationContext opcDataRepresentationCreateContext(OPCMallocContext mctx, const char *title);
int opcDataRepresentationInitContext(OPCDataRepresentationContext ctx, OPCDataRepresentationOptions options);
int opcDataRepresentationRef(OPCDataRepresentationContext ctx);
void opcDataRepresentationUnref(OPCDataRepresentationContext ctx);

int opcDataRepresentationPushData(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk, const void *data, unsigned long size);
int opcDataRepresentationPushAllocation(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk, OPCAllocatorContext allocator, OPCAllocation allocation);
OPCAllocatorContext opcDataRepresentationGetAllocator(OPCDataRepresentationContext ctx, OPCDataInfo info);
void opcDataRepresentationReturnAllocator(OPCDataRepresentationContext ctx, OPCAllocatorContext allocator);
int opcDataRepresentationConvertData(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk);
int opcDataRepresentationGet(OPCDataRepresentationContext ctx, OPCDataChunkContext chunk, OPCRepresentationData *rep);
int opcDataRepresentationMantainAllocator(OPCDataRepresentationContext ctx);



#ifdef _OPC_DATA_REPRESENTATION_C
static struct OPCRepresentationContextT  opc_data_representation = {
    NULL,
    OPC_DATA_REPRESENTATION,
    (OPCRepresentationPushDataFunction)opcDataRepresentationPushData,
    (OPCRepresentationPushAllocationFunction)opcDataRepresentationPushAllocation,
    (OPCRepresentationGetAllocatorFunction)opcDataRepresentationGetAllocator,
    (OPCRepresentationReturnAllocatorFunction)opcDataRepresentationReturnAllocator,
    (OPCRepresentationGetDataFunction)opcDataRepresentationGet,
    (OPCRepresentationConvertDataFunction)opcDataRepresentationConvertData,
    (OPCRepresentationRefFunction)opcDataRepresentationRef,
    (OPCRepresentationUnrefFunction)opcDataRepresentationUnref,
    (OPCRepresentationMantainFunction)opcDataRepresentationMantainAllocator,
};


static const struct OPCDataRepresentationOptionsT opc_data_representation_options = {
    NULL, 	/* allocator */
    NULL, 	/* ReadBase */
    NULL, 	/* WriteBase */
    NULL,	/* GetUserData */
    NULL, 	/* Format by user */
    NULL, 	/* Type by user */
    NULL, 	/* Base by user */
    0,		/* Id by user */
    NULL,	/* BufferPool */
    0,		/* MinBuf Size */
    0,		/* MaxBuf Size */
    0		/* Flags */
};

#endif /* _OPC_DATA_REPRESENTATION_C */

#endif /* _OPC_DATA_REPRESENTATION_H */
