#ifndef _OPC_REPRESENTATION_H
#define _OPC_REPRESENTATION_H

#include <opcdatachunk.h>
#include <opcallocator.h>
#include <opcuserdata.h>
#include <opcdatabuffer.h>

typedef struct OPCRepresentationContextT *OPCRepresentationContext;

typedef int (*OPCRepresentationPushDataFunction)(OPCRepresentationContext ctx, OPCDataChunkContext chunk, const void *data, unsigned long size);
typedef int (*OPCRepresentationPushAllocationFunction)(OPCRepresentationContext ctx, OPCDataChunkContext chunk, OPCAllocatorContext allocator, OPCAllocation allocation);
typedef int (*OPCRepresentationSetDefaultInfoFunction)(OPCRepresentationContext ctx, OPCRepresentationInfo definfo);

typedef OPCAllocatorContext (*OPCRepresentationGetAllocatorFunction)(OPCRepresentationContext ctx, OPCDataInfo info);
typedef void (*OPCRepresentationReturnAllocatorFunction)(OPCRepresentationContext ctx, OPCAllocatorContext allocator);

typedef int (*OPCRepresentationGetDataFunction)(OPCRepresentationContext ctx, OPCDataChunkContext chunk, OPCRepresentationData *rep);
typedef int (*OPCRepresentationConvertDataFunction)(OPCRepresentationContext ctx, OPCDataChunkContext chunk);

typedef int (*OPCRepresentationRefFunction)(OPCRepresentationContext ctx);
typedef void (*OPCRepresentationUnrefFunction)(OPCRepresentationContext ctx);
typedef void (*OPCRepresentationMantainFunction)(OPCRepresentationContext ctx);


enum OPCRepresentationTypeT {
    OPC_UNKNOWN_REPRESENTATION = 0,
    OPC_NATIVE_REPRESENTATION,
    OPC_DATA_REPRESENTATION,
    OPC_OPCXML_REPRESENTATION,
    OPC_XML_REPRESENTATION 
};
typedef enum OPCRepresentationTypeT OPCRepresentationType;

struct OPCRepresentationContextT {
    char *title;
    OPCRepresentationType type;
    OPCRepresentationPushDataFunction PushData;
    OPCRepresentationPushAllocationFunction PushAllocation;
    OPCRepresentationGetAllocatorFunction GetAllocator;
    OPCRepresentationReturnAllocatorFunction ReturnAllocator;
    OPCRepresentationGetDataFunction GetData;
    OPCRepresentationConvertDataFunction ConvertData;
    OPCRepresentationRefFunction Ref;
    OPCRepresentationUnrefFunction Unref;
    OPCRepresentationMantainFunction Mantain;
    OPCRepresentationInfoStr repinfo;
    unsigned short int id;
};

#define opcRepresentationRef(ctx) \
    ((ctx&&ctx->Ref)?ctx->Ref(ctx):ERR_OPC_NOTIMPLEMENTED)
#define opcRepresentationPushData(ctx,chunk,data,size) \
    ((ctx&&ctx->PushData)?ctx->PushData(ctx,chunk,data,size):ERR_OPC_NOTIMPLEMENTED)
#define opcRepresentationPushAllocation(ctx,chunk,allocator,allocation) \
    ((ctx&&ctx->PushAllocation)?ctx->PushAllocation(ctx,chunk,allocator,allocation):ERR_OPC_NOTIMPLEMENTED)
#define opcRepresentationGetAllocator(ctx,info) \
    ((ctx&&ctx->GetAllocator)?ctx->GetAllocator(ctx,info):NULL)
#define opcRepresentationReturnAllocator(ctx,allocator) \
    ((ctx&&ctx->ReturnAllocator)?ctx->ReturnAllocator(ctx,allocator):ERR_OPC_NOTIMPLEMENTED)
#define opcRepresentationGetData(ctx, chunk, rep) \
    ((ctx&&ctx->GetData)?ctx->GetData(ctx,chunk,rep):ERR_OPC_NOTIMPLEMENTED)
#define opcRepresentationConvertData(ctx, chunk) \
    ((ctx&&ctx->ConvertData)?ctx->ConvertData(ctx,chunk):ERR_OPC_NOTIMPLEMENTED)
#define opcRepresentationMantain(ctx) \
    ((ctx&&ctx->Mantain)?ctx->Mantain(ctx):ERR_OPC_NOTIMPLEMENTED)
    

void opcRepresentationUnref(OPCRepresentationContext ctx);
int opcRepresentationSetId(OPCRepresentationContext ctx, unsigned short int repid);
int opcRepresentationDataDataBuffer(OPCRepresentationInfo repinfo, OPCDataBufferContext db);

#endif /* _OPC_REPRESENTATION_H */
