#include <opcerr.h>
#include <opcassert.h>
#include <opcdatabuffer.h>
#include <opcallocator.h>
#include <opcdatafast.h>
#include <opcformatopcxml.h>

#include <opcuserdata.h>

int opcUserDataSendAllocation(OPCUserData user_data, OPCSendFunction func, void *attr) {
    if ((!user_data)||(user_data->info)) return ERR_OPC_PARAM;
    
    return opcAllocatorCall(((OPCAllocatorContext)user_data->info->ctx), ((OPCAllocation)user_data->data), func, attr, 1);
}


static void opcUserDataDestroyDefault(OPCUserData user_data) {
    if (user_data) {
	if (user_data->info) opcFree(user_data->info->malloc, user_data);
	else opcFree(NULL, user_data);
    }
}

int opcUserDataGetDefault(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data) {
    OPCUserDataAllocation data;
    void *ctx;
    
    if ((!user_data)||(!rep)||(!rep->info)) return ERR_OPC_PARAM;
    
    data = (OPCUserDataAllocation)opcMalloc(rep->info->malloc, sizeof(OPCUserDataAllocationStr));
    memcpy(&data,rep,sizeof(struct OPCDataT));
    data->user_data.info = &data->user_info;
    
    data->user_info.ctx = rep->info->ctx;
    data->user_info.Destroy = opcUserDataDestroyDefault;
    data->user_info.malloc = rep->info->malloc;
    
    switch(rep->type) {
	case OPC_DATA_TYPE_DATABUFFER:
	    data->user_info.Send = opcDataBufferSend;
	break;
	case OPC_DATA_TYPE_ALLOCATION:
	    data->user_info.Send = opcUserDataSendAllocation;
	break;
	case OPC_DATA_TYPE_XMLNODEPTR:
	default:
	    data->user_info.Send = NULL;
    }

    return 0;
}

static void opcUserDataDestroyOPCXML(OPCUserData user_data) {
    assert(user_data);
    opcFormatOPCXMLFree((OPCFormatOPCXMLContext)user_data->info->ctx,NULL,NULL,(OPCData)user_data);
    free(user_data);
}

static void opcUserDataDestroyFree(OPCUserData user_data) {
    assert(user_data);
    free(user_data);
}


int opcUserDataFastGetOPCXML(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data) {
    int err;
    OPCDataFastInfo repinfo;
    OPCType type;
    OPCFormatOPCXMLContext format;
    OPCUserInfo info;
    OPCUserDataAllocation data;
    void *ctx;

    if ((!user_data)||(!options)||(!rep)) return ERR_OPC_PARAM;
    if (rep->type!=OPC_DATA_TYPE_DATABUFFER) return ERR_OPC_PARAM;
    repinfo = (OPCDataFastInfo)rep->info;
    if (!repinfo) return ERR_OPC_PARAM;
    if ((!repinfo->formats)||(!repinfo->datafast)) return ERR_OPC_PARAM;
    format=(OPCFormatOPCXMLContext)repinfo->formats->opcxml_format;
    type=repinfo->datafast->type;
    if ((!format)||(!type)) return ERR_OPC_PARAM;
    
    data = (OPCUserDataAllocation)malloc(sizeof(OPCUserDataAllocationStr));
    data->user_data.info = &data->user_info;
    info = data->user_data.info;

    info->ctx = (void*)format;
    info->malloc = NULL;
    info->Send = NULL;
    
    if (options->flags&OPC_REQUEST_OPTIONS_FLAG_XMLNODE_BEAR)
        info->Destroy = opcUserDataDestroyFree;
    else
        info->Destroy = opcUserDataDestroyOPCXML;

    err=opcFormatOPCXMLReadNative(format, type, rep->info->ctx, (OPCData)data, (OPCData)rep);
    if (err) free(data);
    else *user_data = (OPCUserData)data;

    return err;
}


int opcUserDataGetSelf(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data) {
    if (!user_data) return ERR_OPC_PARAM;
    *user_data = (OPCUserData)rep;
    return 0;
}

