#ifndef _OPC_USER_DATA_H
#define _OPC_USER_DATA_H

#include <opclib_types.h>
#include <opcmalloc.h>

#define OPC_DATA_FLAG_SEQUENTIAL 1

enum OPCDataTypeT {
    OPC_DATA_TYPE_UNKNOWN = 0,
    OPC_DATA_TYPE_DATABUFFER,
    OPC_DATA_TYPE_ALLOCATION,
    OPC_DATA_TYPE_XMLNODEPTR
};

typedef enum OPCDataTypeT OPCDataType;

enum OPCDataDestructorTypeT {
    OPC_DATA_DESTRUCTOR_FREE = 0,
    OPC_DATA_DESTRUCTOR_DESTROY,
    OPC_DATA_DESTRUCTOR_RETURN,
    OPC_DATA_DESTRUCTOR_DESTROY_DATA,
    OPC_DATA_DESTRUCTOR_RETURN_DATA
};
typedef enum OPCDataDestructorTypeT OPCDataDestructorType;


typedef struct OPCDataT *OPCData;
typedef struct OPCRepresentationDataT *OPCRepresentationData;
typedef struct OPCUserDataT *OPCUserData;
typedef struct OPCUserDataT OPCUserDataStr;
typedef struct OPCRequestOptionsT *OPCRequestOptions;
typedef struct OPCRequestOptionsT OPCRequestOptionsStr;
typedef struct OPCRepresentationInfoT *OPCRepresentationInfo;
typedef struct OPCRepresentationInfoT OPCRepresentationInfoStr;
typedef struct OPCUserInfoT *OPCUserInfo;

typedef void (*OPCDestroyDataContextFunction)(OPCData);
typedef void (*OPCReturnDataFunction)(void *, OPCData);

typedef int (*OPCRepresentationGetUserDataFunction)(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data);
typedef void (*OPCUserDataDestroyFunction)(OPCUserData user_data);
typedef int (*OPCUserDataSendFunction)(OPCUserData user_data, OPCSendFunction function, void *attr);
typedef void (*OPCRepresentationDestroyFunction)();

struct OPCDataT {
    OPCDataType type;
    void *data;
    unsigned long size;
};

struct OPCUserInfoT {
    void *ctx;
    OPCMallocContext malloc;

    OPCUserDataSendFunction Send;
    OPCUserDataDestroyFunction Destroy; 
};

struct OPCRepresentationInfoT {
    void *ctx;
    OPCMallocContext malloc;

    OPCUserDataSendFunction Send;		/* Only required for mixed User/Rep structures */
    OPCUserDataDestroyFunction Destroy; 	/* Only required for mixed User/Rep structures */

    OPCDataDestructorType destructor_type;
    OPCRepresentationDestroyFunction Destructor;

    OPCRepresentationGetUserDataFunction GetUserData;
};



struct OPCRepresentationDataT {
    OPCDataType type;		/* Should be consistent with OPCData structure */
    void *data;			/* Should be consistent with OPCData structure */
    unsigned long size;		/* Should be consistent with OPCData structure */

    OPCRepresentationInfo info;
};

struct OPCUserDataT {
    OPCDataType type;		/* Should be consistent with OPCData structure */
    void *data;			/* Should be consistent with OPCData structure */
    unsigned long size;		/* Should be consistent with OPCData structure */

    OPCUserInfo info;
};

struct OPCUserDataAllocationT {
    struct OPCUserDataT user_data;
    struct OPCUserInfoT user_info;
};
typedef struct OPCUserDataAllocationT OPCUserDataAllocationStr;
typedef struct OPCUserDataAllocationT *OPCUserDataAllocation;

#define OPC_REQUEST_OPTIONS_FLAG_XMLNODE_BEAR 1
struct OPCRequestOptionsT {
    unsigned long flags;
    const char *locale;
    const void *security;
    const char *subrepresentation;
};

int opcUserDataSendAllocation(OPCUserData user_data, OPCSendFunction func, void *attr);
int opcUserDataGetDefault(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data);
int opcUserDataGetSelf(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data);
int opcUserDataFastGetOPCXML(OPCRepresentationData rep, OPCRequestOptions options, OPCUserData *user_data);

#define opcUserDataGet(rep,options,user_data) \
    ((rep&&rep->info&&rep->info->GetUserData)?rep->info->GetUserData(rep,options,user_data):opcUserDataGetSelf(rep,options,user_data))
#define opcUserDataSend(user_data,func,ctx) \
    ((user_data&&user_data->info&&user_data->info->Send)?user_data->info->Send(user_data,func,ctx):ERR_OPC_NOTIMPLEMENTED)
#define opcUserDataDestroy(user_data) \
    ((user_data&&user_data->info&&user_data->info->Destroy)?user_data->info->Destroy(user_data):ERR_OPC_NOTIMPLEMENTED)

#endif /* _OPC_USER_DATA_H */
