#define _OPC_REQUEST_C
#include <stdio.h>
#include <assert.h>

#include <libxml/parser.h>
#include <libxml/encoding.h>
#include <libxml/xmlsave.h>

#include <hashtable.h>
#include <opcxml.h>

#include <opcerr.h>
#include <opcconfig.h>

#include <opc.h>
#include <opcconfiguration.h>
#include <opcrequest.h>
#include <opcrequestsecurity.h>
#include <opcrequestprocess.h>

OPCRequestContext opcRequestCreateContext() {
    OPCRequestContext ctx;
    ctx = (OPCRequestContext)malloc(sizeof(struct OPCRequestContextT));
    if (ctx) {
	ctx->parser_ctx=NULL;
	ctx->doc=NULL;
	ctx->adoc=NULL;
	
	ctx->opc_commands=NULL;
	ctx->sctx=NULL;
	
//	ctx->start_time=NULL;
//	ctx->end_time=NULL;
	ctx->client_request_handle=NULL;

	ctx->extensions=0;
	ctx->locale=NULL;
    }
    return ctx;
}

int opcRequestInitContext(OPCRequestContext ctx, OPCContext sctx, HashTable opc_commands) {
    assert(ctx);
    
    
    ctx->sctx=sctx;
    ctx->opc_commands=opc_commands;
    
    return 0;
}

void opcRequestFreeContext(OPCRequestContext ctx) {
    assert(ctx);
    if (ctx->parser_ctx) {
	xmlFreeParserCtxt(ctx->parser_ctx);
	ctx->parser_ctx=NULL;
    }
    if (ctx->doc) {
	xmlFreeDoc(ctx->doc);
	ctx->doc=NULL;
    }
    if (ctx->adoc) {
	xmlFreeDoc(ctx->adoc);
	ctx->adoc=NULL;
    }
/*    if (ctx->start_time) {
	opcTimeDestroy(ctx->start_time);
	ctx->start_time=NULL;
    }
    if (ctx->end_time) {
	opcTimeDestroy(ctx->end_time);
	ctx->end_time=NULL;
    }*/
    if (ctx->client_request_handle) {
	xmlFree(ctx->client_request_handle);
	ctx->client_request_handle=NULL;
    }

    ctx->extensions=0;
    ctx->locale=NULL;
}

void opcRequestDestroyContext(OPCRequestContext ctx) {
    assert(ctx);
    opcRequestFreeContext(ctx);
    free(ctx);
}

int opcRequestReturnParserError(int err) {
    switch (err) {
	case 0:
	    return 0;
	default:
	    return ERR_OPC_XLIB_PARSEREQUEST;
    }
}

int opcRequestXMLData(OPCRequestContext ctx, char *data, unsigned long length) {
    int err;
    int preprocessed=0;
    if (length<4) return ERR_OPC_XLIB_CHUNKSMALL;
    if (!ctx->parser_ctx) {
	ctx->parser_ctx=xmlCreatePushParserCtxt(NULL,NULL,data,4,"OPCRequest");
        if (!ctx->parser_ctx) return ERR_OPC_XLIB_CREATEPARSER;
#ifndef _DEBUG_XML
//	xmlSetFeature(ctx->parser_ctx,"SAX function error",&func);
	ctx->parser_ctx->sax->error=NULL;
#endif /* _DEBUG_XML */
	preprocessed=4;
    }
    err=xmlParseChunk(ctx->parser_ctx,data+preprocessed,length-preprocessed,0);
    
    if (err) return opcRequestReturnParserError(err);
    return 0;
}

/*
int opcRequestDebug(OPCRequestContext ctx) {
    int size;
    char *mem;
    
    xmlDocDumpMemory(ctx->doc,&mem,&size);
    puts("++++++++++++++++++++++++++++++++++++++++++");
    puts(mem);
    puts("++++++++++++++++++++++++++++++++++++++++++");
    free(mem);
}
*/

int opcRequestProcess(OPCRequestContext ctx, OPCRequestOpenAttachmentFunction open_attachment, OPCSendFunction send_attachment, OPCRequestCloseAttachmentFunction close_attachment, void *cctx) {
    int err;
    xmlChar *answer;
    xmlNodePtr root, tnode;
    xmlNsPtr ns;
    
	// Parse Request
    err=xmlParseChunk(ctx->parser_ctx,NULL,0,1);
    if (err) return opcRequestReturnParserError(err);
    ctx->doc=ctx->parser_ctx->myDoc;
    xmlFreeParserCtxt(ctx->parser_ctx);
    ctx->parser_ctx=NULL;
    
	// Attachment Send Function
    ctx->open_attachment = open_attachment;
    ctx->send_attachment = send_attachment;
    ctx->close_attachment = close_attachment;
    ctx->cctx = cctx;
    
//    opcRequestDebug(ctx);

	// Rip SOAP headers (if any)
    root = xmlDocGetRootElement(ctx->doc);
    if (!root) return ERR_OPC_XLIB_REQUESTEMPTY;

/*  DS: Temporary some problems with finding namespaces    
    ns = xmlSearchNsByHref(ctx->doc,root,(const xmlChar*)opchdr_namespace);
    if (ns) ctx->extensions|=OPC_EXTENSION_HDR;
    else ns = xmlSearchNsByHref(ctx->doc,root,(const xmlChar*)opc_namespace);
    if (!ns) return ERR_OPC_XLIB_NSNOTFOUND;
    
    ctx->req=xmlFindNode(root,NULL,ns);
*/
    ctx->req=xmlFindNodeByHref(root,NULL,ctx->sctx->config->namespaces.opc_namespace);
    if (!ctx->req) return ERR_OPC_XLIB_NSNOTFOUND;
    

    if (ctx->req==root) ctx->soap=0;
    else {
	tnode=xmlFindNodeByHref(root,"Envelope",NULL);
	if (tnode) ctx->soapns=tnode->nsDef;
	else ctx->soapns=NULL;
	ctx->soap=1;
    }
    
	// Security Parse
    err=opcRequestSecurityRequest(ctx);
    
	// Validate

	// OPC Parse / Create
    err=opcRequestProcessOPC(ctx);
	    
	// Security Create
    err=opcRequestSecurityResponse(ctx);

	// Build Answer
//    xmlDocDumpMemory(ctx->adoc,&answer,&ctx->answer_size);
//    xmlDocDumpMemoryEnc(ctx->adoc,&answer,&ctx->answer_size,ctx->charset);
//    ctx->answer=(char*)answer;
//    puts("--------------------------- Finished -----------------------------------");
//    puts(ctx->answer);
    return 0;
}

int opcRequestSend(OPCRequestContext ctx, xmlOutputWriteCallback func_write, xmlOutputCloseCallback func_close, void *cctx) {
    int err = 0;

#ifdef opc_supported_xmlsave
    xmlSaveCtxtPtr save;
#else /* opc_supported_xmlsave */
    xmlChar *answer;
    int answer_size;
#endif /* opc_supported_xmlsave */


    assert(ctx);
    if (!ctx) return ERR_OPC_PARAM;
    if (!func_write) return 0;

#ifdef opc_supported_xmlsave
    save = xmlSaveToIO(func_write, NULL, cctx, ctx->charset, 0);
    if (!save) return ERR_OPC_XLIB_SERIALIZE_INIT;
    if (xmlSaveDoc(save, ctx->adoc)<0) err = ERR_OPC_XLIB_SERIALIZE;
    if (xmlSaveClose(save)<0) 
	if (!err) err = ERR_OPC_XLIB_SERIALIZE_FINALIZE;
#else /* opc_supported_xmlsave */
    xmlDocDumpMemoryEnc(ctx->adoc,&answer,&answer_size,ctx->charset);
    if (!answer) return ERR_OPC_XLIB_SERIALIZE_INIT;
    if (func_write(cctx, (void*)answer, answer_size)<0) err = ERR_OPC_XLIB_SERIALIZE;
    if ((func_close)&&(func_close(cctx)<0))
	if (!err) err = ERR_OPC_XLIB_SERIALIZE;
    xmlFree(answer);
#endif /* opc_supported_xmlsave */
    return err;
}
