#ifndef _OPC_REQUEST_H
#define _OPC_REQUEST_H

#include <libxml/parser.h>

#include <hashtable.h>

typedef struct OPCRequestContextT *OPCRequestContext;

#include <opc.h>
#include <opcinterface.h>
#include <opcrequestprocess.h>

#define OPC_EXTENSION_HDR 1

typedef void *(*OPCRequestOpenAttachmentFunction)(void *ctx, const char *item, OPCDataInfo info, OPCUserData data);
typedef void (*OPCRequestCloseAttachmentFunction)(void *ctx);

struct OPCRequestContextT {
    OPCContext sctx;
    OPCInterface interface;
    HashTable opc_commands;
    
    xmlParserCtxtPtr parser_ctx;
    xmlDocPtr doc;
    xmlNodePtr req;
    xmlDocPtr adoc;
    xmlNodePtr resp;

    const char* locale;
    const char *charset;
    
    unsigned char soap;
    xmlNsPtr soapns;
    
    unsigned long extensions;
    
    xmlChar *client_request_handle;
    
    struct OPCTimeT start_time, end_time;
    
    OPCRequestOpenAttachmentFunction open_attachment;
    OPCSendFunction send_attachment;
    OPCRequestCloseAttachmentFunction close_attachment;
    void *cctx;
};

OPCRequestContext opcRequestCreateContext();
int opcRequestInitContext(OPCRequestContext ctx, OPCContext sctx, HashTable opc_commands);
void opcRequestFreeContext(OPCRequestContext ctx);
void opcRequestDestroyContext(OPCRequestContext ctx);
int opcRequestXMLData(OPCRequestContext ctx, char *data, unsigned long length);
int opcRequestProcess(OPCRequestContext ctx, OPCRequestOpenAttachmentFunction open_attachment, OPCSendFunction send_attachment, OPCRequestCloseAttachmentFunction close_attachment, void *cctx);
int opcRequestSend(OPCRequestContext ctx, xmlOutputWriteCallback write, xmlOutputCloseCallback close, void *cctx);

#endif /* _OPC_REQUEST_H */

