#include <stdio.h>
#include <assert.h>

#include <libxml/tree.h>

#include <opcconfig.h>
#include <opcerr.h>

#include <opctree.h>
#include <opcitem.h>

#include <opcrequesttools.h>

#include <opcrequestbrowse.h>

void opcRequestBrowseZeroContext(OPCRequestBrowseContext ctx) {
    assert(ctx);
    
    ctx->item=NULL;
    ctx->tree=NULL;
    ctx->subitems=NULL;
    ctx->position=NULL;
    ctx->browse_filter = 0;
    ctx->name_filter = NULL;
    ctx->vendor_filter = NULL;
}

OPCRequestBrowseContext opcRequestBrowseCreateContext() {
    OPCRequestBrowseContext ctx;
    ctx = (OPCRequestBrowseContext)malloc(sizeof(struct OPCRequestBrowseContextT));
    if (ctx) opcRequestBrowseZeroContext(ctx);
    return ctx;
}

int opcRequestBrowseInitContext(OPCRequestBrowseContext ctx, OPCRequestContext rctx, xmlNodePtr node) {
    int err = 0;
    OPCInterface interface;
    OPCTreeContext tree;
    xmlChar *attr;
    xmlChar *browse_filter;

    assert(ctx);
    assert(node);
    assert(rctx);
    interface = rctx->interface;
    assert(interface);

    opcRequestBrowseZeroContext(ctx);
    
    ctx->locale = rctx->locale;
    
    tree = opcInterfaceGetServerTree(interface);
    if (!tree) return ERR_OPC_XLIB_SERVER_TREE;
    else ctx->tree = tree;

    ctx->item=opcRequestGetItemName(node);
    if (!ctx->item) return ERR_OPC_LIB_ITEMNAME;

    attr = xmlGetProp(node, "MaxElementsReturned");
    if (attr) {
	if ((sscanf(attr,"%d\n", &ctx->max_elements_returned)!=1)||(ctx->max_elements_returned<0)) {
	    opcRequestBrowseFreeContext(ctx);
	    return ERR_OPC_XLIB_INVALID_RETURNED_ELEMENTS_NUMBER;
	}
	xmlFree(attr);
	if (!ctx->max_elements_returned) ctx->max_elements_returned=OPC_SERVER_CONFIG_MAX_ELEMENTS_RETURNED;
    } else ctx->max_elements_returned = OPC_SERVER_CONFIG_MAX_ELEMENTS_RETURNED;

    browse_filter = xmlGetProp(node, "BrowseFilter");
    if (browse_filter) {
	switch (browse_filter[0]) {
	    case 'a':
		if (!xmlStrcmp(browse_filter,"all")) ctx->browse_filter=ALL;
		else err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
		break;
	    case 'b':
		if (!xmlStrcmp(browse_filter,"branch")) ctx->browse_filter=BRANCH;
		else err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
		break;
	    case 'i':
		if (!xmlStrcmp(browse_filter,"item")) ctx->browse_filter=ITEM;
		else err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
		break;
	    default:
		err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
	}
	xmlFree(browse_filter);
	if (err) {
	    opcRequestBrowseFreeContext(ctx);
	    return err;
	}
    }

    ctx->name_filter = xmlGetProp(node, "ElementNameFilter");
    ctx->vendor_filter = xmlGetProp(node, "VendorFilter");
    
    printf("Browse: \"%s\".\n",ctx->item);
    err = opcTreeBrowse(tree,ctx->item,ctx->browse_filter,ctx->name_filter,ctx->vendor_filter,&ctx->subitems);
    if (!ctx->subitems) {
	opcRequestBrowseFreeContext(ctx);
	return err;
    }
    
    ctx->position = ctx->subitems;
    
    return 0;
}

int opcRequestBrowseCheckContext(OPCRequestBrowseContext ctx, OPCRequestContext rctx, xmlNodePtr node) {
    int err = 0;
    OPCInterface interface;
    xmlChar *attr;
    xmlChar *browse_filter;

    assert(ctx);
    assert(node);
    assert(rctx);
    interface = rctx->interface;
    assert(interface);
    
    ctx->locale = rctx->locale;

    ctx->tree = opcInterfaceGetServerTree(interface);
    if (!ctx->tree) return ERR_OPC_XLIB_SERVER_TREE;
    
    attr=opcRequestGetItemName(node);
    if (!attr) err = ERR_OPC_LIB_ITEMNAME;
    else {
	if (xmlStrcmp(attr,ctx->item)) err = ERR_OPC_XLIB_CONTINUATION_POINT_ITEMNAME;
	xmlFree(attr);
    }
    if (err) return err;

    attr = xmlGetProp(node, "MaxElementsReturned");
    if (attr) {
	if ((sscanf(attr,"%d\n", &ctx->max_elements_returned)!=1)||(ctx->max_elements_returned<0)) {
	    return ERR_OPC_XLIB_INVALID_RETURNED_ELEMENTS_NUMBER;
	}
	xmlFree(attr);
	if (!ctx->max_elements_returned) ctx->max_elements_returned=OPC_SERVER_CONFIG_MAX_ELEMENTS_RETURNED;
    } else ctx->max_elements_returned = OPC_SERVER_CONFIG_MAX_ELEMENTS_RETURNED;
    

    browse_filter = xmlGetProp(node, "BrowseFilter");
    if (browse_filter) {
	switch (browse_filter[0]) {
	    case 'a':
		if (!xmlStrcmp(browse_filter,"all")) {
		    if (ctx->browse_filter!=ALL) err = ERR_OPC_XLIB_CONTINUATION_POINT_BROWSE_FILTER;
		} else err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
		break;
	    case 'b':
		if (!xmlStrcmp(browse_filter,"branch")) {
		    if (ctx->browse_filter!=BRANCH) err = ERR_OPC_XLIB_CONTINUATION_POINT_BROWSE_FILTER;
		} else err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
		break;
	    case 'i':
		if (!xmlStrcmp(browse_filter,"item")) {
		    if (ctx->browse_filter!=ITEM) err = ERR_OPC_XLIB_CONTINUATION_POINT_BROWSE_FILTER;
		} else err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
		break;
	    default:
		err = ERR_OPC_XLIB_INVALID_BROWSE_FILTER;
	}
	xmlFree(browse_filter);
	if (err) return err;
    }

    attr = xmlGetProp(node, "ElementNameFilter");
    if (attr) {
	if (xmlStrcmp(attr, ctx->name_filter)) err = ERR_OPC_XLIB_CONTINUATION_POINT_NAME_FILTER;
	xmlFree(attr);
    } else if (ctx->name_filter) err = ERR_OPC_XLIB_CONTINUATION_POINT_NAME_FILTER;
    
    attr = xmlGetProp(node, "VendorFilter");
    if (attr) {
	if (xmlStrcmp(attr, ctx->vendor_filter)) err = ERR_OPC_XLIB_CONTINUATION_POINT_VENDOR_FILTER;
	xmlFree(attr);
    } else if (ctx->vendor_filter) err = ERR_OPC_XLIB_CONTINUATION_POINT_VENDOR_FILTER;

    if (err) return err;


    return 0;
}



void opcRequestBrowseFreeContext(OPCRequestBrowseContext ctx) {
    assert(ctx);
    if (ctx->item) xmlFree(ctx->item);
    if (ctx->subitems) opcStringListDestroy(ctx->subitems);
    if (ctx->name_filter) xmlFree(ctx->name_filter);
    if (ctx->vendor_filter) xmlFree(ctx->vendor_filter);
    opcRequestBrowseZeroContext(ctx);
}

void opcRequestBrowseDestroyContext(OPCRequestBrowseContext ctx) {
    assert(ctx);
    opcRequestBrowseFreeContext(ctx);
    free(ctx);
}


int opcRequestBrowseNextElement(OPCRequestBrowseContext ctx, xmlNodePtr pnode) {
    OPCTreeContext tree;
    OPCStringList sl;
    OPCItemContext ictx = NULL;
    xmlNodePtr node;
    
    assert(ctx);
    assert(pnode);
    tree = ctx->tree;
    assert(tree);

    for (sl = ctx->position;!ictx;sl=sl->next) {
	while ((sl)&&(!sl->item)) sl = sl->next;
	if (!sl) return ERR_OPC_XLIB_NO_MORE_ITEMS;
	ictx=opcTreeGetItem(tree,sl->item);
    }
    
    node = xmlNewChild(pnode, NULL, "Elements", NULL);
    xmlSetProp(node,"Name", opcItemGetTitle(ictx,ctx->locale));
    xmlSetProp(node,"ItemName", ictx->fullname);
    xmlSetProp(node,"ItemPath", "");

    if (opcItemHaveChilds(ictx)) xmlSetProp(node,"HasChildren", "true");
    else xmlSetProp(node,"HasChildren", "false");
    if (opcItemIsConnected(ictx)) xmlSetProp(node, "IsItem", "true");
    else xmlSetProp(node, "IsItem", "false");
    
    // Properties are here

    ctx->position = sl;
    opcItemReturn(ictx);
    return 0;
}

int opcRequestBrowseAnswer(OPCRequestBrowseContext ctx, xmlNodePtr pnode) {
    OPCStringList sl;
    int err;
    int i;
    
    assert(ctx);
    assert(pnode);

    printf("Items: %i\n",ctx->max_elements_returned);
    for (i=ctx->max_elements_returned;i;i--) {
	err=opcRequestBrowseNextElement(ctx,pnode);
	if (err) {
	    if (err==ERR_OPC_XLIB_NO_MORE_ITEMS) break;
	    else {
		// Add errors about items
	    }
	}
    }

    if (err==ERR_OPC_XLIB_NO_MORE_ITEMS) return 0;
    if (ctx->position) {
	for (sl=ctx->position;sl;sl=sl->next)
	    if (sl->item) {
		ctx->position = sl;
		return ERR_OPC_XLIB_MORE_ITEMS;
	    }
    }
    return 0;
}

