#define _OPC_REQUESTPROCESS_C
#include <stdio.h>
#include <assert.h>

#include <libxml/tree.h>
#include <libxml/encoding.h>

#include <opcerr.h>
#include <opcconfig.h>
#include <opcdebug.h>

#include <dsgnuintl.h>
#include <hashtable.h>

#include <opcxml.h>

#include <opcrequest.h>

#include <opcrequestprocess.h>

#include <opcrequestbrowse.h>


int opcRequestProcessOPC(OPCRequestContext ctx) {
    xmlNodePtr node;
    xmlNsPtr soapns,opcns;
    OPCCommand cmd;
    char *stmp;
    
    assert(ctx);

    ctx->adoc = xmlNewDoc("1.0");
    
    if (ctx->soap) {
	node=xmlNewChild((xmlNodePtr)ctx->adoc,soapns,"Envelope",NULL);
	if (ctx->soapns) soapns=xmlNewNs(node,ctx->soapns->href,ctx->soapns->prefix);
	else soapns=xmlNewNs(node,"http://www.w3.org/2001/12/soap-envelope",NULL);
	xmlSetNs(node,soapns);
	node=xmlNewChild(node,NULL,"Body",NULL);
    } else {
	node = (xmlNodePtr)ctx->adoc;
    }
    
    opcTimeCurrentTime(&ctx->start_time);
    
    cmd=HashTableFind(ctx->opc_commands,ctx->req->name);
    if (!cmd) return ERR_OPC_LIB_UNKNOWNREQUEST;

    stmp=(char*)alloca(strlen(cmd->name)+9);
    sprintf(stmp,"%sResponse",cmd->name);

    ctx->resp=xmlNewChild(node,NULL,stmp,NULL);
    if (ctx->extensions&OPC_EXTENSION_HDR)
	opcns=xmlNewNs(ctx->resp,opchdr_namespace,NULL);
    else 
	opcns=xmlNewNs(ctx->resp,ctx->sctx->config->namespaces.opc_namespace,NULL);
    xmlSetNs(ctx->resp,opcns);


    ctx->interface = opcCreateInterface(ctx->sctx);
    if (!ctx->interface) return ERR_OPC_MEM;

    cmd->func(ctx);
    
    opcDestroyInterface(ctx->interface);

    return 0;
}

const char *opcRequestGetClientRequestHandle(OPCRequestContext ctx, xmlNodePtr node) {
    assert(ctx);
    
    if (ctx->client_request_handle) xmlFree(ctx->client_request_handle);
    if (node) ctx->client_request_handle = xmlGetProp(node,"ClientRequestHandle");
    else ctx->client_request_handle = NULL;
    return ctx->client_request_handle;
}

const char *opcRequestGetLocale(OPCRequestContext ctx, xmlNodePtr node) {
    int err;
    xmlChar *attr;
    const char *charset;
    
    assert(ctx);

    if (node) attr = xmlGetProp(node,"LocaleID");
    else attr = NULL;
    ctx->locale=opcInterfaceSetLocale(ctx->interface,attr);
    charset=opcInterfaceGetCharset(ctx->interface);
    if ((charset)&&(xmlParseCharEncoding(charset)!=XML_CHAR_ENCODING_UTF8)&&(xmlFindCharEncodingHandler(charset))) {
	ctx->charset = charset;
    } else {
	ctx->charset = opc_default_charset;
    }
    if (attr) xmlFree(attr);
    return ctx->locale;
}


int opcRequestSupportedLocales(const char *locale, xmlNodePtr pnode) {
    xmlNewTextChild(pnode,NULL,"SupportedLocaleIDs",locale);
    return 0;
}

int opcRequestSupportedInterfaces(const char *interface, xmlNodePtr pnode) {
/*    const char **sie; */

    xmlNewChild(pnode,NULL,"SupportedInterfaceVersions",interface);

/*
    if (ctx->extensions&OPC_EXTENSION_HDR) {
	for (siv=sctx->version->SupportedInterfaceVersions;siv;siv=siv->next) {
	    sie=((OPCInterface)siv->item)->extensions;
	    if ((!sie)||(!sie[0])) continue;
	    node=xmlNewChild(pnode,NULL,"SupportedInterfaceExtensions",NULL);
	    xmlSetProp(node,"Version", ((OPCInterface)siv->item)->interface);
	    for (i=0;sie[i];i++) xmlNewChild(node,NULL,sie[i],NULL);
	}
    }
*/

    return 0;
}

/*
opcRequestSetDateTimeProp(xmlNodePtr node, OPCTime time) {
    char *str;
    assert(node);
    assert(time);
    
    str=opcTimeGetDateTime(sctx->start_time,NULL);
    
}
*/

int opcRequestReplyBase(OPCRequestContext ctx, xmlNodePtr pnode) {
    char *time;
    
    opcTimeCurrentTime(&ctx->end_time);

    time=opcTimeGetDateTime(&ctx->start_time,NULL);
    if (!time) return ERR_OPC_MEM;
    xmlSetProp(pnode,"RcvTime",time);
    opcTimeGetDateTime(&ctx->end_time,time);
    xmlSetProp(pnode,"ReplyTime",time);
    free(time);

    if ((ctx->client_request_handle)&&(xmlStrlen(ctx->client_request_handle)))
	xmlSetProp(pnode,"ClientRequestHandle",ctx->client_request_handle);

    xmlSetProp(pnode,"RevisedLocaleID",ctx->locale);
    xmlSetProp(pnode,"ServerState", opcInterfaceGetStatus(ctx->interface));

    return 0;
}

int opcRequestGetStatus(OPCRequestContext ctx) {
    OPCInterface interface;
    xmlNodePtr rnode, pnode;
    char *stime;
    unsigned long i;
    
    assert(ctx);
    interface = ctx->interface;
    assert(interface);
    
    opcRequestGetClientRequestHandle(ctx, ctx->req);
    opcRequestGetLocale(ctx,ctx->req);
    rnode=xmlNewChild(ctx->resp,NULL,"GetStatusResult",NULL);    
    pnode=xmlNewChild(ctx->resp,NULL,"Status",NULL);
    xmlSetProp(pnode,"StartTime",opcInterfaceGetStartTime(interface));
    xmlSetProp(pnode,"ProductVersion",opcInterfaceGetVersion(interface));
    
    xmlNewTextChild(pnode,NULL,"StatusInfo",opcInterfaceGetStatusInfo(interface));
    xmlNewTextChild(pnode,NULL,"VendorInfo",opcInterfaceGetVendorInfo(interface));
    opcInterfaceListSupportedLocales(interface,(OPCLocalesListFunctionT)opcRequestSupportedLocales, (void*)pnode);
    opcInterfaceListSupportedInterfaces(interface,(OPCVersionListFunctionT)opcRequestSupportedInterfaces, (void*)pnode);
    return opcRequestReplyBase(ctx, rnode);
}

int opcRequestBrowse(OPCRequestContext ctx) {
    OPCContext sctx;
    OPCConfigurationContext config;
    OPCRequestBrowseContext bctx;
    OPCHolderBucketContext bucket;
    xmlChar *atmp;
    xmlNodePtr rnode, pnode;
    unsigned long pos, secret;
    char *ch;
    char stmp[32];
    int err;

    assert(ctx);
    sctx = ctx->sctx;
    assert(sctx);
    config = sctx->config;
    assert(config);

    opcRequestGetClientRequestHandle(ctx, ctx->req);
    opcRequestGetLocale(ctx,ctx->req);

    rnode=xmlNewChild(ctx->resp,NULL,"BrowseResult",NULL);

    atmp = xmlGetProp(ctx->req, "ContinuationPoint");
    if (atmp) {
	ch = strchr(atmp,':');

	*ch=0;
	pos = atol(ch+1);
	secret = atol(atmp);
	xmlFree(atmp);
	
	bucket=opcHolderGet(sctx->holder, pos, OPC_HOLDER_BUCKET_MAGIC_REQUEST_BROWSE,secret);
	if (!bucket) err = ERR_OPC_XLIB_INVALID_CONTINUATION_POINT;
	else {
	    bctx = (OPCRequestBrowseContext)bucket->data;

	    err=opcRequestBrowseCheckContext(bctx,ctx,ctx->req);
	    if (!err) {
		err=opcRequestBrowseAnswer(bctx,ctx->resp);
		if (err==ERR_OPC_XLIB_MORE_ITEMS) {
		    pos = opcHolderPut(sctx->holder, bucket);
		    if (pos==(unsigned long)-1) err = ERR_OPC_HOLDER_BUSY;
		    else sprintf(stmp,"%lu:%lu",secret,pos);
		} else opcHolderBucketDestroyContext(bucket);
	    } else opcHolderBucketDestroyContext(bucket);
	}
    } else {
	bctx=opcRequestBrowseCreateContext();
	if (!bctx) return ERR_OPC_MEM;
	err=opcRequestBrowseInitContext(bctx,ctx,ctx->req);
	if (!err) {
	    err=opcRequestBrowseAnswer(bctx,ctx->resp);
	    if (err==ERR_OPC_XLIB_MORE_ITEMS) {
		bucket = opcHolderBucketInitContext(sctx->malloc, bctx, config->browse_timeout, config->browse_timeout, (DestroyContextFunction)opcRequestBrowseDestroyContext);		
	        if (!bucket) err = ERR_OPC_MEM;
		else {
		    opcHolderBucketSetNumbers(bucket, OPC_HOLDER_BUCKET_MAGIC_REQUEST_BROWSE);
		    secret = bucket->secret;
		    pos = opcHolderPut(sctx->holder, bucket);
		    if (pos==(unsigned long)-1) err = ERR_OPC_HOLDER_BUSY;
		    else sprintf(stmp,"%lu:%lu",secret,pos);
		}	    
	    } 
	}
    }

    if (err==ERR_OPC_XLIB_MORE_ITEMS) {
	xmlSetProp(ctx->resp,"ContinuationPoint", stmp);
	xmlSetProp(ctx->resp,"MoreElements","true");
    } else {
	xmlSetProp(ctx->resp,"MoreElements","false");
	opcRequestBrowseDestroyContext(bctx);
    }
    if ((err)&&(err!=ERR_OPC_XLIB_MORE_ITEMS)) return err;
    
//    pnode=xmlNewChild(ctx->resp,NULL,"Properties",NULL);
//    pnode=xmlNewChild(ctx->resp,NULL,"Errors",NULL);

    return opcRequestReplyBase(ctx,rnode);
}

int opcRequestGetProperties(OPCRequestContext ctx) {
    return 0;
}

int opcRequestRead(OPCRequestContext ctx) {
    int err;
    OPCInterface interface;
    OPCRequestOptionsStr req_options;
    OPCConfigurationContext config;
    OPCTreeContext tree;
    xmlNodePtr node, rnode, pnode;
    xmlNodePtr options = NULL, itemlist = NULL, item;
    xmlNodePtr aitem, anode;
    OPCContext sctx;
    OPCItemContext ictx;
    OPCRepresentationData data;
    OPCUserData userdata;
    OPCDataInfo info;
    struct OPCDurationT maxage_duration;
    struct OPCTimeT deadline, current_time;
    const char *ItemPathG, *ItemPath, *ItemName;
    unsigned long MaxAgeG, MaxAge;
    char *stime;
    void *attachment;
    unsigned long i;
    xmlAttrPtr attr;

    assert(ctx);
    
    interface = ctx->interface;
    assert(interface);
    
    sctx = ctx->sctx;
    assert(sctx);

    tree = opcInterfaceGetServerTree(interface);
    assert(tree);

    config = opcInterfaceGetConfiguration(interface);
    assert(config);
    
    rnode=xmlNewChild(ctx->resp,NULL,"ReadResult",NULL);    
    
    
    for (node=ctx->req->children; node; node=node->next) {
	if (node->type != XML_ELEMENT_NODE) continue;
	if ((!options)&&(!xmlStrcmp(node->name, "Options"))) options=node;
	else if (!xmlStrcmp(node->name, "ItemList")) { 
	    itemlist=node;
	    break;
	}
    }

    opcRequestGetClientRequestHandle(ctx, options);
    opcRequestGetLocale(ctx, options);


    // ToDo: analyse options
    opcTimeCurrentTime(&deadline);
    opcTimeAddSeconds(&deadline, config->request_timeout);
    if (options) {
	//ReturnErrorText, ReturnDiagnosticInfo, ReturnItemTime, ReturnItemPath, ReturnItemName
	//_RequestDeadline_

	if (!opcXmlGetPropDateTime(options, "RequestDeadline", &deadline)) {
	    // ToDo: Invalid Deadline specified!
	}
    }    
    
    if (itemlist) {

	    /* Seting request global options */
	req_options.flags|=OPC_REQUEST_OPTIONS_FLAG_XMLNODE_BEAR;
	
	pnode = xmlNewChild(rnode,NULL,"RItemList",NULL);
	// ReqType, ItemPath, MaxAge

	ItemPathG = opcXmlGetPropString(itemlist, "ItemPath", NULL);
	MaxAgeG = opcXmlGetPropLong(itemlist, "MaxAge", 0);
	
	xmlNewNs(pnode,"http://www.w3.org/2001/XMLSchema-instance","xsi");
		
	    // Deadline, timeout
	for (node=itemlist->children;((node)&&(opcTimeCmp(&deadline,opcTimeCurrentTime(&current_time))>0));node=node->next) {
	    if (node->type != XML_ELEMENT_NODE) continue;
	        // ItemPath, _ReqType_?, ItemName, ClientItemHandle, _MaxAge_
	    
	    ItemName = opcXmlGetPropString(node, "ItemName", NULL);
	    ItemPath = opcXmlGetPropString(node, "ItemPath", NULL);
	    if (!ItemPath) ItemPath=ItemPathG;
	    MaxAge = opcXmlGetPropLong(node, "MaxAge", -1);
	    if (MaxAge==(unsigned long)-1) MaxAge=MaxAgeG;
//	    if (ItemPath) puts(ItemPath);
//	    if (ItemName) puts(ItemName);
	    ictx=opcInterfaceGetItem(interface, ItemPath, ItemName);

	    if (!ictx) err=ERR_OPC_ITEM_NOTFOUND;
	    else err=opcItemCheckConnection(ictx);

	    if (!err) {
		err=opcItemGetData(ictx, &info, &data);
	    }

	    if (!err) {
		err=opcUserDataGet(data,&req_options,&userdata);
		if (!err) {
	    	    aitem=xmlNewChild(pnode, NULL, "Items", NULL);
		    
//		    printf("%lu\n",userdata->type);
		    if ((userdata->type == OPC_DATA_TYPE_DATABUFFER)||(userdata->type != OPC_DATA_TYPE_XMLNODEPTR)) {
			if ((ctx->send_attachment)||(ctx->open_attachment)) {
			    if (ctx->open_attachment) {
				attachment = ctx->open_attachment(ctx->cctx, ictx->fullname, info, userdata);
				if (!attachment) err = ERR_OPC;
			    }
			    if ((!err)&&(ctx->send_attachment))
			    	err = opcUserDataSend(userdata,ctx->send_attachment,attachment);
			    if ((!err)&&(ctx->close_attachment))
				ctx->close_attachment(attachment);
			    if (!err) {
				anode = xmlNewChild(aitem, NULL, "Value", NULL);
				anode = xmlNewChild(anode, NULL, "Attachment", NULL);
				xmlSetProp(anode, "href", ictx->fullname);
			    }
			} else {
			    /* ToDo: Implement storing of attachments for server post processing */
			    err = ERR_OPC_NOTIMPLEMENTED;
			}
		    } else {
			if ((OPCRepresentationData)userdata == data) {
			    anode=xmlDocCopyNode((xmlNodePtr)userdata->data, ctx->adoc, 1);
			    if (anode) xmlAddChild(aitem, anode);
			    else err = ERR_OPC; /* ToDo: error */
			} else {
			    xmlAddChild(aitem, (xmlNodePtr)userdata->data);
			}
		    }
		    opcUserDataDestroy(userdata);
		}
		opcItemReturnData(ictx, info);
	    }
/*	    
	    if (!err) {
		puts("Checking Age");
		opcDurationImportMS(&maxage_duration, MaxAge);
		err=opcItemCheckAge(ictx, &maxage_duration);
		if (err==ERR_OPC_ITEM_VALUE_OUTOFDATE_ACQUIRING) {
		    puts("reaquire");
		    // ToDo: wait
		    // err=0
		}
		printf("Age: %lu\n",err);
	    }
*/
/*	    
	    if (!err) err=opcItemGetOPCXMLData(ictx, &info, &data);
	    
	    if (!err) {
	        aitem=xmlNewChild(pnode, NULL, "Items", NULL);
		// ItemPath, ItemName, ClientItemHandle, Timestamp, ResultID
		// ValueTypeQualifier
		// [ DiagnosticInfo, Value, Quality ]
	        if ((data)&&(data->ptr)) {
		    puts("+++++ GotInformation ++++++++++");
		// Localize strings

		    anode=xmlDocCopyNode((xmlNodePtr)data->ptr, ctx->adoc, 1);
//		    anode=xmlCopyNode((xmlNodePtr)data->ptr,1);
//    attr=xmlHasProp((xmlNodePtr)data->ptr,"type");
//    printf("ANODE: NS: %p %s\n",attr->ns, attr->ns->href);
//    attr=xmlHasProp(anode,"type");
//    printf("ANODE: NS: %p\n",attr->ns);
//    xmlReconciliateNs(ctx->answer, anode);
//    attr=xmlHasProp(anode,"type");
//    printf("ANODE: NS: %p\n",attr->ns);
		    
		    if (node) xmlAddChild(aitem, anode);
		    else {
			// ToDO: error
			opcItemReturn(ictx);
			continue;
		    }
    attr=xmlHasProp(anode,"type");
    printf("ANODE: NS: %p\n",attr->ns);
//
//    xmlStaticCopyNode((xmlNodePtr)data->ptr, ctx->adoc, aitem, 1);

		    anode=xmlDocCopyNode((xmlNodePtr)data->ptr, ctx->adoc, 1);
		    xmlAddChild(aitem, anode);
    attr=xmlHasProp(anode,"type");
    printf("ANODE: NS: %p\n",attr->ns);

		}
		puts(ictx->fullname);
	    }
*/	    
	    // ToDo: Error message
	    if (err) {
		if (ItemPath&&ItemName)
		    opcDebugRequest(OPC_DEBUG_ERROR, "Error processing item \"%s/%s\": %s", ItemPath, ItemName, opcError(err));
		else if (ItemPath||ItemName)		    
		    opcDebugRequest(OPC_DEBUG_ERROR, "Error processing item \"%s\": %s", ItemName?ItemName:ItemPath, opcError(err));
		else 
		    opcDebugRequest(OPC_DEBUG_ERROR, "Error processing item \"/\": %s", opcError(err));
	    }
	    
	    if (ictx) opcItemReturn(ictx);
	}
	
//	xmlReconciliateNs(ctx->answer, aitem);
    }


    return opcRequestReplyBase(ctx, rnode);
}

int opcRequestSubscribe(OPCRequestContext ctx) {
    return 0;
}

int opcRequestSubscribePolledRefresh(OPCRequestContext ctx) {
    return 0;
}

int opcRequestSubscriptionCancel(OPCRequestContext ctx) {
    return 0;
}

int opcRequestWrite(OPCRequestContext ctx) {
    return 0;
}
