#ifndef _ILOCK_H
#define _ILOCK_H

#include <pthread.h>
#include <opcmalloc.h>

/*
OPCItem Locking facility. Should be based on multiple read one write scheme.

Linux Implementation Notes (glibc 2.3.2):
1. EDEADLK isn't returned when trying occure lock from thread alredy have it
2. Lock locked from one thread can be unlocked from another one

This implementation provides partly support for EDEADLK returning. Partly means 
what in cases we have alredy occured rdlock and now trying get wrlock, - no 
error will be returned and deadlock will occure.
*/


struct ILockContextT {
    OPCMallocContext malloc;
#ifdef OPC_SUPPORT_RWLOCK
    pthread_rwlock_t lock_mutex;
    pthread_rwlock_t pid_mutex;
#else
    pthread_mutex_t lock_mutex;
    pthread_mutex_t pid_mutex;
#endif /* OPC_SUPPORT_RWLOCK */
    pthread_t pid;
    unsigned char initialized;
};

typedef struct ILockContextT *ILockContext;

ILockContext ILockCreateContext(OPCMallocContext mctx);
int ILockInitContext(ILockContext ctx);
void ILockFreeContext(ILockContext ctx);
void ILockDestroyContext(ILockContext ctx);
int ILockRead(ILockContext ctx);
int ILockWrite(ILockContext ctx);
int ILockTryRead(ILockContext ctx);
int ILockTryWrite(ILockContext ctx);
int ILockReturn(ILockContext ctx);

#endif /* _ILOCK_H */
