#include <stdio.h>
#include <errno.h>
#include <assert.h>

#include <opcmalloc.h>
#include <lock.h>

static void LockZeroContext(LockContext ctx) {
    ctx->initialized = 0;
}

LockContext LockCreateContext(OPCMallocContext mctx) {
    LockContext ctx;
    ctx = (LockContext)opcMalloc(mctx, sizeof(struct LockContextT));
    if (ctx) {
	ctx->malloc = mctx;
	LockZeroContext(ctx);
    }
    return ctx;
}

int LockInitContext(LockContext ctx) {
    assert(ctx);
    LockZeroContext(ctx);
#ifdef OPC_SUPPORT_RWLOCK
    pthread_rwlock_init(&ctx->lock_mutex,NULL);
#else
    pthread_mutex_init(&ctx->lock_mutex,NULL);
#endif /* OPC_SUPPORT_RWLOCK */
    ctx->initialized = 1;
    return 0;
}

void LockFreeContext(LockContext ctx) {
    assert(ctx);
#ifdef OPC_SUPPORT_RWLOCK
    if (ctx->initialized) pthread_rwlock_destroy(&ctx->lock_mutex);
#else
    if (ctx->initialized) pthread_mutex_destroy(&ctx->lock_mutex);
#endif /* OPC_SUPPORT_RWLOCK */
    LockZeroContext(ctx);
}

void LockDestroyContext(LockContext ctx) {
    assert(ctx);
    LockFreeContext(ctx);
    opcFree(ctx->malloc, ctx);
}

int LockRead(LockContext ctx) {
    assert(ctx);
    assert(ctx->initialized);
#ifdef OPC_SUPPORT_RWLOCK
    return pthread_rwlock_rdlock(&ctx->lock_mutex);
#else
    return pthread_mutex_lock(&ctx->lock_mutex);
#endif /* OPC_SUPPORT_RWLOCK */
}

int LockWrite(LockContext ctx) {
    assert(ctx);
    assert(ctx->initialized);
#ifdef OPC_SUPPORT_RWLOCK
    return pthread_rwlock_wrlock(&ctx->lock_mutex);
#else
    return pthread_mutex_lock(&ctx->lock_mutex);
#endif /* OPC_SUPPORT_RWLOCK */
}

int LockTryRead(LockContext ctx) {
    assert(ctx);
    assert(ctx->initialized);
#ifdef OPC_SUPPORT_RWLOCK
    return pthread_rwlock_tryrdlock(&ctx->lock_mutex);
#else
    return pthread_mutex_trylock(&ctx->lock_mutex);
#endif /* OPC_SUPPORT_RWLOCK */
}

int LockTryWrite(LockContext ctx) {
    assert(ctx);
    assert(ctx->initialized);
#ifdef OPC_SUPPORT_RWLOCK
    return pthread_rwlock_trywrlock(&ctx->lock_mutex);
#else
    return pthread_mutex_trylock(&ctx->lock_mutex);
#endif /* OPC_SUPPORT_RWLOCK */
}

int LockReturn(LockContext ctx) {
    assert(ctx);
    assert(ctx->initialized);
#ifdef OPC_SUPPORT_RWLOCK
    return pthread_rwlock_unlock(&ctx->lock_mutex);
#else
    return pthread_mutex_unlock(&ctx->lock_mutex);
#endif /* OPC_SUPPORT_RWLOCK */
}
