#ifndef _LOCK_H
#define _LOCK_H

#include <pthread.h>
#include <opcmalloc.h>

/*
OPCItem Locking facility. Should be based on multiple read one write scheme.

Linux Implementation Notes (glibc 2.3.2):
1. EDEADLK isn't returned when trying occure lock from thread alredy have it
2. Lock locked from one thread can be unlocked from another one


ToDo: I need "strong writer synchronization". Check what I'll have here... 

ASSUMPTIONS
-----------
Relocking from some thread policy ( Self locking deadlocks )

Original Lock 		New Lock		Required	Lock(RW)	ILock(RW)
Read			Read			Yes		No(Yes)		Yes(Yes)
Write			Read			No		No(No)		Yes(Yes)
Write			Write			No		No(No)		Yes(Yes)
Read			Write			No		No(No)		No(No)

"Yes" means locking function will correctly lock/unlock second time or return EDEADLK error

*/


struct LockContextT {
    OPCMallocContext malloc;
#ifdef OPC_SUPPORT_RWLOCK
    pthread_rwlock_t lock_mutex;
#else
    pthread_mutex_t lock_mutex;
#endif /* OPC_SUPPORT_RWLOCK */
    unsigned char initialized;
};

typedef struct LockContextT *LockContext;

LockContext LockCreateContext(OPCMallocContext mctx);
int LockInitContext(LockContext ctx);
void LockFreeContext(LockContext ctx);
void LockDestroyContext(LockContext ctx);
int LockRead(LockContext ctx);
int LockWrite(LockContext ctx);
int LockTryRead(LockContext ctx);
int LockTryWrite(LockContext ctx);
int LockReturn(LockContext ctx);

#endif /* _LOCK_H */
