#ifndef _OPC_PLUGIN_H
#define _OPC_PLUGIN_H

#include <time.h>

#include <thread.h>
#include <opcmalloc.h>

#define OPC_PLUGIN_EXTENSION ".so"

#define OPC_PLUGIN_RUN_FLAG 256
#define OPC_PLUGIN_RELOAD_FLAG 512
#define OPC_PLUGIN_BREAK_FLAG 1024
#define OPC_PLUGIN_ERROR_FLAG 2048

typedef struct OPCPluginContextT *OPCPluginContext;

typedef void* (*plugin_create_function)();
typedef int (*plugin_init_function)(void*,OPCPluginContext,void*);
typedef void (*plugin_free_function)(void*);
typedef void (*plugin_destroy_function)(void*);
typedef int (*plugin_step_function)(void*);

typedef struct PluginInfoContextT *PluginInfoContext;
struct PluginInfoContextT {
    char title[32];
    char version[16];
    plugin_create_function create;
    plugin_init_function init;
    plugin_step_function step;
    plugin_free_function free;
    plugin_destroy_function destroy;
};


struct OPCPluginContextT {
    struct ThreadContextT thread;
    char *fullname;
    char *filename;
    time_t modification_time;
    void *plugin_attribute;
    unsigned char mark;
    
    OPCMallocContext malloc;
    OPCPluginContext next;
};

void opcPluginZeroContext(OPCPluginContext ctx);
OPCPluginContext opcPluginCreateContext(OPCMallocContext mctx);
int opcPluginInitContext(OPCPluginContext ctx, const char *path, const char *filename, void *attr);
int opcPluginFreeContext(OPCPluginContext ctx);
int opcPluginDestroyContext(OPCPluginContext ctx);

void opcPluginRun(OPCPluginContext ctx);
void opcPluginStop(OPCPluginContext ctx);
void opcPluginReload(OPCPluginContext ctx);
void opcPluginTerminate(OPCPluginContext ctx);
int opcPluginContinue(OPCPluginContext ctx);


void opcPluginMark(OPCPluginContext ctx);
void opcPluginUnMark(OPCPluginContext ctx);
int opcPluginMarked(OPCPluginContext ctx);

int opcPluginCheckName(const char *name);

#endif /* _OPC_PLUGIN_H */
