#ifndef _THREAD_H
#define _THREAD_H

#include <pthread.h>

#include <opclib.h>

#include <lock.h>
#include <condition.h>
#include <opcmalloc.h>


#define OPC_THREAD_RUNNING_FLAG 1
#define OPC_THREAD_TERMINATE_FLAG 2
#define OPC_THREAD_JOINREQUIRED_FLAG 4

struct ThreadContextT {
    OPCMallocContext malloc;

    dword status;
    ConditionContext cond;
    LockContext lock;
    pthread_t thread;

    unsigned char initialized;
};

typedef struct ThreadContextT *ThreadContext;

typedef int* (*ThreadFunctionT)(void*);


ThreadContext ThreadCreateContext(OPCMallocContext mctx);
int ThreadInitContext(ThreadContext ctx);
int ThreadFreeContext(ThreadContext ctx);
int ThreadDestroyContext(ThreadContext ctx);
void ThreadZeroContext(ThreadContext ctx);

/* Theoretically imideatly after status checked in Plugin Check/Wait functions it can be
changed from another thread. Realy it must not be the case, becuase we have only one 
controller thread, but.... If You need to be aware of such situation it's required to 
reimplement (or better add new) functions presesting write lock after return. And only 
from controller return lock */

void ThreadSetStatus(ThreadContext ctx, dword bits);
void ThreadSetStatusBits(ThreadContext ctx, dword bits);
void ThreadResetStatusBits(ThreadContext ctx, dword bits);
int ThreadCheckStatusBitsOn(ThreadContext ctx, dword bits);
int ThreadCheckStatusBitsOff(ThreadContext ctx, dword bits);
int ThreadCheckStatusBits(ThreadContext ctx, dword onbits, dword offbits);
int ThreadCheckStatusSomeBitsOn(ThreadContext ctx, dword bits);
int ThreadCheckStatusSomeBitsOff(ThreadContext ctx, dword bits);
int ThreadCheckStatusSomeBits(ThreadContext ctx, dword onbits, dword offbits);
void ThreadWaitStatusBitsOn(ThreadContext ctx, dword bits);
void ThreadWaitStatusBitsOff(ThreadContext ctx, dword bits);
void ThreadWaitStatusBits(ThreadContext ctx, dword onbits, dword offbits);
void ThreadWaitStatusSomeBitsOn(ThreadContext ctx, dword bits);
void ThreadWaitStatusSomeBitsOff(ThreadContext ctx, dword bits);
void ThreadWaitStatusSomeBits(ThreadContext ctx, dword onbits, dword offbits);
int ThreadRun(ThreadContext ctx, ThreadFunctionT thread_function, void *attr);
void ThreadTerminate(ThreadContext ctx);
int ThreadWaitTerminate(ThreadContext ctx);
int ThreadTerminateAndWait(ThreadContext ctx);
int ThreadIsRunning(ThreadContext ctx);
int ThreadIsTerminated(ThreadContext ctx);

#endif /* _THREAD_H */
