#include <stdint.h>

#include <opctools.h>

#include <dime.h>

#define ALIGN(pos) \
    if (pos&3) pos = (pos&~3)+4;

static char *dime_xml_type_name = "text/xml";

int dimeSendHeader(const char *id, unsigned long size, DimeType type, const char *type_name, unsigned char first, unsigned char last, unsigned char chunked, OPCSendFunction func, void *cctx) {
    int err;
    char *buffer;
    unsigned long length, wbytes;
    unsigned short typelength;
    unsigned short idlength;
    unsigned char header = 1; /* version */
    
    if (!id) return -1;
    idlength = strlen(id);
    if (!idlength) return -1;
    
    if (type == DIME_TYPE_BINARY) {
	type = DIME_TYPE_NONE;
	typelength = 0;
    } else if (type<DIME_TYPE_NONE) {
	typelength=strlen(type_name);
    } else if (type==DIME_TYPE_NONE) typelength=0;
    else if (type==DIME_TYPE_XML) {
	type = DIME_TYPE_MIME;
	type_name = dime_xml_type_name;
	typelength = 8;
    } else return -1;
    
    length = 12 + idlength + typelength;
    buffer = (char*)malloca(length+6);
    if (!buffer) return -1;
    
    if (first) header|=32;
    if (last) header|=64;
    if (chunked) header|=128;
    
    *(uint8_t*)(buffer) = header;
    *(uint8_t*)(buffer+1) = type;
    *(uint16_t*)(buffer+2) = 0;
    *(uint16_t*)(buffer+4) = htons(idlength);
    *(uint16_t*)(buffer+6) = htons(typelength);
    *(uint32_t*)(buffer+8) = htonl(size);

    memcpy(buffer+12,id,idlength);
    ALIGN(idlength);
    
    if (typelength) {
	memcpy(buffer+12+idlength, type_name, typelength);
	ALIGN(typelength);
    }
    
    length = 12 + idlength + typelength;
    err = func(cctx, buffer, length);
    freea(buffer);

    return err;
}

int dimeSendChunk(unsigned long size, unsigned char last, OPCSendFunction func, void *cctx) {
    char buffer[12];
    unsigned char header = 129; /* version + not_last */
    
    if (last) {
	header&=~128;
	if (last>1) header|=64;
    }
    
    *(uint8_t*)(buffer) = header;
    *(uint8_t*)(buffer+1) = DIME_TYPE_NOTAPPLICABLE;
    *(uint16_t*)(buffer+2) = 0;
    *(uint32_t*)(buffer+4) = 0;
    *(uint32_t*)(buffer+8) = htonl(size);

    return func(cctx, buffer, 12);
}

static char dime_last_chunk[12] = {
    1,0,0,0,
    0,0,0,0,
    0,0,0,0
};

static char dime_last_record[12] = {
    65,0,0,0,
    0,0,0,0,
    0,0,0,0
};

int dimeSendCloseChunk(OPCSendFunction func, void *cctx) {
    return func(cctx, &dime_last_chunk, 12);
}

int dimeSendCloseRecord(OPCSendFunction func, void *cctx) {
    return func(cctx, &dime_last_record, 12);
}
