#ifndef _DOUBLEHASH_H
#define _DOUBLEHASH_H

#include <opcmalloc.h>


/* 
 DoubleHashCleanFunction:
    arg1: Pointer to current data
    arg2: Supplied attribute
    return:
	0: Clean isn't required
	1: Clean from DoubleHash table
	2: Clean from DoubleHash and Destroy (if required)
*/
typedef int (*DoubleHashCleanFunctionT)(void *, void *);

typedef struct DoubleHashContextT *DoubleHashContext;

struct DoubleHashContextT {
    OPCMallocContext mctx;
    
    void **buffer;
        
    unsigned long size;		/* should be prime number */
    unsigned long step_size;
    unsigned long max_size;

    unsigned char gift;
    void (*cfunc)(void *data);
};

DoubleHashContext DoubleHashCreateContext(OPCMallocContext mctx);
int DoubleHashInitContext(DoubleHashContext ctx, unsigned long size, unsigned long step_size, unsigned char free_on_clear, void (*cfunc)(void *data));
void DoubleHashFreeContext(DoubleHashContext ctx);
void DoubleHashDestroyContext(DoubleHashContext ctx);

unsigned long DoubleHashInsert(DoubleHashContext ctx, void *data, unsigned long hash);
unsigned long DoubleHashInsertS(DoubleHashContext ctx, void *data, const char *str);
void DoubleHashRemove(DoubleHashContext ctx, unsigned long pos);
void DoubleHashUnlink(DoubleHashContext ctx, unsigned long pos);

const void *DoubleHashGetBuffer(DoubleHashContext ctx);
const void *DoubleHashGetLink(DoubleHashContext ctx, unsigned long pos);
void *DoubleHashGet(DoubleHashContext ctx, unsigned long pos);

int DoubleHashSetMaximumSize(DoubleHashContext ctx, unsigned long max_size);

unsigned long DoubleHashClean(DoubleHashContext ctx, DoubleHashCleanFunctionT clean_func, void *attr);

#endif /* _DOUBLEHASH_H */
