#ifndef _HASHTABLE_H
#define _HASHTABLE_H

#include <opcmalloc.h>

#define hash_table_coef 0.618
#define HASH_TYPE_LAST 5
#define HASH_TYPE_POINTER 0
#define HASH_TYPE_STATICPOINTER 1
#define HASH_TYPE_ARRAY 2
#define HASH_TYPE_FUNCTION 3
#define HASH_TYPE_HASH 4
#define HASH_TYPE_HASH_FUNCTION 5

typedef struct HashStringPtrT *HashStringPtr;
typedef struct HashStringArrayT *HashStringArray;
typedef struct HashStringHashT *HashStringHash;
typedef struct HashTableBucketT *HashTableBucket;
typedef struct HashTableT *HashTable;
typedef int (*HashTableWalkT)(const void *, void*);
typedef int (*HashTableCleanT)(const void *);
typedef const char * (*HashTableStringT)(const void *);
typedef unsigned long (*HashTableHashT)(const void *);
typedef void (*HashTableDestructorT)(void *data);
typedef HashTableDestructorT HashTableDestructorFunction;
 
#define HASHTABLE_DESTRUCTIONMODE_SINGLE 0
#define HASHTABLE_DESTRUCTIONMODE_MULTI 1
#define HASHTABLE_DESTRUCTIONMODE_ALL 2
typedef void (*HashTablePPDFunctionT)(HashTable, unsigned char, void *);

struct HashStringPtrT {
    char *str;
};

struct HashStringArrayT {
    char str[1];
};

struct HashStringHashT {
    unsigned long hash;
};


struct HashTableBucketT {
    unsigned long hash;
    void *data;
    HashTableBucket next;
};

struct HashTableT {
    OPCMallocContext malloc;
    unsigned char gift;
    unsigned char mode;
    unsigned long size;
    void (*cfunc)(void *data);
    HashTableStringT string_func;
    HashTablePPDFunctionT prefunc, postfunc;
    void *ppfuncattr;
    HashTableBucket *bucket;
};

HashTable HashTableInit(OPCMallocContext mctx, unsigned long size, unsigned char hash_type, HashTableStringT string_func);
void HashTableSetDestructor(HashTable ctx, HashTableDestructorT cfunc);
void HashTableSetPPDFunctions(HashTable ctx, HashTablePPDFunctionT prefunc, HashTablePPDFunctionT postfunc, void *attr);
void HashTableFree(HashTable ctx);
void HashTableDestroy(HashTable ctx);
int HashTableInsert(HashTable ctx, void *data);
int HashTableAdd(HashTable ctx, void *data, unsigned char report);
int HashTableDeleteMulti(HashTable ctx, const char *string);
int HashTableDelete(HashTable ctx, const char *string);
int HashTableRemove(HashTable ctx, const void *data);
int HashTableUnlink(HashTable ctx, const void *data);
void *HashTableFindNext(HashTable ctx, const char *string, const void *start);
void *HashTableFind(HashTable ctx, const char *string);
void *HashTableHashFind(HashTable ctx, unsigned long hash);
void *HashTableHashFindNext(HashTable ctx, unsigned long hash, const void *start);
int HashTableWalk(HashTable ctx, HashTableWalkT func, void *attr);
int HashTableClean(HashTable ctx, HashTableCleanT func);
#endif /* _HASHTABLE_H */
