#include <stdio.h>
#include <assert.h>

#include <opcerr.h>

#include <opcbinaryconfiguration.h>

static struct OPCBinaryConfigurationContextT opc_binary_configurationT;
OPCBinaryConfigurationContext opc_binary_configuration = &opc_binary_configurationT;

static unsigned char IEEE_754_4_bigendian[] = 
  {0x3c, 0x00, 0x00, 0x00};
static unsigned char IEEE_754_4_littleendian[] = 
  {0x00, 0x00, 0x00, 0x3c};
static unsigned char IEEE_754_8_bigendian[] = 
  {0x3f, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static unsigned char IEEE_754_8_littleendian[] = 
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x3f};
static unsigned char IEEE_754_8_mixedendian[] = 
  {0x00, 0x00, 0x80, 0x3f, 0x00, 0x00, 0x00, 0x00};
static unsigned char IEEE_754_16_bigendian[] = 
  {0x3f, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static unsigned char IEEE_754_16_littleendian[] = 
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x3f};
static unsigned char IEEE_754_16_mixedendian[] = 
  {0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 0x00, 
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

OPCBinaryConfigurationContext opcBinaryConfigurationCreateContext(OPCMallocContext mctx) {
    OPCBinaryConfigurationContext ctx;

    ctx = (OPCBinaryConfigurationContext)opcMalloc(mctx,sizeof(struct OPCBinaryConfigurationContextT));
    if (ctx) {
	ctx->malloc=mctx;
//	opcBinaryConfigurationZeroContext(ctx);
    }
    return ctx;
}

int opcBinaryConfigurationInitContext(OPCBinaryConfigurationContext ctx) {
    int i;
    uint16 test=0xFF;
    double float_magic=0.0078125;
    char *alloc1, *alloc2;
    enum opc_binary_configuration_test_enumeration {
	OPC_BINARY_CONFIGURATION_TEST_ENUMERATION 
    };
    struct {
	int8 first;
	void *offset;
    } *struct_pointer = NULL;
    struct {
	int8 first;
	int8 offset;
    } *struct_int8 = NULL;
    struct {
	int8 first;
	int16 offset;
    } *struct_int16 = NULL;
    struct {
	int8 first;
	int32 offset;
    } *struct_int32 = NULL;
    struct {
	int8 first;
	int64 offset;
    } *struct_int64 = NULL;
    struct {
	int8 first;
	enum opc_binary_configuration_test_enumeration  offset;
    } *struct_enum = NULL;
    struct {
	int8 first;
	float offset;
    } *struct_float = NULL;
    struct {
	int8 first;
	double offset;
    } *struct_double = NULL;
    struct {
	int8 first;
	long double offset;
    } *struct_long_double = NULL;

    struct {
	int8 first;
	struct {
	    int8 first;
	} offset;
    } *struct_struct_min = NULL;
    struct {
	int8 first;
	struct {
	    int8 first;
	    long double second;
	} offset;
    } *struct_struct_combo = NULL;

    struct {
	int8 first;
	struct {
	    long double first;
	} offset;
    } *struct_struct_max = NULL;
    

    assert(ctx);
    
    ctx->pointer_size = sizeof(void*);
    ctx->float_size = sizeof(float);
    ctx->double_size = sizeof(double);
    ctx->long_double_size = sizeof(long double);
    ctx->enum_size = sizeof(enum opc_binary_configuration_test_enumeration);
    if ((uint8)test==0xFF) ctx->endianess = ENDIANESS_LITTLE_ENDIAN;
    else ctx->endianess = ENDIANESS_BIG_ENDIAN;
    
    ctx->float_endianess = 0xFF;
    switch (ctx->double_size) {
    case 4:
	if (memcmp(&float_magic, &IEEE_754_4_bigendian[0], 4) == 0) {
	    ctx->float_endianess = ENDIANESS_BIG_ENDIAN;
	} else if (memcmp(&float_magic, &IEEE_754_4_littleendian[0], 4) == 0) {
	    ctx->float_endianess = ENDIANESS_LITTLE_ENDIAN;
	}
	break;
    case 8:
	if (memcmp(&float_magic, &IEEE_754_8_bigendian[0], 8) == 0) {
	    ctx->float_endianess = ENDIANESS_BIG_ENDIAN;
	} else if (memcmp(&float_magic, &IEEE_754_8_littleendian[0], 8) == 0) {
	    ctx->float_endianess = ENDIANESS_LITTLE_ENDIAN;
	} else if (memcmp(&float_magic, &IEEE_754_8_mixedendian[0], 8) == 0) {
	    ctx->float_endianess = ENDIANESS_MIXED_ENDIAN;
	}
	break;
    case 16:
	if (memcmp(&float_magic, &IEEE_754_16_bigendian[0], 16) == 0) {
	    ctx->float_endianess = ENDIANESS_BIG_ENDIAN;
	} else if (memcmp(&float_magic, &IEEE_754_16_littleendian[0], 16) ==0){
	    ctx->float_endianess = ENDIANESS_LITTLE_ENDIAN;
	} else if (memcmp(&float_magic, &IEEE_754_16_mixedendian[0], 16) == 0){
	    ctx->float_endianess = ENDIANESS_MIXED_ENDIAN;
	}
	break;
    } 
    if (ctx->float_endianess==0xFF) return ERR_OPC_DETERMINE_FLOAT_FORMAT;
    else ctx->float_format = FLOAT_FORMAT_IEEE;

/*  ToDo: Really I think zero is OK */
    alloc1 = (char*)malloc(1);
    alloc2 = (char*)malloc(1);
    if ((!alloc1)||(!alloc2)) return ERR_OPC_DETERMINE_ALLOCATION_ALIGNMENT;
    ctx->align_allocation = alloc2 - alloc1;
    free(alloc1);
    free(alloc2);

    for (i=0;i<6;i++) 
	if (ctx->align_allocation==(1<<i)) break;
    if (i==6) ctx->align_allocation = 0; //return ERR_OPC_DETERMINE_ALLOCATION_ALIGNMENT;
    
    ctx->align_pointer = (unsigned long)&struct_pointer->offset;
    ctx->align_int8 = (unsigned long)&struct_int8->offset;
    ctx->align_int16 = (unsigned long)&struct_int16->offset;
    ctx->align_int32 = (unsigned long)&struct_int32->offset;
    ctx->align_int64 = (unsigned long)&struct_int64->offset;
    ctx->align_enum = (unsigned long)&struct_enum->offset;
    ctx->align_float = (unsigned long)&struct_float->offset;
    ctx->align_double = (unsigned long)&struct_double->offset;
    ctx->align_long_double = (unsigned long)&struct_long_double->offset;
    ctx->align_struct_min = (unsigned long)&struct_struct_min->offset;
    ctx->align_struct_max = (unsigned long)&struct_struct_max->offset;

    while (ctx->align_allocation%ctx->align_pointer) ctx->align_allocation*=2;
    while (ctx->align_allocation%ctx->align_int64) ctx->align_allocation*=2;
    while (ctx->align_allocation%ctx->align_double) ctx->align_allocation*=2;
    while (ctx->align_allocation%ctx->align_long_double) ctx->align_allocation*=2;
    while (ctx->align_allocation%ctx->align_struct_max) ctx->align_allocation*=2;

    if (ctx->align_struct_min!=ctx->align_struct_max) {
        if (((unsigned long)&struct_struct_max->offset)==ctx->align_struct_min) {
	    ctx->align_struct_algo = ALIGN_STRUCT_FIRST;
        } else if (((unsigned long)&struct_struct_max->offset)==ctx->align_struct_max) {
	    ctx->align_struct_algo = ALIGN_STRUCT_MAX;
	} else {
	    return ERR_OPC_DETERMINE_STRUCT_ALIGN;
	}
    } else ctx->align_struct_algo = ALIGN_STRUCT_FIRST;

    printf("Sizes. Pointer: %u, Float: %u, Double: %u, Long Double: %u, Enumeration %u\n",ctx->pointer_size, ctx->float_size, ctx->double_size, ctx->long_double_size, ctx->enum_size);
    printf("Endianess. Integer: %u, Floats: %u\n",ctx->endianess,ctx->float_endianess);
    printf("Align. Allocation: %u, Pointer: %u\n",ctx->align_allocation, ctx->align_pointer);
    printf("Align. Int8: %u, Int16: %u, Int32: %u, Int64: %u, Enum: %u\n",ctx->align_int8,ctx->align_int16,ctx->align_int32,ctx->align_int64,ctx->align_enum);
    printf("Align. Float: %u, Double: %u, Long Double: %u\n",ctx->align_float,ctx->align_double,ctx->align_long_double);
    printf("Align. Structure Algo: %u, Structure Min: %u, Structure Max: %u\n",ctx->align_struct_algo,ctx->align_struct_min,ctx->align_struct_max);
    return 0;
}



void opcBinaryConfigurationFreeContext(OPCBinaryConfigurationContext ctx) {
    assert(ctx);
}

void opcBinaryConfigurationDestroyContext(OPCBinaryConfigurationContext ctx) {
    opcBinaryConfigurationFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}

int opcBinaryConfigurationInit() {
    return opcBinaryConfigurationInitContext(&opc_binary_configurationT);
}

void opcBinaryConfigurationDestroy() {
    opcBinaryConfigurationFreeContext(&opc_binary_configurationT);
}

/*
main() {
    OPCBinaryConfigurationContext ctx;
    
    ctx = opcBinaryConfigurationCreateContext(NULL);
    opcBinaryConfigurationInitContext(ctx);
}
*/
