#define _OPC_ERROR_C

#include <dsgnuintl.h>

#include <opcerr.h>
#include <hashtable.h>
#include <opcmalloc.h>
#include <opcdebug.h>

#include <opcerror.h>


static HashTable error_messages = NULL;

int opcErrorInit(OPCMallocContext mctx) {
    int i;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcErrorInit");    
    
    error_messages = HashTableInit(mctx,opc_errors_number,HASH_TYPE_HASH,NULL);
    if (!error_messages) return ERR_OPC_MEM;
    
    for (i=0;i<opc_errors_number;i++) 
	if (HashTableInsert(error_messages,(void*)(opc_errors+i))) {
	    opcErrorDestroy();
	    return ERR_OPC_ERROR_INIT;
	}

    return 0;
}

void opcErrorDestroy() {
    opcDebugServer(OPC_DEBUG_NOTICE, "opcErrorDestroy");    

    if (error_messages) {
	HashTableDestroy(error_messages);
	error_messages = NULL;
    }
}

const char *opcError(int err, const char *locale) {
    OPCError emsg;
    
    if (!err) {
	if (locale) return dsgettext(opc_no_error,locale);
	else return opc_no_error;
    }

    if (error_messages) {
	emsg = (OPCError)HashTableHashFind(error_messages, err);
	if (emsg) {
	    if (locale) return dsgettext(emsg->str,locale);
	    else return emsg->str;
	}
    } 

    if (locale) return dsgettext(opc_unknown_error,locale);
    return opc_unknown_error;
}
