#ifndef _OPC_ERROR_H
#define _OPC_ERROR_H

#include <opcconfig.h>

typedef struct OPCErrorT *OPCError;

struct OPCErrorT {
    unsigned long err;
    const char *str;
};

int opcErrorInit();
void opcErrorDestroy();
const char *opcError(int err, const char *locale);

#ifdef _OPC_ERROR_C
static char *opc_no_error="Ok";
static char *opc_unknown_error="Unknown Error";

static const struct OPCErrorT opc_errors[] = {
	{0x30000, lgs("WRN_OPC_CLI_RECONNECT")},
	{0x30001, lgs("WRN_OPC_CLI_CONNECTED")},
	{0x100000, lgs("ERR_OPC_MEM")},
	{0x100001, lgs("ERR_OPC_NOTFOUND")},
	{0x100002, lgs("ERR_OPC_STACK")},
	{0x100003, lgs("ERR_OPC_PATH")},
	{0x100004, lgs("ERR_OPC_NOTIMPLEMENTED")},
	{0x100005, lgs("ERR_OPC_PARAM")},
	{0x100006, lgs("ERR_OPC_OPTIONS")},
	{0x100007, lgs("ERR_OPC_NOTAVAILABLE")},
	{0x100008, lgs("ERR_OPC_NOTSUPPORTED")},
	{0x100009, lgs("ERR_OPC_NOTAPPLICABLE")},
	{0x10000A, lgs("ERR_OPC_BUSY")},
	{0x10000B, lgs("ERR_OPC_LOCK")},
	{0x10000C, lgs("ERR_OPC_NOTINITIALIZED")},
	{0x100010, lgs("ERR_OPC_LIBRARY_INIT")},
	{0x100020, lgs("ERR_OPC_ERROR_INIT")},
	{0x100030, lgs("ERR_OPC_MALLOC_INIT")},
	{0x100040, lgs("ERR_OPC_FRONTEND_INIT")},
	{0x100041, lgs("ERR_OPC_FRONTEND_THREAD_RETURNCODE")},
	{0x100043, lgs("ERR_OPC_FRONTEND_THREAD_JOIN")},
	{0x100044, lgs("ERR_OPC_FRONTEND_THREAD_CANCELED")},
	{0x100045, lgs("ERR_OPC_FRONTEND_THREAD_RUN")},
	{0x100046, lgs("ERR_OPC_FRONTEND_CHILD_THREAD_JOIN")},
	{0x100047, lgs("ERR_OPC_FRONTEND_CHILD_THREAD_CANCELED")},
	{0x100050, lgs("ERR_OPC_BACKEND_INIT")},
	{0x100051, lgs("ERR_OPC_BACKEND_THREAD_RETURNCODE")},
	{0x100052, lgs("ERR_OPC_BACKEND_THREAD_JOIN")},
	{0x100053, lgs("ERR_OPC_BACKEND_THREAD_CANCELED")},
	{0x100054, lgs("ERR_OPC_BACKEND_THREAD_RUN")},
	{0x100060, lgs("ERR_OPC_THREAD_CANCELED")},
	{0x100061, lgs("ERR_OPC_THREAD_RETURNCODE")},
	{0x100070, lgs("ERR_OPC_BUFFPOOL_INIT")},
	{0x100071, lgs("ERR_OPC_BUFFPOOL_GET")},
	{0x100080, lgs("ERR_OPC_REQUESTPOOL_INIT")},
	{0x100081, lgs("ERR_OPC_REQUEST_INIT")},
	{0x100090, lgs("ERR_OPC_GETTEXT_INIT")},
	{0x1000A0, lgs("ERR_OPC_PROPERTIES_INIT")},
	{0x1000A1, lgs("ERR_OPC_PROPERTIES_INSTANCE_INIT")},
	{0x1000A2, lgs("ERR_OPC_PROPERTIES_NO_PROPERTY")},
	{0x1000A3, lgs("ERR_OPC_PROPERTIES_UNSERIALIZE")},
	{0x1000B0, lgs("ERR_OPC_PLUGIN_LOAD")},
	{0x1000B1, lgs("ERR_OPC_PLUGIN_INFOFUNCTION")},
	{0x1000B2, lgs("ERR_OPC_PLUGIN_INVALIDINFO")},
	{0x1000B3, lgs("ERR_OPC_PLUGIN_INIT")},
	{0x1000B4, lgs("ERR_OPC_PLUGIN_PATH")},
	{0x1000C0, lgs("ERR_OPC_SIGNALS_INIT")},
	{0x1000C1, lgs("ERR_OPC_SIGNAL_SET_INIT")},
	{0x1000C2, lgs("ERR_OPC_SIGNAL_SET_ADD")},
	{0x1000C3, lgs("ERR_OPC_SIGNAL_MASK_INIT")},
	{0x1000C4, lgs("ERR_OPC_SIGNAL_MASK_SET")},
	{0x1000C5, lgs("ERR_OPC_SIGNAL_ACTION_SET")},
	{0x1000C6, lgs("ERR_OPC_SIGNAL_NOT_SUPPORTED")},
	{0x1000C7, lgs("ERR_OPC_SIGNAL_INIT")},
	{0x1000C8, lgs("ERR_OPC_SIGNAL_EVENT")},
	{0x1000D0, lgs("ERR_OPC_TREE_INIT")},
	{0x1000E0, lgs("ERR_OPC_TRANSLIBRARY_INIT")},
	{0x1000F0, lgs("ERR_OPC_SECLIBRARY_INIT")},
	{0x1100001, lgs("ERR_OPC_FE_COMMANDS_INIT")},
	{0x1100002, lgs("ERR_OPC_FE_SCHEMA_PARSE")},
	{0x1110000, lgs("ERR_OPC_FE_MEM")},
	{0x1110001, lgs("ERR_OPC_FE_OVERLOADED")},
	{0x1110002, lgs("ERR_OPC_FE_CHILDSPAWN")},
	{0x1110010, lgs("ERR_OPC_FE_SOCKET")},
	{0x1110011, lgs("ERR_OPC_FE_BIND")},
	{0x1110012, lgs("ERR_OPC_FE_LISTEN")},
	{0x1110013, lgs("ERR_OPC_FE_PERMS")},
	{0x1110014, lgs("ERR_OPC_FE_ACCEPT")},
	{0x1110015, lgs("ERR_OPC_FE_SEND")},
	{0x1110016, lgs("ERR_OPC_FE_RECV")},
	{0x1110017, lgs("ERR_OPC_FE_SENDERR")},
	{0x1110018, lgs("ERR_OPC_FE_SELECT")},
	{0x1110019, lgs("ERR_OPC_FE_FCNTL_NONBLOCK")},
	{0x1110020, lgs("ERR_OPC_FE_NETWORK_SOCKET")},
	{0x1110011, lgs("ERR_OPC_FE_NETWORK_BIND")},
	{0x1110012, lgs("ERR_OPC_FE_NETWORK_LISTEN")},
	{0x1110020, lgs("ERR_OPC_FE_CLIREQUEST")},
	{0x2000000, lgs("ERR_OPC_I_INVALIDSTATE")},
	{0x2000001, lgs("ERR_OPC_I_NOINTERFACES")},
	{0x2110000, lgs("ERR_OPC_BE_MEM")},
	{0x2110001, lgs("ERR_OPC_BE_OVERLOADED")},
	{0x2110002, lgs("ERR_OPC_BE_CHILDSPAWN")},
	{0x2110010, lgs("ERR_OPC_BE_SOCKET")},
	{0x2110011, lgs("ERR_OPC_BE_BIND")},
	{0x2110012, lgs("ERR_OPC_BE_LISTEN")},
	{0x2110013, lgs("ERR_OPC_BE_PERMS")},
	{0x2110014, lgs("ERR_OPC_BE_ACCEPT")},
	{0x2110015, lgs("ERR_OPC_BE_SEND")},
	{0x2110016, lgs("ERR_OPC_BE_RECV")},
	{0x2110017, lgs("ERR_OPC_BE_SENDERR")},
	{0x2110020, lgs("ERR_OPC_BE_SOCKET_NAME_TOO_LONG")},
	{0x3000000, lgs("ERR_OPC_LIB_UNKNOWNREQUEST")},
	{0x3000001, lgs("ERR_OPC_LIB_STARTTIME")},
	{0x3000002, lgs("ERR_OPC_LIB_VERSION")},
	{0x3000003, lgs("ERR_OPC_LIB_ITEMNAME")},
	{0x3100000, lgs("ERR_OPC_ITEM_INIT")},
	{0x3100001, lgs("ERR_OPC_ITEM_ALREDY_REGISTERED")},
	{0x3100002, lgs("ERR_OPC_ITEM_ALREDY_DESTROYED")},
	{0x3100003, lgs("ERR_OPC_ITEM_CREATE_PATH")},
	{0x3100004, lgs("ERR_OPC_ITEM_HAVE_MANUAL_CHILDS")},
	{0x3100005, lgs("ERR_OPC_ITEM_NOT_LINKED_TO_TREE")},
	{0x3100006, lgs("ERR_OPC_ITEM_FILTERED")},
	{0x3100007, lgs("ERR_OPC_ITEM_NOTFOUND")},
	{0x3100008, lgs("ERR_OPC_ITEM_NOTCONNECTED")},
	{0x3100009, lgs("ERR_OPC_ITEM_MISSINGDATA")},
	{0x310000A, lgs("ERR_OPC_ITEM_CREATE_DATAMANAGER")},
	{0x310000B, lgs("ERR_OPC_ITEM_PARAMS")},
	{0x310000C, lgs("ERR_OPC_ITEM_SERVER_NOTFOUND")},
	{0x310000D, lgs("ERR_OPC_ITEM_SERVER_INCOMPLETE")},
	{0x3100100, lgs("ERR_OPC_ITEM_DATA_CONVERSION")},
	{0x3100101, lgs("ERR_OPC_ITEM_DATA_CONVERSION_FLOAT")},
	{0x3100105, lgs("ERR_OPC_DETERMINE_FLOAT_FORMAT")},
	{0x3100106, lgs("ERR_OPC_DETERMINE_STRUCT_ALIGN")},
	{0x3100107, lgs("ERR_OPC_DETERMINE_ALLOCATION_ALIGNMENT")},
	{0x3100110, lgs("ERR_OPC_FORMAT_DESCRIPTION")},
	{0x3100111, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY")},
	{0x3100112, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_SCHEMA")},
	{0x3100120, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY")},
	{0x310012E, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_CREATE")},
	{0x3100121, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_PARSE")},
	{0x3100122, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_VALIDATE")},
	{0x3100123, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_XPATH_CONTEXT")},
	{0x3100124, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_XPATH_REGISTER_NAMESPACE")},
	{0x3100125, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_FIND_XSI_NAMESPACE")},
	{0x3100126, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_INVALID_DICTIONARY")},
	{0x3100130, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_ENDIANESS")},
	{0x3100131, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_DICTIONARY_UNSUPPORTED_FLOATING_FORMAT")},
	{0x3100127, lgs("ERR_OPC_FORMAT_ALREDY_REGISTERED")},
	{0x3100128, lgs("ERR_OPC_FORMAT_INCOMPLETE_FORMAT_STRUCTURE")},
	{0x3100129, lgs("ERR_OPC_FORMAT_INIT_NATIVE")},
	{0x310012A, lgs("ERR_OPC_FORMAT_INIT_OPCXML")},
	{0x310012B, lgs("ERR_OPC_FORMAT_NATIVE_CREATE_INSTANCE")},
	{0x310012C, lgs("ERR_OPC_FORMAT_OPCXML_CREATE_INSTANCE")},
	{0x310012D, lgs("ERR_OPC_FORMAT_MISSING_METHOD_IMPORTDATA")},
	{0x3100130, lgs("ERR_OPC_FORMAT_MISSING_METHOD_READNATIVE")},
	{0x3100131, lgs("ERR_OPC_FORMAT_MISSING_METHOD_WRITENATIVE")},
	{0x3100133, lgs("ERR_OPC_FORMAT_MISSING_METHOD_SEND")},
	{0x3100132, lgs("ERR_OPC_FORMAT_OPCXML_INVALID_OPTIONS")},
	{0x310012E, lgs("ERR_OPC_FORMATS_CREATE_FORMAT_CONTEXT")},
	{0x310012F, lgs("ERR_OPC_FORMATS_HASHTABLE_INSERT")},
	{0x3100140, lgs("ERR_OPC_FORMATS_REGISTER_NATIVE_FORMAT")},
	{0x3100141, lgs("ERR_OPC_FORMATS_REGISTER_OPCXML_FORMAT")},
	{0x3100142, lgs("ERR_OPC_DATA_CREATE_REPRESENTATION")},
	{0x3100143, lgs("ERR_OPC_DATA_CREATE_NATIVE_REPRESENTATION")},
	{0x3100144, lgs("ERR_OPC_DATA_CREATE_OPCXML_REPRESENTATION")},
	{0x3100145, lgs("ERR_OPC_DATA_CREATE_ALLOCATOR_FOR_NATIVE_REPRESENTATION")},
	{0x3100151, lgs("ERR_OPC_DATA_CREATE_ALLOCATOR_FOR_OPCXML_REPRESENTATION")},
	{0x3100147, lgs("ERR_OPC_DATA_DATABUFFER_CREATE")},
	{0x31000146, lgs("ERR_OPC_DATA_NOTINITIALISED")},
	{0x31000150, lgs("ERR_OPC_DATA_INVALID_REPRESENTATION")},
	{0x31000148, lgs("ERR_OPC_DATA_INCOMPLETE_FORMATS_CONTEXT")},
	{0x31000149, lgs("ERR_OPC_DATA_INCOMPLETE_SERVER_CONTEXT")},
	{0x31000152, lgs("ERR_OPC_DATA_REPRESENTATION_INVALID_ALLOCATOR")},
	{0x31000169, lgs("ERR_OPC_DATA_FAST_CREATE_CONTEXT")},
	{0x31000160, lgs("ERR_OPC_DATAFAST_CREATE_ALLOCATOR")},
	{0x31000161, lgs("ERR_OPC_DATAFAST_INCOMPLETE_FORMATS_CONTEXT")},
	{0x31000162, lgs("ERR_OPC_DATAFAST_INCOMPLETE_SERVER_CONTEXT")},
	{0x31000163, lgs("ERR_OPC_DATAFAST_CREATE_LOCK")},
	{0x31000164, lgs("ERR_OPC_DATAFAST_PARAMS")},
	{0x31000165, lgs("ERR_OPC_DATAFAST_OPTIONS")},
	{0x31000166, lgs("ERR_OPC_DATAFAST_ALLOCATOR_BUSY")},
	{0x31000167, lgs("ERR_OPC_DATAFAST_NOTINITIALISED")},
	{0x31000168, lgs("ERR_OPC_DATAFAST_CREATE_DESTRUCTOR")},
	{0x3100133, lgs("ERR_OPC_DATAREPRESENTATION_CREATE_DATABUFFER")},
	{0x3100132, lgs("ERR_OPC_DATAREPRESENTATION_UNSUPPORTED_FORMAT_TYPE")},
	{0x3100134, lgs("ERR_OPC_DATAREPRESENTATION_UNSUPPORTED_ALLOCATOR_TYPE")},
	{0x3100135, lgs("ERR_OPC_DATAREPRESENTATION_CONVERT")},
	{0x3100136, lgs("ERR_OPC_DATAREPRESENTATION_INCOMPLETE_OPTIONS")},
	{0x3100137, lgs("ERR_OPC_DATAREPRESENTATION_INCOMPLETE_CONTEXT")},
	{0x3100138, lgs("ERR_OPC_DATAREPRESENTATION_DATABUFFER_CREATE")},
	{0x3100139, lgs("ERR_OPC_DATAREPRESENTATION_ALLOCATOR_CREATE")},
	{0x310013A, lgs("ERR_OPC_DATAREPRESENTATION_UNSUPPORTED_ALLOCATOR")},
	{0x31003120, lgs("ERR_OPC_DATA_CREATE_CONTEXT")},
	{0x31003121, lgs("ERR_OPC_DATA_DISABLED_REPRESENTATION")},
	{0x31003118, lgs("ERR_OPC_DATA_MISSING_NATIVE_REPRESENTATION")},
	{0x31003119, lgs("ERR_OPC_DATA_CREATE_NATIVE_REPRESENTATION")},
	{0x3100311A, lgs("ERR_OPC_DATA_CREATE_OPCXML_REPRESENTATION")},
	{0x3100311B, lgs("ERR_OPC_DATA_INCOMPLETE_FORMATS")},
	{0x3100311C, lgs("ERR_OPC_DATA_REGISTER_REPRESENTATION_ALREDY")},
	{0x3100311D, lgs("ERR_OPC_DATA_REGISTER_REPRESENTATION_INCOMPLETE")},
	{0x3100311E, lgs("ERR_OPC_DATA_OPTIONS")},
	{0x3100311F, lgs("ERR_OPC_DATA_PARAMS")},
	{0x31003122, lgs("ERR_OPC_DATA_NOTINITIALIZED")},
	{0x3100300, lgs("ERR_OPC_DATA_BUFFER")},
	{0x3100301, lgs("ERR_OPC_DATA_BUFFER_CREATE")},
	{0x3100310, lgs("ERR_OPC_DATACHUNK")},
	{0x3100311, lgs("ERR_OPC_DATACHUNK_EMPTY")},
	{0x3100312, lgs("ERR_OPC_BUFFERPOOL")},
	{0x3100313, lgs("ERR_OPC_BUFFERPOOL_CREATE")},
	{0x3100314, lgs("ERR_OPC_FORMATS")},
	{0x3100315, lgs("ERR_OPC_FORMATS_CREATE")},
	{0x3100316, lgs("ERR_OPC_PROPERTIES")},
	{0x3100317, lgs("ERR_OPC_PROPERTIES_CREATE")},
	{0x3100320, lgs("ERR_OPC_PROPERTIES_ALREDY_REGISTERED")},
	{0x31000321, lgs("ERR_OPC_PROPERTIES_INVALID_PROPERTY")},
	{0x31000322, lgs("ERR_OPC_PROPERTIES_HASHTABLE_INSERT_FAILED")},
	{0x31000323, lgs("ERR_OPC_PROPERTIES_HASHTABLE_BROKEN")},
	{0x3100324, lgs("ERR_OPC_PROPERTIES_HASHTABLE_INIT")},
	{0x3100325, lgs("ERR_OPC_PROPERTIES_LIMIT_EXCEEDED")},
	{0x3100326, lgs("ERR_OPC_PROPERTIES_ID_ALREDY_USED")},
	{0x3100318, lgs("ERR_OPC_CHUNKPOOL")},
	{0x3100319, lgs("ERR_OPC_CHUNKPOOL_CREATE")},
	{0x3100330, lgs("ERR_OPC_PROPERTY")},
	{0x3100331, lgs("ERR_OPC_PROPERTY_CREATE")},
	{0x3100334, lgs("ERR_OPC_STANDARDTYPE")},
	{0x3100335, lgs("ERR_OPC_STANDARDTYPE_CREATE")},
	{0x3100219, lgs("ERR_OPC_DATABUFFER_OPTIONS")},
	{0x310021A, lgs("ERR_OPC_DATABUFFER_OPTIONS_BUFFERPOOL")},
	{0x3100210, lgs("ERR_OPC_BUFFERPOOL_GET_BUFFER")},
	{0x3100211, lgs("ERR_OPC_DATABUFFER_ALLOCATE_CHUNK")},
	{0x3100212, lgs("ERR_OPC_DATABUFFER_ACCEPT_CHUNK")},
	{0x3100218, lgs("ERR_OPC_ALLOCATOR_INCOMPLETE")},
	{0x3100213, lgs("ERR_OPC_ALLOCATOR_ALLOCATE_CHUNK")},
	{0x3100214, lgs("ERR_OPC_ALLOCATOR_ACCEPT_CHUNK")},
	{0x3100215, lgs("ERR_OPC_CHUNKPOOL_GET_CHUNK")},
	{0x3100216, lgs("ERR_OPC_ITEM_VALUE_OUTOFDATE")},
	{0x3100217, lgs("ERR_OPC_ITEM_VALUE_OUTOFDATE_ACQUIRING")},
	{0x3100301, lgs("ERR_OPC_LOCK_CREATE")},
	{0x3100200, lgs("ERR_OPC_DATA_CHUNK")},
	{0x3100201, lgs("ERR_OPC_DATA_ALLOCATE_NEW_CHUNK")},
	{0x3100133, lgs("ERR_OPC_DICTIONARY")},
	{0x3100134, lgs("ERR_OPC_DICTIONARY_PARSE")},
	{0x3100135, lgs("ERR_OPC_DICTIONARY_VALIDATE")},
	{0x3100135, lgs("ERR_OPC_INVALID_ENUMERATION_DESCRIPTION")},
	{0x3100136, lgs("ERR_OPC_FORMAT_DESCRIPTION_OPCBINARY_FORMAT_NOT_SUPPORTED")},
	{0x3100137, lgs("ERR_OPC_TYPE_UNSUPPORTED")},
	{0x3100138, lgs("ERR_OPC_TYPE_UNSUPPORTED_BY_FORMAT")},
	{0x3100139, lgs("ERR_OPC_TYPE_CONVERSION_FAILED")},
	{0x310013A, lgs("ERR_OPC_TYPE_UNSUPPORTED_CONFIGURATION")},
	{0x310013B, lgs("ERR_OPC_TYPE_INVALID_SIZE")},
	{0x3F10000, lgs("ERR_OPC_HOLDER_BUSY")},
	{0x4000000, lgs("ERR_OPC_XLIB_CREATEPARSER")},
	{0x4000001, lgs("ERR_OPC_XLIB_CHUNKSMALL")},
	{0x4000002, lgs("ERR_OPC_XLIB_CHUNKBIG")},
	{0x4000100, lgs("ERR_OPC_XLIB_PARSEREQUEST")},
	{0x4000101, lgs("ERR_OPC_XLIB_REQUESTEMPTY")},
	{0x4000102, lgs("ERR_OPC_XLIB_NSNOTFOUND")},
	{0x4000200, lgs("ERR_OPC_XLIB_INVALID_BROWSE_FILTER")},
	{0x4000201, lgs("ERR_OPC_XLIB_INVALID_RETURNED_ELEMENTS_NUMBER")},
	{0x4000202, lgs("ERR_OPC_XLIB_INVALID_CONTINUATION_POINT")},
	{0x4000203, lgs("ERR_OPC_XLIB_CONTINUATION_POINT_BROWSE_FILTER")},
	{0x4000204, lgs("ERR_OPC_XLIB_CONTINUATION_POINT_NAME_FILTER")},
	{0x4000205, lgs("ERR_OPC_XLIB_CONTINUATION_POINT_VENDOR_FILTER")},
	{0x4000206, lgs("ERR_OPC_XLIB_CONTINUATION_POINT_ITEMNAME")},
	{0x4000207, lgs("ERR_OPC_XLIB_SERIALIZE_INIT")},
	{0x4000208, lgs("ERR_OPC_XLIB_SERIALIZE")},
	{0x4000209, lgs("ERR_OPC_XLIB_SERIALIZE_FINALIZE")},
	{0x4A00300, lgs("ERR_OPC_XLIB_SERVER_TREE")},
	{0x4A00301, lgs("ERR_OPC_XLIB_NO_MORE_ITEMS")},
	{0x4A00302, lgs("ERR_OPC_XLIB_MORE_ITEMS")},
	{0x5000000, lgs("ERR_OPC_CLI_MEM")},
	{0x5000001, lgs("ERR_OPC_CLI_APACHE")},
	{0x5000010, lgs("ERR_OPC_CLI_SRVNOTFOUND")},
	{0x5000011, lgs("ERR_OPC_CLI_SOCKET")},
	{0x5000012, lgs("ERR_OPC_CLI_BIND")},
	{0x5000013, lgs("ERR_OPC_CLI_CONNECT")},
	{0x5000014, lgs("ERR_OPC_CLI_UNDERSTAND")},
	{0x5000015, lgs("ERR_OPC_CLI_BUSY")},
	{0x5000020, lgs("ERR_OPC_CLI_SEND")},
	{0x5000021, lgs("ERR_OPC_CLI_RECV")},
	{0x5000030, lgs("ERR_OPC_CLI_SRVRESPONSE")},
	{0x5000031, lgs("ERR_OPC_CLI_XMLREQUEST")},
	{0x5000032, lgs("ERR_OPC_CLI_SRVINITRC")},
	{0x5000033, lgs("ERR_OPC_CLI_SRVINTERNAL")}
};
#define opc_errors_number 264

#endif /* _OPC_ERROR_C */

#endif /* _OPC_ERROR_H */
