#include <stdio.h>
#include <assert.h>

#include <opcmalloc.h>
#include <opcstringlist.h>

OPCStringList opcStringListCreate(OPCMallocContext mctx) {
    OPCStringList sl;
    sl=(OPCStringList)opcMalloc(mctx,sizeof(struct OPCStringListT));
    if (sl) {
	sl->malloc=mctx;
	sl->item=NULL;
	sl->next=NULL;
	sl->free=NULL;
	
    }
}

void opcStringListDestroy(OPCStringList list) {
    OPCStringList cur;

    assert(list);

    while (list) {
	cur=list;
	if (cur->item) opcFree(cur->malloc, cur->item);
	list=list->next;
	opcFree(cur->malloc,cur);
    }
}

void opcStringListFree(OPCStringList list) {
    OPCStringList cur;

    assert(list);

    if (list->next) {
	opcStringListDestroy(list->next);
	list->next=NULL;
    }
    if (list->item) {
	opcFree(list->malloc, list->item);
	list->item=NULL;
    }
    list->free=list;
}

int opcStringListIsEmpty(OPCStringList list) {
    OPCStringList cur;
    
    assert(list);
    
    for (cur=list;cur;cur=cur->next)
	if (cur->item) return 0;
    return 1;
}

int opcStringListIsIn(OPCStringList list, const void *item, unsigned long size) {
    OPCStringList cur;
    
    assert(list);
    assert(item);
    assert(size>0);
    
    for (cur=list;cur;cur=cur->next) {
	if ((cur->item)&&(cur->size==size)&&(memcmp(cur->item,item,size)==0))
	    return 1;
    }
    return 0;
}

static int opcStringListRemoveSM(OPCStringList list, const void *item, unsigned long size, unsigned char multi) {
    int res=0;
    OPCStringList cur;
    OPCStringList pre=NULL;
    OPCStringList end=NULL;
    
    assert(list);
    assert(item);
    assert(size>0);
    
    list->free=NULL;
    for (cur=list;cur;cur=cur->next) {
	if (cur->item) {
	    if ((cur->size==size)&&(memcmp(cur->item,item,size)==0)) {
		res++;
		if (pre) {
		    if (!end) for (end=cur;end->next;end=end->next);
		    pre->next=cur->next;
		    cur->next=NULL;
		    end->next=cur;
		    end=end->next;
		    if (!list->free) list->free=end;
		} else {
		    opcFree(cur->malloc, cur->item);
		    cur->item=NULL;
		    list->free=cur;
		}
		if (!multi) break;
	    }
	} else {
	    if (!list->free) list->free=cur;
	}
	pre=cur;
    }
    if (!list->free) list->free=pre;
    return res;
}

int opcStringListRemove(OPCStringList list, const void *item, unsigned long size) {
    return opcStringListRemoveSM(list,item,size,0);
}

int opcStringListRemoveMulti(OPCStringList list, const void *item, unsigned long size) {
    return opcStringListRemoveSM(list,item,size,1);
}

int opcStringListAdd(OPCStringList list, const void *item,  unsigned long size) {
    OPCStringList cur;
    
    assert(list);
    assert(item);
    assert(size>0);
    
    if (list->free) cur=list->free;
    else cur=list;
    
    for (;cur;cur=cur->next) {
    	if (!cur->item) {
	    list->free=cur;
	    break;
	}
	else if (!cur->next) {
	    cur->next=(OPCStringList)opcMalloc(cur->malloc,sizeof(struct OPCStringListT));
	    if (!cur->next) return 1;
	    cur=cur->next;
	    cur->next=NULL;
	    list->free=cur;
	    break;
	}
    }

    cur->item=(char*)opcMalloc(cur->malloc,size);
    if (!cur->item) return 1;
    memcpy(cur->item,item,size);
    cur->size=size;
    return 0;
}
