#include <assert.h>

#include <opctreenode.h>


static inline void opcTreeNodeZeroContext(OPCTreeNodeContext ctx) {
    assert(ctx);
    ctx->destructor=NULL;
    ctx->parrent=NULL;
    ctx->next=NULL; ctx->prev=NULL;
}

OPCTreeNodeContext opcTreeNodeCreateContext(OPCMallocContext mctx) {
    OPCTreeNodeContext ctx;

    ctx = (OPCTreeNodeContext)opcMalloc(mctx,sizeof(struct OPCTreeNodeContextT));
    if (ctx) {
	ctx->malloc=mctx;
	opcTreeNodeZeroContext(ctx);
    }
    return ctx;
}

inline int opcTreeNodeInitContext(OPCTreeNodeContext ctx, OPCDestroyContextFunction destructor) {
    assert(ctx);
    opcTreeNodeZeroContext(ctx);    
    return 0;
}

inline void opcTreeNodeFreeContext(OPCTreeNodeContext ctx) {
    assert(ctx);
    opcTreeNodeUnlink(ctx);
    opcTreeNodeZeroContext(ctx);
}

void opcTreeNodeDestroyContext(OPCTreeNodeContext ctx) {
    opcTreeNodeFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}

int opcTreeNodeUnlink(OPCTreeNodeContext ctx) {
    if (ctx->parrent) {
	if (ctx->prev) ctx->prev->next = ctx->next;
	else ctx->parrent->first_child = ctx->next;
	
	if (ctx->next) ctx->next->prev = ctx->prev;
	else ctx->parrent->last_child = ctx->prev;
	
	ctx->parrent=NULL;
	ctx->prev=NULL; ctx->next=NULL;
    }
    return 0;
}


inline int opcTreeNodeInsertAfter(OPCTreeNodeContext ctx, OPCTreeNodeContext sibling) {
    assert(ctx);
    assert(ctx->parrent);
    assert(!sibling->parrent);
    
    sibling->next = ctx->next;
    ctx->next = sibling;
    sibling->prev = ctx;
    
    if (!sibling->next) ctx->parrent->last_child=sibling;
    sibling->parrent = ctx->parrent;

    return 0;
}

inline int opcTreeNodeInsertBefore(OPCTreeNodeContext ctx, OPCTreeNodeContext sibling) {
    assert(ctx);
    assert(ctx->parrent);
    assert(!sibling->parrent);
    
    sibling->prev = ctx->prev;
    ctx->prev = sibling;
    sibling->next = ctx;
    
    if (!sibling->prev) ctx->parrent->first_child=sibling;
    sibling->parrent = ctx->parrent;
    
    return 0;
}

static inline void opcTreeComplexNodeZeroContext(OPCTreeComplexNodeContext ctx) {
    opcTreeNodeZeroContext((OPCTreeNodeContext)ctx);
    ctx->first_child=NULL; ctx->last_child=NULL;
}

OPCTreeComplexNodeContext opcTreeComplexNodeCreateContext(OPCMallocContext mctx) {
    OPCTreeComplexNodeContext ctx;

    ctx = (OPCTreeComplexNodeContext)opcMalloc(mctx,sizeof(struct OPCTreeComplexNodeContextT));
    if (ctx) {
	((OPCTreeNodeContext)ctx)->malloc=mctx;
	opcTreeComplexNodeZeroContext(ctx);
    }
    return ctx;
}

inline int opcTreeComplexNodeInitContext(OPCTreeComplexNodeContext ctx, OPCDestroyContextFunction destructor) {
    assert(ctx);
    opcTreeComplexNodeZeroContext(ctx);
    return 0;
}

inline void opcTreeComplexNodeFreeContext(OPCTreeComplexNodeContext ctx) {
    OPCTreeNodeContext node,next_node;
    assert(ctx);
    
    opcTreeNodeFreeContext((OPCTreeNodeContext)ctx);
    for (node=ctx->first_child;node;node=next_node) {
	node->parrent=NULL;
	next_node = node->next;
	if (node->destructor) node->destructor(node);
	else opcTreeNodeFreeContext(node);
    }
}

void opcTreeComplexNodeDestroyContext(OPCTreeComplexNodeContext ctx) {
    opcTreeComplexNodeFreeContext(ctx);
    opcFree(((OPCTreeNodeContext)ctx)->malloc,ctx);
}


inline int opcTreeComplexNodeAddChild(OPCTreeComplexNodeContext ctx, OPCTreeNodeContext child) {
    assert(ctx);
    assert(child);
    assert(!child->parrent);

    if (ctx->last_child) ctx->last_child->next = child;
    else ctx->first_child = child;
    child->prev = ctx->last_child;
    child->parrent = ctx;
    ctx->last_child = child;
    return 0;
}
