#include <stdio.h>
#include <assert.h>

#include <libxml/tree.h>

#include <opcxml.h>

/* DS: Rewrite me! Stack instead of recursion, FindFirst, FindNext */
xmlNodePtr xmlFindNode(const xmlNodePtr parent, const xmlChar *name, const xmlNsPtr ns) {
    xmlNodePtr cur;
    xmlNodePtr ret;
    
    cur = parent;
    while(cur != NULL) {

/*	
	if (cur->ns) {
	    printf("%s %p(%s)\n",cur->name, cur->ns, cur->ns->prefix);
	    printf("%s\n",cur->ns->href);
	} else printf("%s %p\n",cur->name, cur->ns);
	Hm.. Problem in SearchNS
	

	printf("%s %p(%s) %p(%s)\n",cur->name, cur->ns, cur->ns->prefix, ns, ns->prefix);
	printf("%s\n%s\n",cur->ns->href,ns->href);
	Problem is following:
	<SOAP-ENV:Envelope xmlns:ns1="http://opcfoundation.org/webservices/XMLDA/1.0/">
	    ....
		<GetStatus  xmlns="http://opcfoundation.org/webservices/XMLDA/1.0/">
	
	SearchNS founds ns1 and think what it isn't equal to namespace in GetStatus
*/

        if ((cur->type == XML_ELEMENT_NODE)&&(ns==cur->ns)&&
	   ((name==NULL)||(xmlStrEqual(cur->name, name)))) return(cur);
	if(cur->children != NULL) {
	    ret = xmlFindNode(cur->children, name, ns);
	    if(ret != NULL) return(ret);	    
	}
	cur = cur->next;
    }
    
    if (!ns) return NULL;

    cur = parent;
    while(cur != NULL) {
        if ((cur->type == XML_ELEMENT_NODE)&&((ns==cur->ns)||(!xmlStrcmp(ns->href,cur->ns->href)))&&
	   ((name==NULL)||(xmlStrEqual(cur->name, name)))) return(cur);
	if(cur->children != NULL) {
	    ret = xmlFindNode(cur->children, name, ns);
	    if(ret != NULL) return(ret);	    
	}
	cur = cur->next;
    }

    return(NULL);
}

xmlNodePtr xmlFindNodeByHref(const xmlNodePtr parent, const xmlChar *name, const char *ns) {
    xmlNodePtr cur;
    xmlNodePtr ret;

    cur = parent;
    while(cur != NULL) {
        if ((cur->type == XML_ELEMENT_NODE)&&
	   ((ns==NULL)||(!xmlStrcmp(ns,cur->ns->href)))&&
	   ((name==NULL)||(xmlStrEqual(cur->name, name)))) return(cur);
	if ((cur->children != NULL)) {
	    ret = xmlFindNodeByHref(cur->children, name, ns);
	    if(ret != NULL) return(ret);	    
	}
	cur = cur->next;
    }

    return(NULL);
}

int opcXmlGetPropInt(const xmlNodePtr node, const xmlChar *name, int default_value) {
    xmlChar *str;
    xmlAttrPtr attr;

    assert(node);
    assert(name);
    
    attr=xmlHasProp(node, name);
    if (attr) {
	 str=attr->children->content;
	 if (str) return atoi(str);
    }
    return default_value;
}

void opcXmlSetPropInt(xmlNodePtr node, const xmlChar *name, int value) {
    char str[16];

    assert(node);
    assert(name);
    
    sprintf(str, "%i", value);
    xmlSetProp(node, name, str);
}


long opcXmlGetPropLong(const xmlNodePtr node, const xmlChar *name, long default_value) {
    xmlChar *str;
    xmlAttrPtr attr;

    assert(node);
    assert(name);
    
    attr=xmlHasProp(node, name);
    if (attr) {
	 str=attr->children->content;
	 if (str) return atol(str);
    }
    return default_value;
}

void opcXmlSetPropLong(xmlNodePtr node, const xmlChar *name, int value) {
    char str[16];

    assert(node);
    assert(name);
    
    sprintf(str, "%li", value);
    xmlSetProp(node, name, str);
}

unsigned char opcXmlGetPropBoolean(const xmlNodePtr node, const xmlChar *name, unsigned char default_value) {
    xmlChar *str;
    xmlAttrPtr attr;

    assert(node);
    assert(name);
    
    attr=xmlHasProp(node, name);
    if (attr) {
	str=attr->children->content;
	if (str) {
	    if ((str[0]=='t')||(str[0]=='1')) return 1;
	    else return 0;
	}
    }
    return default_value;
}

void opcXmlSetPropBoolean(xmlNodePtr node, const xmlChar *name, int value) {
    assert(node);
    assert(name);
    
    if (value) xmlSetProp(node, name, "true");
    else xmlSetProp(node, name, "false");
}

const char *opcXmlGetPropString(const xmlNodePtr node, const xmlChar *name, const char *default_value) {
    xmlChar *str;
    xmlAttrPtr attr;
    
    assert(node);
    assert(name);
    
    attr=xmlHasProp(node, name);
    if (attr) {
	str=attr->children->content;
	if (str) return str;
    }
    return default_value;
}

OPCTime opcXmlGetPropDateTime(const xmlNodePtr node, const xmlChar *name, OPCTime time) {
    xmlChar *str;
    xmlAttrPtr attr;

    assert(node);
    assert(name);
    assert(time);

    attr=xmlHasProp(node, name);
    if (attr) {
	str=attr->children->content;
	if (str) return opcTimeSetDateTime(time, str);
    }
    return time;
}

