#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_protocol.h"

#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>

#include <opcerr.h>
#include <opcconfig.h>

#undef opc_multi_server
//#define DEBUG_INFO APLOG_INFO
#define DEBUG_INFO APLOG_NOTICE
#define MIME_BOUNDARY "qazwsxedcrfvtgbyhnujm"
#define ATACHMENT_MIME_TYPE "application/binary"
#define MAIN_CONTENT_ID "SOAP_message"

/* In apache 1.3.x this variables is per thread, so no conflicts will be 
occured */
static int opc_socket = -1;
static char *opc_buffer;
static int opc_error = 0;
static char opc_last_server[max_server_name]="";


static int opc_log(request_rec *r, char *str) {
//    ap_log_error(APLOG_MARK,APLOG_NOTICE,r->server,str);
    ap_log_error(APLOG_MARK,APLOG_INFO,r->server,str);
}

static void opc_close_socket() {
    close(opc_socket);
    opc_socket=-1;
}

static int opc_return_error(request_rec *r, int err, unsigned char close_socket, int aerr) {
	// Closing socket if required
#ifndef opc_multi_server
    if (close_socket)
#endif
	if (opc_socket>0) opc_close_socket();

	// Logging OPC Error code and Errno
    if (err) {
	ap_log_error(APLOG_MARK,APLOG_NOTICE,r->server,"OPC Error: %lX, Errno: %lX(%lu)", err,errno,errno);
    }

	// Returning apropriate error to client
    if (aerr) return aerr;

    switch (err) {
	case 0:
	    return OK;
	case ERR_OPC_CLI_SRVNOTFOUND:
	case ERR_OPC_CLI_CONNECT:
//	    return HTTP_NOT_FOUND;
	    return HTTP_SERVICE_UNAVAILABLE;
	case ERR_OPC_CLI_XMLREQUEST:
	    return HTTP_BAD_REQUEST;
	case ERR_OPC_CLI_BUSY:
	    return HTTP_SERVICE_UNAVAILABLE;	
    }
    
    return HTTP_INTERNAL_SERVER_ERROR;
}

static int opc_check_socket(int s) {
/* Want work, find another way...
   fd_set efd;
   struct timeval etv;

   FD_ZERO(&efd);
   FD_SET(s,&efd);
   etv.tv_sec=0;
   etv.tv_usec=0;
   return (select(s+1,NULL,NULL,&efd,&etv)==0);*/
   return 1;
}

static int opc_init_socket(const char *srv) {
    struct sockaddrT mysock;
    int err=0;
    
	// Checking if alredy initialized
#ifndef opc_multi_server
    if (!strcmp(opc_last_server,srv)) {
	if ((opc_socket>0)&&(opc_check_socket(opc_socket))) return WRN_OPC_CLI_CONNECTED;
	else if (opc_socket>0) {
	    opc_close_socket();
	    err=WRN_OPC_CLI_RECONNECT;
	}
    } else {
	if (opc_socket>0) {
	    opc_close_socket();
	    err=WRN_OPC_CLI_RECONNECT;
	}
    }
#endif
    
    opc_socket=socket(PF_UNIX,SOCK_STREAM,0);
    if (opc_socket<=0) return ERR_OPC_CLI_SOCKET;

	// Optimize string operations
    mysock.sa_family=AF_UNIX;
    strcpy(mysock.sa_data,opc_temp_dir);
    strcat(mysock.sa_data,srv);
    strcat(mysock.sa_data,"/socket");

    if (connect(opc_socket,(struct sockaddr*)&mysock,3+strlen(mysock.sa_data))<0) {
	opc_close_socket();
	return ERR_OPC_CLI_CONNECT;
    }
#ifndef opc_multi_server
    strcpy(opc_last_server,srv);    
#endif
    return err;
}

static void opc_log_invalid_srvresponse(request_rec *r, unsigned long *resp, unsigned long resp_size) {
    if (resp_size<sizeof(unsigned long))
	ap_log_error(APLOG_MARK,APLOG_NOTICE,r->server,"Incorrect server response, length: %lu",resp_size);
    else
	ap_log_error(APLOG_MARK,APLOG_NOTICE,r->server,"Incorrect server response: %lX, length: %lu",*resp,resp_size);
}

static int opc_new_request(request_rec *r, char *req_string, unsigned long req_size, unsigned long req_response) {
    int readed;
    if (send(opc_socket,req_string,req_size,0)<0) return ERR_OPC_CLI_SEND;
    readed=recv(opc_socket,opc_buffer,opc_buffer_size,0);
    if (readed<=0) return ERR_OPC_CLI_RECV;
    if (readed!=sizeof(unsigned long int)) {
	opc_log_invalid_srvresponse(r,(unsigned long*)opc_buffer,readed);
	return ERR_OPC_CLI_SRVRESPONSE;
    }
    if (ntohl(*((unsigned long int*)opc_buffer))!=req_response) {
	switch (*(unsigned long*)opc_buffer) {
	    case OPC_FECMD_REPEAT:
		ap_log_error(APLOG_MARK, APLOG_NOTICE,r->server, "It's seems previous request isn't correctly closed! Repeating initialization request!");
		return opc_new_request(r,req_string,req_size,req_response);
	    case OPC_FECMD_INVALIDREQ:
		ap_log_error(APLOG_MARK, APLOG_NOTICE,r->server,"It's seems server doesn't understand us!");
	        return ERR_OPC_CLI_UNDERSTAND;
	    case OPC_FECMD_BUSY:
		ap_log_error(APLOG_MARK, APLOG_NOTICE,r->server,"It's seems server is busy!");
		return ERR_OPC_CLI_BUSY;
	    break;
	    case OPC_FECMD_OPC_INVALIDREQ:
		ap_log_error(APLOG_MARK, APLOG_NOTICE,r->server,"It's seems server doesn't understand us! Responded what invalid OPC request submited!");
	        return ERR_OPC_CLI_UNDERSTAND;
	    default:
		opc_log_invalid_srvresponse(r,(unsigned long*)opc_buffer,readed);
	        return ERR_OPC_CLI_SRVRESPONSE;
	}
    }
    return 0;
}

static inline int opc_handler_dime(request_rec *r, unsigned long readed, unsigned long pos) { 
    char *buffer;
    ssize_t bytes;
    uint32_t size;
    unsigned char last = 0;

    r->content_type = "multipart/dime";
    ap_send_http_header(r); // send headers 

    while(!last) {
	if (readed<12) {
	    if (readed) {
		ap_log_error(APLOG_MARK, APLOG_NOTICE,r->server,"Moving %lu %lu %u", readed, pos, *(uint8_t*)(opc_buffer+pos));
		memmove(opc_buffer,opc_buffer+pos,readed);
/*
		*((uint32_t*)opc_buffer)=*((uint32_t*)(opc_buffer+pos));
		if (readed>4) {
	    	    *((uint32_t*)(opc_buffer+4))=*((uint32_t*)(opc_buffer+pos+4));
		    if (readed>8) {
			*((uint32_t*)(opc_buffer+8))=*((uint32_t*)(opc_buffer+pos+8));
		    }
		}
*/
	    }
	    pos = 0;	
	    bytes = recv(opc_socket,opc_buffer+readed,opc_buffer_size-readed,0);
	    if (bytes<=0) return opc_return_error(r,ERR_OPC_CLI_SRVINTERNAL,0,0);
	    readed+=bytes;
	    if (readed<12) return opc_return_error(r,ERR_OPC_CLI_SRVINTERNAL,0,0);
	}
	
	buffer = opc_buffer + pos;
	size = 12 + ntohs(*(uint16_t*)(buffer+2));
	if (size&3) size = (size&~3)+4;
	size += ntohs(*(uint16_t*)(buffer+4));
	if (size&3) size = (size&~3)+4;
	size += ntohs(*(uint16_t*)(buffer+6));
	if (size&3) size = (size&~3)+4;
	size += ntohl(*(uint32_t*)(buffer+8));
	if (size&3) size = (size&~3)+4;

	last = (*(uint8_t*)(buffer))&64;
	
	ap_log_error(APLOG_MARK, APLOG_NOTICE,r->server,"New DIME part %lu %u\n",size,(*(uint8_t*)(buffer)));

	while (size>readed) {
	    ap_rwrite(opc_buffer+pos,readed,r);
	    size-=readed;
	    readed = recv(opc_socket, opc_buffer, opc_buffer_size, 0);
	    if (readed<=0) return opc_return_error(r,ERR_OPC_CLI_SRVINTERNAL,0,0);
	    pos = 0;
	}
    
	if (size) {
	    ap_rwrite(opc_buffer+pos,size,r);
	    readed-=size;
	    pos+=size;
	}
    }

    return opc_return_error(r,0,0,0);
}

static inline int opc_handler_attachment(request_rec *r, unsigned long *readedm, unsigned long *posm) {
    uint32_t size;
    uint16_t tsize;
    ssize_t bytes;
    unsigned long readed, pos;
    
    readed = *readedm;
    pos = *posm;
    
    
    if (readed<sizeof(uint32_t)+sizeof(uint16_t)) {
	memmove(opc_buffer, opc_buffer+pos, readed);
/*	if (readed) {
	    *((uint32_t*)opc_buffer)=*((uint32_t*)(opc_buffer+pos));
	    if (readed>sizeof(uint32_t)) 
		*((uint32_t*)(opc_buffer+sizeof(uint32_t)))=*((uint32_t*)(opc_buffer+pos+sizeof(uint32_t)));
	}*/
	bytes = recv(opc_socket,opc_buffer+readed,opc_buffer_size-readed,0);
	if (bytes<sizeof(uint32_t)+sizeof(uint16_t)) return -1;
	readed+=bytes;
	pos = 0;
    }
    
    size = ntohl(*(uint32_t*)(opc_buffer+pos));
    tsize = ntohs(*(uint16_t*)(opc_buffer+pos+sizeof(uint32_t)));
    readed -= sizeof(uint32_t) + sizeof(uint16_t);
    pos += sizeof(uint32_t) + sizeof(uint16_t);
    
    ap_rprintf(r, "--%s\r\n", MIME_BOUNDARY);
    ap_rprintf(r, "Content-Type: %s\r\n", ATACHMENT_MIME_TYPE);
    ap_rprintf(r, "Content-Lengh: %lu\r\n", size);
    ap_rprintf(r, "Content-Transfer-Encoding: 8bit\r\n");
    ap_rprintf(r, "Content-ID: ");

    
    while (tsize>readed) {
	ap_rwrite(opc_buffer+pos,readed,r);
	tsize-=readed;
	readed = recv(opc_socket, opc_buffer, opc_buffer_size, 0);
	if (readed<=0) return -1;
	pos = 0;
    }
    
    if (tsize) {
	ap_rwrite(opc_buffer+pos,tsize,r);
	readed-=tsize;
	pos+=tsize;
    }
    
    ap_rprintf(r, "\r\n\r\n");
	
    while (size>readed) {
	ap_rwrite(opc_buffer+pos,readed,r);
	size-=readed;
	readed = recv(opc_socket, opc_buffer, opc_buffer_size, 0);
	if (readed<=0) return -1;
	pos = 0;
    }
    
    if (size) {
	ap_rwrite(opc_buffer+pos,size,r);
	readed-=size;
	pos+=size;
    }
    
    ap_rprintf(r, "\r\n");
    
    *readedm = readed;
    *posm = pos;

    return 0;
}

/* here's the content handler */
static int opc_handler(request_rec *r) {
    const char *opc_file;			// OPC Server Name
    char *opc_dir;				// Directory on server
    int err,rc;					// Error code
    const char *hostname;			// Client host name
    const char username[16]="";			// Client user name
    char opc_request[opc_reqresp_buffer_size];	// OPC Request
    unsigned long i,j;				// Just counters
    unsigned long readed;			// Bytes readed from socket
    unsigned long len;				// Response size in bytes
    unsigned long pos;				// Current position in buffer
    char attachments = 0;			// Response with attachments
    char dime = 0;				// DIME transfer


	// Check if initialization was successfull
    if (opc_error) return opc_return_error(r,opc_error,0,0);
    
	// Requesting message body, accepting chunks (not tested - DS)
    if ((rc=ap_setup_client_block(r,REQUEST_CHUNKED_DECHUNK/*PASS*/))!=OK) 
	return opc_return_error(r,ERR_OPC_CLI_APACHE,0,rc);
    
	// Getting server name and path
    for (i=0;r->uri[i]!=0;i++)
	if (r->uri[i]=='/') opc_file=r->uri+i+1;

    if (strlen(opc_file)>max_server_name) return opc_return_error(r,ERR_OPC_CLI_SRVNOTFOUND,0,0);
    opc_dir = (char*)ap_palloc(r->pool,i+2+max_server_name);
    if (!opc_dir) return opc_return_error(r,ERR_OPC_CLI_MEM,0,0);
    strncpy(opc_dir,r->uri,(opc_file-r->uri));
    opc_dir[opc_file-r->uri]=0;

	// Getting information about clients
    hostname = ap_get_remote_host(r->connection,r->per_dir_config,REMOTE_NAME);

	// Initialising connection to the server
    ap_hard_timeout("OPC XML-DA Server",r);
    err=opc_init_socket(opc_file);
    ap_reset_timeout(r);

    if (err>ERR_OPC) return opc_return_error(r,err,0,0);
#ifndef opc_multi_server
    else if (err==WRN_OPC_CLI_RECONNECT) ap_log_error(APLOG_MARK,DEBUG_INFO,r->server,"Reconnecting OPC server: %s", opc_file);
    else if (err!=WRN_OPC_CLI_CONNECTED) ap_log_error(APLOG_MARK,DEBUG_INFO,r->server,"Connecting OPC server: %s", opc_file);
#endif
	
	// hostname,username assumed to have less then 255 chars
    *((unsigned long int*)opc_request)=htonl(OPC_FECMD_NEWREQ);
    opc_request[sizeof(unsigned long int)]=strlen(hostname);
    for (i=0,j=sizeof(unsigned long int)+1;hostname[i]!=0;i++,j++) 
	opc_request[j]=hostname[i];
    opc_request[j]=strlen(username);
    for (i=0,j++;username[i]!=0;i++,j++)
	opc_request[j]=username[i];
    

    err=opc_new_request(r,opc_request,j,OPC_FECMD_NEWREQ);
	// Trying to reinitialise socket
    if (err==ERR_OPC_CLI_SEND) {
#ifndef opc_multi_server
	opc_last_server[0]=0;
#endif
	err=opc_init_socket(opc_file);
	if (err>ERR_OPC) return opc_return_error(r,err,0,0);
#ifndef opc_multi_server
	ap_log_error(APLOG_MARK,DEBUG_INFO,r->server,"Reconnecting OPC server: %s", opc_file);
#endif
	err=opc_new_request(r,opc_request,j,OPC_FECMD_NEWREQ);
    }
    ap_kill_timeout(r);
    if (err) return opc_return_error(r,err,1,0);

    
	// Processing Request
    if (!ap_should_client_block(r)) {
	    // Sending to server request termination command
	*((unsigned long int*)opc_buffer)=htonl(OPC_FECMD_CANCEL);
	if (send(opc_socket,opc_buffer,sizeof(unsigned long int),0)<0) return opc_return_error(r,ERR_OPC_CLI_SEND,1,0);

	strcat(opc_dir,"error.html");
	ap_internal_redirect(opc_dir,r);
	return opc_return_error(r,0,0,0);
    } else {
	// Initialising (if required) client socket connection
	ap_hard_timeout("OPC XML-DA Request",r);
	while ((len=ap_get_client_block(r,opc_buffer+sizeof(unsigned long),opc_buffer_size-sizeof(unsigned long)))>0) {
	    ap_reset_timeout(r);
	    *((unsigned long int*)opc_buffer)=htonl(len);
	    if (send(opc_socket,opc_buffer,len+sizeof(unsigned long int),0)<0) return opc_return_error(r,ERR_OPC_CLI_SEND,1,0);
	    ap_reset_timeout(r);
	}
/******** Feel full buffer
	opc_buffer[0]=0;
	while ((len=ap_get_client_block(r,opc_buffer+slen,opc_buffer_size-slen))>0) {
	    opc_buffer[slen+len]=0;
	    slen+=len;
	    ap_reset_timeout(r);
	}*/	

	*((unsigned long int*)opc_buffer)=htonl(OPC_FECMD_XMLEND);
	if (send(opc_socket,opc_buffer,sizeof(unsigned long int),0)<0) return opc_return_error(r,ERR_OPC_CLI_SEND,1,0);

	ap_kill_timeout(r);
    } 

	
	// Reading answer
    pos = 0;
    while (1) {
	if (!pos) {
	    readed=recv(opc_socket,opc_buffer,opc_buffer_size,0);
	    if (readed<sizeof(unsigned long)) return opc_return_error(r,ERR_OPC_CLI_RECV,1,0);
	}
	else if (readed<sizeof(unsigned long)) {
	    if (readed) memmove(opc_buffer, opc_buffer+pos, readed);
	    pos=recv(opc_socket,opc_buffer,opc_buffer_size,0);
	    if (pos<=0) return opc_return_error(r,ERR_OPC_CLI_RECV,1,0);
	    readed+=pos;
	    if (readed<sizeof(unsigned long)) return opc_return_error(r,ERR_OPC_CLI_RECV,1,0);
	    pos = 0;
	}
    
	len=ntohl(*(unsigned long*)(opc_buffer+pos));
	if (len>OPC_FECMD) {
	    switch (len) {
		case OPC_FECMD_DIME_RESPONSE:
		    readed-=sizeof(uint32_t);
		    pos+=sizeof(uint32_t);
		    return opc_handler_dime(r, readed, pos);
		case OPC_FECMD_XMLEND:
		    if (attachments==-2) ap_rprintf(r, "\r\n--%s--", MIME_BOUNDARY);
		    return opc_return_error(r,0,0,0);
		case OPC_FECMD_INTERNAL_ERROR:
		    return opc_return_error(r,ERR_OPC_CLI_SRVINTERNAL,0,0);
		case OPC_FECMD_RCERROR:
		    return opc_return_error(r,ERR_OPC_CLI_SRVINITRC,0,0);
		case OPC_FECMD_OPC_INVALIDREQ:
		    return opc_return_error(r,ERR_OPC_CLI_XMLREQUEST,0,0);
		case OPC_FECMD_ATTACHMENT:
		    if (attachments<=0) {
			if (attachments<0) return opc_return_error(r, ERR_OPC_CLI_SRVINTERNAL,0,0);
			r->content_type = "multipart/related; boundary=" MIME_BOUNDARY "; type=text/xml; start=" MAIN_CONTENT_ID "\r\n";
			ap_send_http_header(r);
			attachments = 1;
		    }
		    readed-=sizeof(uint32_t);
		    pos+=sizeof(uint32_t);
		    err = opc_handler_attachment(r, &readed, &pos);
		    if (err) opc_return_error(r, ERR_OPC_CLI_SRVINTERNAL,0,0);
		    continue;
		default:
		    opc_log_invalid_srvresponse(r,(unsigned long*)opc_buffer,readed);
		    return opc_return_error(r,ERR_OPC_CLI_SRVRESPONSE,0,0);
	    }
	} else if (attachments>=0) {
	    if (attachments) {
		ap_rprintf(r, "--%s\r\n", MIME_BOUNDARY);
		ap_rprintf(r, "Content-Type: %s\r\n", "text/xml");
		ap_rprintf(r, "Content-ID: %s\r\n\r\n", MAIN_CONTENT_ID);
		attachments = -2;
	    } else {
	        r->content_type = "text/xml";
		ap_send_http_header(r); // send headers 
		attachments = -1;
	    }
	}
	readed-=sizeof(unsigned long);
	pos+=sizeof(unsigned long);
	

	if (readed<=len) {
	    if (readed) {
		ap_rwrite(opc_buffer+pos,readed,r);
		len-=readed;
	    }
	    pos = 0;
	    
	    if (len) {
		readed=recv(opc_socket,opc_buffer,opc_buffer_size,0);
		if (readed<=0) return opc_return_error(r,ERR_OPC_CLI_RECV,1,0);
	    
		while (readed<=len) {
		    ap_rwrite(opc_buffer,readed,r);
		    len-=readed;
		    if (len) {
			readed=recv(opc_socket,opc_buffer,opc_buffer_size,0);
			if (readed<=0) return opc_return_error(r,ERR_OPC_CLI_RECV,1,0);
		    }
		}
	    }
	}
	
	if (len) {
	    ap_rwrite(opc_buffer+pos,len,r);
	    pos+=len;
	    readed-=len;
	}
    }
    
    return opc_return_error(r,0,0,0);
}

void opc_child_init (server_rec *sr, pool *sp) {
    opc_buffer=(char*)ap_palloc(sp,opc_buffer_size+sizeof(uint32_t));
    if (!opc_buffer) {
	opc_error=ERR_OPC_CLI_MEM;
	return;
    }
	// Trying preinitialise connection to OPC server
#ifndef opc_multi_server
    if (opc_last_server[0]!=0) opc_init_socket(opc_last_server);
#endif
}

void opc_child_exit (server_rec *sr, pool *sp) {
#ifndef opc_multi_server
    if (opc_socket) {
	close(opc_socket);
	opc_socket=-1;
    }
#endif
}

/* Make the name of the content handler known to Apache */
static handler_rec opc_handlers[] =
{
    {"opc-handler", opc_handler},
    {NULL}
};

/* Tell Apache what phases of the transaction we handle */
module MODULE_VAR_EXPORT opc_module =
{
   STANDARD_MODULE_STUFF,
   NULL,               /* module initializer                 */
   NULL,               /* per-directory config creator       */
   NULL,               /* dir config merger                  */
   NULL,               /* server config creator              */
   NULL,               /* server config merger               */
   NULL,               /* command table                      */
   opc_handlers,     /* [9]  content handlers              */
   NULL,               /* [2]  URI-to-filename translation   */
   NULL,               /* [5]  check/validate user_id        */
   NULL,               /* [6]  check user_id is valid *here* */
   NULL,               /* [4]  check access by host address  */
   NULL,               /* [7]  MIME type checker/setter      */
   NULL,               /* [8]  fixups                        */
   NULL,               /* [10] logger                        */
   NULL,               /* [3]  header parser                 */
   opc_child_init,     /* process initialization             */
   opc_child_exit,     /* process exit/cleanup               */
   NULL                /* [1]  post read_request handling    */
};
