#define _BACKEND_C

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/socket.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#include <opcerr.h>
#include <opcconfig.h>

#include <opcdebug.h>
#include <opcmalloc.h>
#include <opctools.h>

#include <opc.h>
#include <opcpluginset.h>
#include <opcsignal.h>

#include <backend.h>

void opcBackendZeroContext(OPCBackendContext ctx) {
    assert(ctx);
    
    ctx->socket=-1;
    ctx->sctx=NULL;
    ctx->plugins=NULL;
    ctx->sigctx=NULL;
    ctx->socket_name=NULL;
}

OPCBackendContext opcBackendCreateContext(OPCMallocContext mctx) {
    unsigned long i;
    OPCBackendContext ctx;

    opcDebugServer(OPC_DEBUG_NOTICE, "opcBackendCreateContext");    

    ctx = (OPCBackendContext)opcMalloc(mctx, sizeof(struct OPCBackendContextT));
    if (ctx) {
	ctx->malloc=mctx;
	opcBackendZeroContext(ctx);
    }
    return ctx;    
}

void opcBackendFreeContext(OPCBackendContext ctx) {
    unsigned long i;
    assert(ctx);

    opcDebugServer(OPC_DEBUG_NOTICE, "opcBackendFreeContext");    
    if (ctx->plugins) opcPluginSetDestroyContext(ctx->plugins);
    if (ctx->socket>0) close(ctx->socket);
    if (ctx->socket_name) {
	unlink(ctx->socket_name);
	opcExchangeString(ctx->malloc, &ctx->socket_name, NULL);
    }
    
    opcBackendZeroContext(ctx);
}

void opcBackendDestroyContext(OPCBackendContext ctx) {
    assert(ctx);

    opcDebugServer(OPC_DEBUG_NOTICE, "opcBackendDestroyContext");

    opcBackendFreeContext(ctx);
    opcFree(ctx->malloc,ctx);
}

int opcBackendInitContext(OPCBackendContext ctx, OPCContext sctx, OPCSignalContext sigctx) {
    int err;

    assert(ctx);    
    assert(sctx);

    opcDebugServer(OPC_DEBUG_NOTICE, "opcBackendInitContext");    

    opcBackendZeroContext(ctx);

    ctx->sctx=sctx;
    ctx->sigctx=sigctx;

        // Load all server plugins
    ctx->plugins = opcPluginSetCreateContext(ctx->malloc);
    if (!ctx->plugins) return ERR_OPC_BE_MEM;
    err=opcPluginSetInitContext(ctx->plugins,opc_servers_dir,(void*)ctx);
    if (err) return err;
    
    return 0;
}

int opcBackendInitSocket(OPCBackendContext ctx, const char *socket_name) {
    int s;
    struct sockaddrT mysock;

    assert(ctx);
    
	// Creating unix domain listening socket

    if (strlen(socket_name)+1>sizeof(mysock.sa_data)) return ERR_OPC_BE_SOCKET_NAME_TOO_LONG;

    unlink(socket_name);

    s=socket(PF_UNIX,SOCK_STREAM,0);
    if (!s) return ERR_OPC_BE_SOCKET;

    mysock.sa_family=AF_UNIX;

    strcpy(mysock.sa_data,socket_name);

    if (bind(s,(struct sockaddr*)&mysock,3+sizeof(mysock.sa_data))==-1) {
	close(s);
	return ERR_OPC_BE_BIND;
    }

    if (listen(s,1)<0) {
	unlink(socket_name);
	close(s);
	return ERR_OPC_BE_LISTEN;	
    }

    if (chmod(socket_name,00777)<0) {
	unlink(socket_name);
	close(s);
	return ERR_OPC_BE_PERMS;
    }

    ctx->socket=s;
    
    if (opcExchangeString(ctx->malloc, &ctx->socket_name, socket_name)) {
	unlink(socket_name);
	close(s);
	return ERR_OPC_MEM;
    }

	// Creating TCP/IP listening socket (planed)
    return 0;
}


int *opcBackendListen(OPCBackendContext ctx) {
    assert(ctx);
    opcPluginSetStart(ctx->plugins);
    return ThreadReturnInt(NULL,0);
}

