#include <stdio.h>
#include <pthread.h>
#include <assert.h>

#include <bufferpool.h>

BufferPool BufferPoolCreate(int size) {
    BufferPool ctx;
    
    ctx = (BufferPool)malloc(sizeof(struct BufferPoolT));
    if (ctx) {
	ctx->destroy=0;
	ctx->buffersize=size;
	ctx->bpsize=opc_bufferpull_step;
	ctx->nbusy=0;
	ctx->nfree=0;
        pthread_mutex_init(&ctx->mutex,NULL);
	ctx->busy=(char**)malloc(opc_bufferpull_step*sizeof(char*));
	ctx->free=(char**)malloc(opc_bufferpull_step*sizeof(char*));
	if ((!ctx->busy)||(!ctx->free)) {
	    if (ctx->busy) free(ctx->busy);
	    if (ctx->free) free(ctx->free);
	    free(ctx);
	    ctx=NULL;
	}
    }
    return ctx;
}

    // DS: Some improvents and optimizations required
void BufferPoolCleanup(BufferPool ctx) {
    unsigned long i;
    char **tfree;
    char **tbusy;
    
    assert(ctx);
    assert(opc_bufferpull_cleanupratio>1);

    if (ctx->nbusy>2) i=ctx->nbusy;
    else i=2;
    
    if ((ctx->bpsize/i)>opc_bufferpull_cleanupratio) {
	pthread_mutex_lock(&ctx->mutex);
	for (i=ctx->bpsize/2;i<ctx->nfree;i++)
	    if (ctx->free[i])
		free(ctx->free[i]);
	if (ctx->nfree>ctx->bpsize/2) ctx->nfree=ctx->bpsize/2;

	tbusy=(char**)realloc(ctx->busy,(ctx->bpsize/2)*sizeof(char*));
	tfree=(char**)realloc(ctx->free,(ctx->bpsize/2)*sizeof(char*));
	if ((tbusy)||(tfree)) {
	    ctx->bpsize/=2;
	    ctx->busy=tbusy;
	    ctx->free=tfree;
	} else {
	    if (tbusy) ctx->busy=tbusy;
	    if (tfree) ctx->free=tfree;
	}
	pthread_mutex_unlock(&ctx->mutex);
    }
}

void BufferPoolClearFree(BufferPool ctx) {
    unsigned long i;
    
    pthread_mutex_lock(&ctx->mutex);
    for (i=0;i<ctx->nfree;i++)
	if (ctx->free[i]) free(ctx->free[i]);
    ctx->nfree=0;
    pthread_mutex_unlock(&ctx->mutex);
}

void BufferPoolDestroy(BufferPool ctx) {
    BufferPoolClearFree(ctx);
    ctx->destroy=1;
}

    /* Buffer must not be used at this stage */
void BufferPoolDestroyForce(BufferPool ctx) {
    unsigned long i;
    BufferPoolClearFree(ctx);
    free(ctx->free);
    for (i=0;i<ctx->nbusy;i++)
	if (ctx->busy[i]) free(ctx->busy[i]);
    free(ctx->busy);
    pthread_mutex_destroy(&ctx->mutex);
    free(ctx);
}

char *BufferPoolGet(BufferPool ctx) {
    char **tfree;
    char **tbusy;

    assert(ctx);

#ifndef opc_bufferpull_permitgetdestroy
    assert(!ctx->destroy);
#endif /* opc_bufferpull_permitgetdestroy */

    pthread_mutex_lock(&ctx->mutex);
    
    if (ctx->nfree==0) {
	ctx->free[0]=(char*)malloc(ctx->buffersize*sizeof(char));
	if (!ctx->free[0]) return NULL;
	ctx->nfree++;
    }
    
    if (ctx->nbusy==ctx->bpsize) {
	ctx->bpsize+=opc_bufferpull_step;
	tbusy=(char**)realloc(ctx->busy,ctx->bpsize*sizeof(char*));
	tfree=(char**)realloc(ctx->free,ctx->bpsize*sizeof(char*));
	if ((!ctx->busy)||(!ctx->free)) {
	    ctx->bpsize-=opc_bufferpull_step;
	    if (tbusy) ctx->busy=tbusy;
	    if (tfree) ctx->free=tfree;
	    return NULL;
	} else {
	    ctx->busy=tbusy;
	    ctx->free=tfree;
	}
    }
    
    ctx->busy[ctx->nbusy++]=ctx->free[--(ctx->nfree)];
    
    pthread_mutex_unlock(&ctx->mutex);
    
    return ctx->busy[ctx->nbusy-1];
}

int BufferPoolReturn(BufferPool ctx, char *buffer) {
    unsigned long i;
    
    assert(ctx);
    assert(buffer);
    
    pthread_mutex_lock(&ctx->mutex);

    for (i=0;i<ctx->nbusy;i++)
	if (ctx->busy[i]==buffer) break;
    
    if (i==ctx->nbusy) return -1;
    
    if ((ctx->nfree==ctx->bpsize)
#ifdef opc_bufferpull_fbmaxratio
	||((ctx->nbusy)&&((ctx->nfree/ctx->nbusy)>opc_bufferpull_fbmaxratio))
#endif /* opc_bufferpull_fbmaxratio */
	)
	free(ctx->busy[i]);
    else
	ctx->free[ctx->nfree++]=ctx->busy[i];
	

    if (i<(--(ctx->nbusy)))
	ctx->busy[i]=ctx->busy[ctx->nbusy];

    pthread_mutex_unlock(&ctx->mutex);

	// Reducing size    
    BufferPoolCleanup(ctx);
    
    if ((ctx->destroy)&&(ctx->nbusy==0)) BufferPoolDestroyForce(ctx);
    return 0;
}
