#ifndef _FRONTENDCHILD_H
#define _FRONTENDCHILD_H
#include <pthread.h>

typedef struct OPCFrontendChildContextT *OPCFrontendChildContext;

#include <frontend.h>
#include <opcrequestpool.h>
#include <opcrequest.h>

#include <bufferpool.h>

#define opc_frontend_child_waittime 1
#define opc_frontend_child_primary_buffer_size 1024

#define OPC_FRONTEND_CHILD_STATUS_TERMINATE 1
#define OPC_FRONTEND_CHILD_STATUS_READER_TERMINATED 2
#define OPC_FRONTEND_CHILD_STATUS_PARSER_TERMINATED 4
#define OPC_FRONTEND_CHILD_STATUS_ENDEVENT 8
#define OPC_FRONTEND_CHILD_STATUS_READER_RUNING 16
#define OPC_FRONTEND_CHILD_STATUS_PARSER_RUNING 32
#define OPC_FRONTEND_CHILD_STATUS_READY 64

struct OPCFrontendChildBPT {
    char *buffer;
    unsigned long size;
    struct OPCFrontendChildBPT *next;
};

typedef struct OPCFrontendChildBPT *OPCFrontendChildBP;

struct OPCFrontendChildContextT {
    int socket;
    char buffer[opc_frontend_child_primary_buffer_size];
    char *hostname;
    char *username;
    
    OPCFrontendContext fctx;
    BufferPool buffpool;

    OPCFrontendChildBP lbp;
    pthread_mutex_t lbp_mutex;
    pthread_cond_t lbp_cond;
    pthread_mutex_t lbp_cond_mutex;
    
    pthread_t reader_pthread;
    pthread_t parser_pthread;

    unsigned long int status;
    pthread_mutex_t status_mutex;
    pthread_cond_t status_cond;
    pthread_mutex_t status_cond_mutex;
    
    pthread_mutex_t error_mutex;
    
    OPCRequestPoolContext rpctx;
    OPCRequestContext rctx;
    
    int err;
    
    unsigned char format;	/* 0 - no attachments, 1,2 - DIME with attachments */
};

typedef void* (*pthread_routine)(void*);

OPCFrontendChildContext opcFrontendChildCreateContext();
int opcFrontendChildInitContext(OPCFrontendChildContext ctx, OPCFrontendContext fctx, int s);
void opcFrontendChildFreeContext(OPCFrontendChildContext ctx);
void opcFrontendChildDestroyContext(OPCFrontendChildContext ctx);
int* opcFrontendChildReader(OPCFrontendChildContext ctx);
int* opcFrontendChildParser(OPCFrontendChildContext ctx);

int opcFrontendChildIsForClean(OPCFrontendChildContext ctx);
int opcFrontendChildIsTerminate(OPCFrontendChildContext ctx);
int opcFrontendChildIsReady(OPCFrontendChildContext ctx);
#endif /* _FRONTENDCHILD_H */
