#include <stdio.h>
#include <unistd.h>
#include <signal.h>


#include <opcserver.h>
#include <opcdebug.h>
#include <opcerror.h>
#include <opcconfig.h>

static int RunFlag = 1;
static void signal_nop_handler(int signo) { RunFlag = 0; }

main() {
    int err;
    const char *serr;
    int signal = 0;
    struct sigaction act;
    sigset_t set;
    OPCServerContext ctx;

    act.sa_handler = signal_nop_handler;
    act.sa_flags = 0;

    if (sigaction(SIGINT,&act,NULL)==-1) {
	printf("Initialization Error: Installing SIGINT handler\n");
	return 0;
    }
    
    if ((sigemptyset(&set)==-1)||(sigaddset(&set,SIGINT)==-1)||(sigprocmask(SIG_BLOCK,&set,NULL)==-1)) {
	printf("Initialization Error: Blocking SIGINT signal\n");
	return 0;    
    }
    
    err = opcServerInit();
    if (err) {
	printf("Initialization Error: Server Global Initialization Error: %lx\n", err);
	return 0;
    }
    
    ctx=opcServerCreateContext();
    if (!ctx) {
	serr = opcError(err,NULL);
	opcServerDestroy();
	printf("Initialization Error: Cann't allocate memory for OPCServerContext\n");
	printf("%s\n", serr);
	return 0;
    }
    
    err=opcServerInitContext(ctx);
    if (err) {
	serr = opcError(err,NULL);
	opcServerDestroyContext(ctx);
	opcServerDestroy();
	printf("Initalization Error: Server Initialization Error: %lx\n", err);
	printf("%s\n", serr);
	return 0;
    }
    
    err=opcServerRun(ctx);
    if (err) {
	printf("Execution Error: Server Execution Error: %lx\n", err);
	printf("%s\n",opcError(err,NULL));
    }
        
    pthread_sigmask(SIG_UNBLOCK,&set,NULL);
    while ((RunFlag)&&(!err)&&(signal!=SIGINT)) {
//	err=sigwait(&set,&signal);
	sleep(opc_maintain_delay);
	opcServerMantain(ctx);
    }
    pthread_sigmask(SIG_BLOCK,&set,NULL);

    if (err) printf("Execution Error: Waiting for SIGINT signal failed!\n");
    
    serr = opcError(0,NULL);
    
    opcServerDestroyContext(ctx);
    opcServerDestroy();
    
    printf("\nFinished: %s\n\n",serr);
    return 0;
}
